"alignOptim" <-
function(idRef,idOth,dat,tf.mz,method="grid",grid=10,plot.indicator=TRUE){

  ##-idRef is the column number of the reference spectrum
  ##-idOth is the column number of the other spectrum
  ##-dat is the array of the proteomics data
  ##-tf.mz is the matrix indicating which rows are valid

  ##Identify the mz of reference spectrum
  mzRef<-dat[tf.mz[,idRef],idRef,1]
  mzOth<-dat[tf.mz[,idOth],idOth,1]

  ##============================##
  ##Set up the optimization step##
  ##============================##
  
  ##If the method specified is optimize
  if(method=="optimize"){
    interval<-min(length(mzRef),length(mzOth))
    optimValue<-optimize(ObjFunction,interval=c(-interval,interval),
                         mzRef=mzRef,mzOth=mzOth)

    ##===========================##
    ##Find the optimal step value##
    ##===========================##
    if(optimValue$minimum%%1!=0){
      
      optimStep<-c(floor(optimValue$minimum),ceiling(optimValue$minimum))
      objValue<-lapply(optimStep,ObjFunction,mzRef=mzRef,mzOth=mzOth)
      stepMin<-which.min(objValue)
    
      objValue<-objValue[stepMin]
      if(objValue!=optimValue$objective){
        stop("the min value from optimize is different from hand computed")
      }
      step<-unlist(optimStep[[stepMin]])
    }else{
      step<-optimValue$minimum
    }
  }else{
    ##Using grid search method: Compute the objective function of all steps
    step.tmp<--grid:grid
    obj.tmp<-apply(matrix(step.tmp,ncol=1),1,"ObjFunction",mzRef,mzOth)

    ##Find the id of the objective function with the lowest value
    min.id<-which.min(obj.tmp)
    
    step<-step.tmp[min.id]

#    ##To plot the objective function
#    if(plot.indicator==TRUE){
#      plot(step.tmp,obj.tmp,ylab="objective function",xlab="step")
#      abline(v=step.tmp[min.id])
#    }
  }

  ##=============================##
  ##To get the optimial alignment##
  ##=============================##
  valid<-alignSpec(mzRef,mzOth,step)
  
  return(list(step=step,valid=valid))
}

