arrayInfoPA<-function(output){

  ##==Purpose: To extract the mz and intensity of the peaks via parametric approach
  ##
  ##==Input:
  ##    -output: Object from the function findPeaks() or quantifyPeaks().
  ##
  ##==Output:
  ##    -array with rows corresponding to peaks, columns correpsonding to spectra and
  ##     the 3rd dimension corresponding to c("mz","intensity","converge"). There converge
  ##     indicate whether the EM-aglorithm has converged and NA if no applicable.
  ##
  ##==Comments:
  ##  080207:- Made changes to the type of argument output is. It can be a object from findPeaks()
  ##           or from quantifyPeaks(). If faciliates the process of exporting peaks information out.
  ##
  ##  120207:- Made changes to the next if() statement. Should be infoPA and not output$infoPA

  ##Extract the array with the information to compute the peak's intensity and mz
  if(!is.null(output$quantify.dat)){
    infoPA<-output$quantify.dat$infoPA
  }else{
    infoPA<-output$infoPA
  }

  if(is.null(infoPA)){
    stop("You did not specify the parametric approach to quantify the peaks.
          Please specify it and then run the program again")
  }

  ##Extract the necessary parameters
  mu<-infoPA[,,"muEst"]
  sigma<-infoPA[,,"sigmaEst"]
  s<-infoPA[,,"sEst"]
  p<-infoPA[,,"pEst"]
  area.spect<-infoPA[,,"area.spect"]
  min.min.int<-infoPA[,,"min.min.int"]
  
  ##Estimate of the mode-s first (see eq 16 of paper)
  mode<-exp(mu-sigma^2)
  ##Estimate of the intensity
  log.norm<-matrix(dlnorm(c(mode),c(mu),c(sigma)),ncol=ncol(mu))
  intensity<-area.spect*p*log.norm+min.min.int
  ##Estimate of the mode
  mode<-mode+s

  ##Prepare to output the estimated peak's intensity and mz.
  out<-array(NA,dim=c(dim(mode),3),dimnames=list(rownames(infoPA),colnames(infoPA),c("mz","intensity","converge")))
  out[,,1]<-mode
  out[,,2]<-intensity
  out[,,3]<-infoPA[,,"converge"]
  
  return(out)
}
