"ccl.file.merge" <-
function(mz.mean, file.name, pctspectra=10,forcemanual=TRUE,
                   masswindow=0.3, bmwfirst=TRUE, bmwsecond=TRUE,
                   bmwestimated=TRUE, bmwdelete=TRUE, snfirst=5,
                   snsecond=2,merge=FALSE)
######################################################################
#                                                                    #
#====================================================================#
#              Description of function ccl.file()                    #
#====================================================================#
#                                                                    #
# Task                                                               #
#       Output significant windows into .ccl files for Ciphergen     #
#       to read in for further analysis                              #
#                                                                    #
# Usage                                                              #
#       ccl.file(mz.mean,filename="C:/test.ccl")                     #
#                                                                    # 
#                                                                    #
# Arguments                                                          #
#       -mz.mean: a vector of the mean mz of the peak from ARS       #
#       -file.name is the directory and the file name of the .ccl    #
#        file.                                                       #
#       -The following is the Biomaker Wizard option with default    #
#        value                                                       #
#       -pctspectra=10: Min Peak Threshold: 10
#       -forcemanual=TRUE: "Override threshold for user-detected     #
#        peaks" option checked.                                      #
#       -masswindow=0.3: Cluster window: 0.3
#       -bmwfirst=TRUE: Most likely choosing between "Auto-detected  #
#        peaks to cluster" is TRUE while "Cluster only user detected #
#        peaks" is FALSE.                                            #
#       -bmwsecond=TRUE: "Second Pass:" option is checked.           #
#       -bmwestimated=TRUE: "Add estimated peaks to complete         #
#        clusters" option checked.                                   #
#       -bmwdelete=TRUE: "Delete user-detected peaks below           #
#        threshold" option checked.                                  #
#       -snfirst=5: the signal-to-noise ration for the first         #
#        pass is 5                                                   #
#       -snsecond=2: the signal-to-noise ration for the first        #
#        pass is 2                                                   #
#                                                                    #
# Description of the return value                                    #
#       None. Just write a .ccl file to the directory specified by   #
#        file.name                                                   #
#                                                                    #
# Expamle                                                            #
#       ccl.file(zoom,"C:\file.ccl")                                 #
#                                                                    #
#====================================================================#
#              Description of Development                            #
#====================================================================#
#                                                                    #
# Author(s)                                                          #
#       Tan Chuen Seng,Andreas Quandt,Alexander Ploner,Yudi Pawitan  #
#                                                                    #
# Changes:                                                           #
#  29102004: 1.Allow for consecutive significant windows to be       #
#             mergered as 1 region through tuning parameter 'merge'. #
#            2.Use paste instead of apply to paste <VALUE> to mz.    #
#               values                                               #
#  17052005: Allow quantifyPeak output $arrayInfo to be read instead.#
#                                                                    #
#  02082008: Export ARS peaks                                        #
######################################################################

{
  
  #=================================================
  # The first portion of the .ccl file that indicate
  # the biomarker wizard options
  #=================================================
  
  text.ccl<-rbind(c('<?xml version="1.0"?>'),
                  c('<CLUSTERFILE>'),
                  c('<BMWPARAMS>'),
                  c(paste('<PCTSPECTRA>',round(pctspectra/100,10),
                          '</PCTSPECTRA>',sep="")),
                  c(paste('<FORCEMANUAL>',
                          ifelse(forcemanual==TRUE,'true','false'),
                          '</FORCEMANUAL>',sep="")),
                  c(paste('<MASSWINDOW>',round(masswindow/100,10),
                          '</MASSWINDOW>',sep="")),
                  c(paste('<BMWFIRST>',
                          ifelse(bmwfirst==TRUE,'true','false'),
                          '</BMWFIRST>',sep="")),
                  c(paste('<BMWSECOND>',
                          ifelse(bmwsecond==TRUE,'true','false'),
                          '</BMWSECOND>',sep="")),
                  c(paste('<BMWESTIMATED>',
                          ifelse(bmwestimated==TRUE,'true','false'),
                          '</BMWESTIMATED>',sep="")),
                  c(paste('<BMWDELETE>',
                          ifelse(bmwdelete==TRUE,'true','false'),
                          '</BMWDELETE>',sep="")),
                  c(paste('<SNFIRST>',round(snfirst,10),
                          '</SNFIRST>',sep="")),
                  c(paste('<SNSECOND>',round(snsecond,10),
                          '</SNSECOND>',sep="")),
                  c('</BMWPARAMS>'),
                  c('<CLUSTERLIST>'))


  print(length(mz.mean))  
  
  mz.text.ccl<-paste('<VALUE>',mz.mean,'</VALUE>',sep="")

  #==============================================================
  # The complete .ccl file with biomarker options and mz of peaks 
  # from ARS
  #==============================================================
  
  text.ccl<-rbind(text.ccl,matrix(mz.text.ccl,ncol=1),
                  c('</CLUSTERLIST>'),
                  c('</CLUSTERFILE>'))

  #================================================================
  # Write the .ccl file to the directory and file name as indicated
  # by file.name
  #================================================================
  
  write.table(text.ccl,
              file = file.name,
              append = FALSE,
              quote = FALSE,
              sep = " ",
              eol = "\n",
              na = "NA",
              dec = ".",
              row.names = FALSE,
              col.names = FALSE,
              qmethod = c("escape", "double"))  
  
}

