##Comments:
##   -26022007: It was noted that the way the extension was done for situations where the peak is on the edges was inappropriate.
##              This was changed.
##
##   -06032007:
##      1. Change the way the variable "maxWithin" is computed to avoid the following warning message:
##         "the condition has length > 1 and only the first element will be used in: if (maxWithin) {"
##
##   -03042007
##      1. There was a problem because the original cluster region is a dip and not a peak.
##         Therefore, I removed the stop() function and return the orginal window.
##
##   -04042007
##      1. There was a problem because the original cluster region is a dip and not a peak.


"extendDomSpect"<-
function(idRef,data.default,row.id,minmaxInfo,row.id.maxExt,warning.path,cluster.id){
  
  data.cluster<-data.default[row.id,,]
  ##Save the dominant spectrum previous row.id indices
  row.id0<-row.id
  
  ##noise in the cluster
  noise<-max(minmaxInfo[,"noise"])##*ifelse(length(row.id)<=7,1.5,1)
  
  ##==To determine if there is a need to extend the spectra
  ##**Criterion 1: The dominant spectrum max point at the edge of the cluster
  extendCluster1<-(nrow(minmaxInfo)<=2)
  
  ##**Criterion 2: Either valleys of the dominant spectra are at the edges of the cluster
  ##               Persistency in extending the cluster is considered if the intensity difference
  ##               between the peak and valley at the edges are less than the noise level.
  
  ##..Identify the valleys that are at the edges
  ##The row indices in minmaxInfo that correspond to valley
  valley.indice<-which(minmaxInfo[,"max"]==0)
  ##The row indices in 'minmaxInfo' that correspond to valleys at both ends (possibility of multiple peaks)
  valleyEnd.indice<-range(valley.indice)
  ##Indicate whether any of the valleys are at the edge
  valleyEdge<-minmaxInfo[valleyEnd.indice,"id"]%in%c(1,length(row.id))
      
  ##...To determine whether to be persistent in extending the window if the slope is unexpected.
  if(nrow(minmaxInfo)>2){
    
    ##The intensity difference between the peak and valley (i.e. peak-valley).
    ##(This object 'valleyEdgeInt' will be used in Criterion 3)
    valleyEdgeInt<-c(diff(minmaxInfo[valleyEnd.indice[1]:(valleyEnd.indice[1]+1),"intensity"]),
                     diff(minmaxInfo[valleyEnd.indice[2]:(valleyEnd.indice[2]-1),"intensity"]))

###Changes 06032007(Start)
    if(all(valleyEdgeInt<=0)) stop("Something wrong in computing the difference between intensity.")
###Changes 06032007(End)
    
    ##The intensity difference between the valleys at the edge and their corresponding peaks.
    ##If the valley is not at the edge, then set the intensity difference as infinity.
    valleyEdgePersist<-ifelse(valleyEdge,valleyEdgeInt,rep(Inf,2))
    
    ##Logical indication if the extension process should proceed even if the slope is counter to what is expected.
    ##This is because the intensity difference from the peak and valley is less than the noise.
    ##Re-define noise if there are only 5 points
    if(length(row.id)<=5){##When there are only 5 points
      ##Re-define the noise as the 2nd largest from 4 items
      noise<-sort(abs(diff(data.cluster[,idRef,2])))[3]+10^(-10)
    }
    
    persistExt2<-(valleyEdgePersist<noise)
        
  }else{##There is only one valley (i.e. should perform tasks under Criterion 1)
    
    persistExt2<-FALSE
    
  }
  
  ##The second criterion (at least one of the valleys are at the edges).
  extendCluster2<-(sum(valleyEdge)>0)
  
  ##**Criterion 3: The either valleys of the dominant spectra are near the edges of the cluster (i.e. not at the edge!)
  ##               AND its signal is < noise.
  if(nrow(minmaxInfo)>2){

    ##The number of points between the valley and the edge of the cluster
    no.ptBetValleyEdge<-c(1,-1)*minmaxInfo[valleyEnd.indice,"id"]+c(-1,nrow(data.cluster))
    
    if(any(no.ptBetValleyEdge<0)) stop("This should not happen! Check the code!!!")
        
    ##Indicate whether the left or right valley is near the edge/boundary but not at the edge/boundary
    nearby<-(no.ptBetValleyEdge>0 & no.ptBetValleyEdge<=2)
    
    ##The valley(s) near the edges with intensity difference with its peak less than noise
    ##If no such valleys exist, then we will give it NA's
    valleyNear<-ifelse((nearby & valleyEdgeInt<noise),valleyEnd.indice,rep(NA,2))
    
    ##The third criterion (whether to extend nearby)
    extendCluster3<-!all(is.na(valleyNear))
        
  }else{##There is only one valley (i.e. should perform tasks under Criterion 1)
    
    extendCluster3<-FALSE
    
  }
      
##browser()  
  ##========================================================================##
  ##==StepA: Find the peak of the spectra and the extendable region first.==##
  ##==       Then determine whether a valley can be detected in the       ==##
  ##==       extendable region.                                           ==##
  ##========================================================================##
  
  if(extendCluster1){##When the peak is on the edge
    
    if(nrow(minmaxInfo)!=2){
      stop("This should not happen. Peaks occuring at edges happens when there is only 1 peak")
    }
    
    ##Determine the whether the peak is at the right edge (TRUE) or the left edge (FALSE).
    peakEdge<-which(minmaxInfo[,"max"]==1)
        
    ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
    extRegion<-c(ifelse(peakEdge==1,row.id.maxExt[1],row.id[length(row.id)]):
                 ifelse(peakEdge==1,row.id[1],row.id.maxExt[2]))
    
###04042007(Start): Made changes here      
    if(length(extRegion)==1){
      ##That is no possible extension at the present direction.
      ##Change the extension to the other direction
      peakEdge<-!peakEdge
      extRegion<-c(ifelse(peakEdge==1,row.id.maxExt[1],row.id[length(row.id)]):
                   ifelse(peakEdge==1,row.id[1],row.id.maxExt[2]))
      
      ##That is no possible extension at the other direction too.
      ##Return the original row indices
      if(length(extRegion)==1){
        write(paste("extendDomSpect(StepA): The following region was not extendable successfully: cluster.id:",
                    cluster.id, "spectrum.id:", idRef),
              file = paste(warning.path, "warning.txt",sep = ""),append = TRUE)
        print(paste("The following region was not extendable successfully: cluster.id:", cluster.id, "spectrum.id:", 
                    idRef, " by the function extendDomSpect at StepA", 
                    sep = ""))
        
        return(row.id)
      }
    }
###04042007(End): Made changes here
##browser()      
    ##The first difference of the intensities in the extendable region.
    diff1.intensity<-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        
    ##Locate the peak and the valley at the extendable region
    if(peakEdge==1){##The peak is at the left edge
      
      ##Slopes that are positive in the extended region
      slopePositive<-(diff1.intensity>0)
###26022007(Start): Made changes here      
      ##Find the first peak on the left extendable region
      ##If all are negative, then there is no peak (i.e. peakValley=0)
      peakValley<-ifelse(any(slopePositive),max(which(slopePositive)),0)
      ##Exclude the region between the first peak of the left (peakValley)
      ##and the original left edge of the cluster
      ##If there is no peak (i.e. peakValley=0), then validpeakValley=0.
      validpeakValley<-ifelse(peakValley!=0,1,0):ifelse(peakValley!=0,peakValley,0)
      ##Append to peakValley the next valley to the left before the peak
      ##If there is no peak (i.e. peakValley=0)
      peakValley<-c(peakValley,
                    ifelse(any(!slopePositive[validpeakValley]),
                           max(which(!slopePositive[validpeakValley])),0))
      ##The possible extension for the left indice, provided the valley is real
      leftExt<-(extRegion[1]+peakValley[2])
      
      if(peakValley[1]>peakValley[2]){##A peak and valley was detected
###26022007(End): Made changes here        
        ##The actual extension to the left after taking into consideration that the clusters do not overlap
        row.id<-c(leftExt:row.id[length(row.id)])
      }else{##A valley/peak was not detected
        ##The actual extension to the left after taking into consideration that the clusters do not overlap
        row.id<-c(extRegion[1]:row.id[length(row.id)])
###Changes 03042007(Start)        
#        stop("The region with peak on the left only: After extension, still no improvement!!!")
###Changes 03042007(End)        
      }
      
    }else{##The peak is at the right edge
      
      ##Slopes that are positive in the extended region
      slopePositive<-(diff1.intensity>0)
###26022007(Start): Made changes here
      ##Find the first peak to the right extendable region   
      peakValley<-ifelse(any(!slopePositive),min(which(!slopePositive)),length(extRegion))
      ##Exclude the region between the original right edge of the cluster
      ##and the first peak of the right (peakValley)
      validpeakValley<-ifelse(peakValley<=length(slopePositive),peakValley,0):
                       ifelse(peakValley<=length(slopePositive),length(slopePositive),0)
      ##Append to peakValley the next valley to the right after the peak
      peakValley<-c(peakValley,
                    ifelse(any(slopePositive[validpeakValley]),
                           (peakValley+(min(which(slopePositive[validpeakValley]))-1)),
                           length(slopePositive)))
      ##The possible extension for the right indice, provided the valley is real
      rightExt<-(extRegion[1]+(peakValley[2]-1))
###26022007(End): Made changes here
      if(diff(peakValley)>0){##A peak and valley was detected
        ##The actual extension to the right after taking into consideration that the clusters do not overlap
        row.id<-c(row.id[1]:rightExt)
      }else{##A valley/peak was not detected
        ##The actual extension to the left after taking into consideration that the clusters do not overlap
        row.id<-c(row.id[1]:extRegion[2])
###Changes 03042007(Start)                
#        stop("The region with peak on the right only: After extension, still no improvement!!!")
###Changes 03042007(Start)                
      }
      
    }      
  }


  ##================================================================================##
  ##==StepB: Find whether the valley at the edge of the cluster should be extended==##
  ##================================================================================##
  
  if(extendCluster2 & !extendCluster1){##The valley at the edge may not be the true valley and the peak is not at the edge
    
    ##============================================##
    ##==Extension of the valley on the left edge==##
    ##============================================##
    
    ##The left valley may not be the true valley
    if(valleyEdge[1]){##The left valley should be extended
      
      ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
      ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) leftExt=row.id[1]
      extRegion<-c(row.id.maxExt[1]:row.id[1])
      
###04042007(Start): Made changes here      
#      ##That is no possible extension at the left.
#      ##Return the original left row indice
#      if(length(extRegion)==1){
#        write(paste("extendDomSpect(StepB): The following region was not extendable successfully: cluster.id:",
#                    cluster.id, "spectrum.id:", idRef,),
#              file = paste(warning.path, "warning.txt",sep = ""),append = TRUE)
#        print(paste("The following region was not extendable successfully: cluster.id:", cluster.id, "spectrum.id:", 
#                    idRef, " by the function extendDomSpect at StepB", 
#                    sep = ""))
#        
#        leftExt<-row.id[1]
#      }else{
###04042007(End): Made changes here

      
      ##The first difference of the intensities in the extendable region.
      ##If extRegion is of length 1, it will return numeric(0)
      diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
      ##Slopes that are positive in the extended region
      ##If extRegion is of length 1, it will return logical(0)
      slopePositive<-(diff1.intensity>0)
      ##Find the first valley on the left extendable region
      ##If extRegion is of length 1, it will return 0
      valley<-ifelse(any(!slopePositive),max(which(!slopePositive)),0)
      ##The possible extension for the left indice, provided the valley is real
      leftExt<-(extRegion[1]+valley)
      ##The row indices of 'data.default' that correspond to the new left valley and its nearest peak
      new.indice1<-c(leftExt,row.id0[minmaxInfo[(valleyEnd.indice[1]+1),"id"]])
      ##The intensity difference between the new valley and the peak near the left edge of a window
      int.diff1<-diff(data.default[new.indice1,idRef,2])
      ##Indicator if the new extension to the left is still below the noise level (i.e. no improvment)
      noImprovement2<-(int.diff1<noise)

      ##==========================================================================================##
      ##==Decide whether to be persistent in extending the cluster with a new valley on the left==##
      ##==========================================================================================##
      if(persistExt2[1] & noImprovement2){##There might be a need to be persistent as the signal is less than the noise
        
        ##...First, detect the peak on the left extended region
        ##The REVISED extendable region (extRegion) using the indices of 'data.default'.
        ##This revised extendable region is to find a potential peak.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) leftPeak1=row.id[1]
        extRegion<-c(row.id.maxExt[1]:leftExt)
        
###04042007(Start): Made changes here      
#          ##That is no possible extension at the left direction.
#          ##Return the left row indice
#          if(length(extRegion)==1){
#            write(paste("extendDomSpect(StepB): The following region was not extendable successfully: cluster.id:",
#                        cluster.id, "spectrum.id:", idRef,),
#                  file = paste(warning.path, "warning.txt",sep = ""),append = TRUE)
#            print(paste("The following region was not extendable successfully: cluster.id:", cluster.id, "spectrum.id:", 
#                        idRef, " by the function extendDomSpect at StepB with persistence", 
#                        sep = ""))
#          
#            leftExt<-row.id[1]
#          
#          }else{
###04042007(End): Made changes here
        
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the first peak on the left REVISED extendable region
        ##If extRegion is of length 1, it will return 0
        peak1<-ifelse(any(slopePositive),max(which(slopePositive)),0)
        ##A possible peak on the left extension
        leftPeak1<-(extRegion[1]+peak1)
        ##The increase in intensity with the possible new peak and previous new valley.
        int.diff2<-diff(data.default[c(leftExt,leftPeak1),idRef,2])
        
        ##...Second, detect the next valley on the left extended region
        ##The REVISED extendable region (extRegion) using the indices of 'data.default'.
        ##This revised extendable region is to find a potential valley.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) leftValley1=row.id[1]
        extRegion<-c(row.id.maxExt[1]:leftPeak1)
        
###04042007(Start): Made changes here      
#            ##That is no possible extension at the left direction.
#            ##Return the original left row indice
#            if(length(extRegion)==1){
#              write(paste("extendDomSpect(StepB): The following region was not extendable successfully: cluster.id:",
#                          cluster.id, "spectrum.id:", idRef,),
#                    file = paste(warning.path, "warning.txt",sep = ""),append = TRUE)
#              print(paste("The following region was not extendable successfully: cluster.id:", cluster.id, "spectrum.id:", 
#                          idRef, " by the function extendDomSpect at StepB with persistence", 
#                          sep = ""))
#              
#              leftExt<-row.id[1]
#              
#            }else{
###04042007(End): Made changes here        
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the next valley on the left REVISED extendable region
        ##If extRegion is of length 1, it will return 0
        valley1<-ifelse(any(!slopePositive),max(which(!slopePositive)),0)
        ##A possible peak on the left extension
        leftValley1<-(extRegion[1]+valley1)
        ##The increase in intensity with the possible new peak and another new valley.
        int.diff3<-diff(data.default[c(leftValley1,leftPeak1),idRef,2])
        
        ##The persistence is improving the signal (i.e. R object cont):
        ##  Case 1: int.diff3>noise (i.e. the signal from the difference between leftValley1 and leftPeak1 is bigger than the noise); or
        ##  Case 2: int.diff2>noise (i.e. the signal from the difference between leftPeak1 and leftExt is bigger than the noise); or
        ##
        ##  If the signal is still smaller than the noise, we will then check if the the signal was improved with the extension.
        ##
        ##  Case 3: int.diff3>int.diff1 (i.e. the signal from the difference between leftValley1 and leftPeak1 is bigger than
        ##                                    the signal from the difference between leftExt and its nearest peak within the cluster)
        ##  Case 4: int.diff2>int.diff1 (i.e. the signal from the difference between leftPeak1 and leftExt is bigger than
        ##                                    the signal from the difference between leftExt and its nearest peak within the cluster)
              
        cont<-(int.diff3>noise | int.diff2>noise | int.diff3>int.diff1 | int.diff2>int.diff1)
        
        ##To ensure things are going on as expected
        if(int.diff3<0 | int.diff2<0 | int.diff1<0){
          stop("Something is wrong with my intensity difference in the left extension.")
        }
        
        ##To decide whether to extend the cluster to include the new valley (leftValley1) and peak (leftPeak1).
        if(cont & (leftValley1>=row.id.maxExt[1]) & (leftPeak1>leftValley1)){##We should accept the 2nd valley and not the 1st valley
          leftExt<-leftValley1
        }
              
#            }##Loop of the extRegion from: Second, detect the next valley on the left extended region
#          
#          }##Loop of the extRegion from: First, detect the peak on the left extended region
#          
      }##Loop to decide whether to be persistent in extension
        
#      }##Loop of the extRegion from: The left valley should be extended
        
    }else{##The left valley need not be extended
      
      leftExt<-row.id[1]
      
    }
    
    
    ##=============================================##
    ##==Extension of the valley on the right edge==##
    ##=============================================##
    
    ##The right valley may not be the true valley
        
    if(valleyEdge[2]){##The right valley could be extended
      
      ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
      ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) leftExt=row.id[1]      
      extRegion<-c(row.id[length(row.id)]:row.id.maxExt[2])
      ##The first difference of the intensities in the extendable region.
      ##If extRegion is of length 1, it will return numeric(0)
      diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
      ##Slopes that are positive in the extended region
      slopePositive<-(diff1.intensity>0)
      ##Find the valley on the right extendable region
      valley<-ifelse(any(slopePositive),min(which(slopePositive)),length(extRegion))
      ##The possible extension for the right indice, provided the valley is real
      rightExt<-(extRegion[1]+valley-1)
      ##The row indices of 'data.default' that correspond to the new right valley and the peak
      new.indice1<-c(rightExt,row.id0[minmaxInfo[(valleyEnd.indice[2]-1),"id"]])
      ##The intensity difference between the new valley and the peak near the right edge of a window
      int.diff1<-diff(data.default[new.indice1,idRef,2])
      ##Indicator if the new extension to the left is still below the noise level (i.e. no)
      noImprovement2<-(int.diff1<noise)
      
      if(persistExt2[2] & noImprovement2){##There might be a need to be persistent as the signal is less than the noise
        
        ##...First detect the peak on the right extended region
        ##The REVISED extendable region (extRegion) using the indices of 'data.default'.
        ##This revised extendable region is to find a potential peak.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) rightPeak1=row.id[length(row.id)]        
        extRegion<-c(rightExt:row.id.maxExt[2])
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the first peak on the right REVISED extendable region
        ##If extRegion is of length 1, it will return length(extRegion)
        peak1<-ifelse(any(!slopePositive),min(which(!slopePositive)),length(extRegion))
        ##A possible peak on the right extension
        rightPeak1<-(extRegion[1]+peak1-1)
        ##The increase in intensity with the possible new peak and previous new valley.
        int.diff2<-diff(data.default[c(rightExt,rightPeak1),idRef,2])
        
        ##...Second detect the next valley on the right extended region
        ##The REVISED extendable region (extRegion) using the indices of 'data.default'.
        ##This revised extendable region is to find a potential valley.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) rightValley1=row.id[length(row.id)]        
        extRegion<-c(rightPeak1:row.id.maxExt[2])
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the next valley on the right REVISED extendable region
        ##If extRegion is of length 1, it will return length(extRegion)
        valley1<-ifelse(any(slopePositive),min(which(slopePositive)),length(extRegion))
        ##A possible peak on the right extension
        rightValley1<-(extRegion[1]+valley1-1)
        ##The increase in intensity with the possible new peak and another new valley.
        int.diff3<-diff(data.default[c(rightValley1,rightPeak1),idRef,2])
        
        ##The persistence is improving the signal (i.e. R object cont):
        ##  Case 1: int.diff3>noise (i.e. the signal from the difference between rightValley1 and rightPeak1 is bigger than the noise); or
        ##  Case 2: int.diff2>noise (i.e. the signal from the difference between rightPeak1 and rightExt is bigger than the noise); or
        ##
        ##  If the signal is still smaller than the noise, we will then check if the the signal was improved with the extension.
        ##
        ##  Case 3: int.diff3>int.diff1 (i.e. the signal from the difference between rightValley1 and rightPeak1 is bigger than
        ##                                    the signal from the difference between rightExt and its nearest peak within the cluster)
        ##  Case 4: int.diff2>int.diff1 (i.e. the signal from the difference between rightPeak1 and rightExt is bigger than
        ##                                    the signal from the difference between rightExt and its nearest peak within the cluster)
        cont<-(int.diff3>noise | int.diff2>noise | int.diff3>int.diff1 | int.diff2>int.diff1)
        
        if(int.diff3<0 | int.diff2<0 | int.diff1<0){
          browser()
          stop("Something is wrong with my intensity difference for the right extension")
        }

        ##To decide whether to extend the cluster to include the new valley (leftValley1) and peak (leftPeak1).
        if(cont & (rightValley1<=row.id.maxExt[2]) & (rightPeak1<rightValley1)){##We should accept the 2nd valley and not the 1st valley
          rightExt<-rightValley1
        }
      }
      
    }else{##The right valley need not be extended
      rightExt<-row.id[length(row.id)]
    }
    
    ##The actual extension to the left after taking into consideration that the clusters do not overlap
    row.id<-c(leftExt:rightExt)
  }
  
  ##=======================================================================##
  ##==StepC: Find new valley of the spectra whose valley is near the edge==##
  ##=======================================================================##
  if(extendCluster3 & !extendCluster1){##The nearby valley at the edge may not be the true valley
    
    ##=============================================================##
    ##==Extension applied to valleys near the left edges/boundary==##
    ##=============================================================##
    if(!is.na(valleyNear[1])){##The nearby valley on the left has a weak signal
      
      ##The row indices of 'data.default' corresponding to the region bewteen the left edge/boundary and the valley
      within<-row.id0[1]:row.id0[minmaxInfo[valleyNear[1],"id"]]
      
      ##=======================================================================##
      ##==Decide what type of slope to anticipate at the left extended region==##
      ##=======================================================================##
      if(length(within)==3){##There are three points in the region between the left valley and cluster edge.
###Changes 06032007(Start)
        ##There is a peak in the region between the right valley and cluster edge: True(=Peak)/FALSE(=None)
        maxWithin<-all((diff(data.default[within,idRef,2])>0)==c(TRUE,FALSE))
###Changes 06032007(End)
        
      }else{##There are only two points in the region between the left valley and cluster edge.
        
        if(length(within)!=2) stop("The nearness criterion is wrong!")
        
        ##There is no peak in the region between the right valley and cluster edge (2 points only).
        maxWithin<-FALSE
        
      }
      
      ##================================================##
      ##==Steps to decide whether to extend the region==##
      ##================================================##
      
      if(maxWithin){##There is a peak between the left valley and cluster edge
        
        ##Locate the row indice of the peak in the region between the left valley and the edge
        peakTmp<-row.id0[2]
        
        ##..Find a valley
        ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) leftExt=row.id0[1]      
        extRegion<-c(row.id.maxExt[1]:row.id0[1])
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the valley on the left extendable region
        ##If extRegion is of length 1, it will return 0
        valley<-ifelse(any(!slopePositive),max(which(!slopePositive)),0)
        ##The possible extension for the left indice.
        leftExt<-(extRegion[1]+valley)
        ##The row indices of 'data.default' that correspond to the new left valley and "peak" between the left edge and the valley
        new.indice1<-c(leftExt,peakTmp)
        ##The intensity difference between the new peak and the valley near the right edge of a window
        int.diff1<-diff(data.default[new.indice1,idRef,2])
        
        
        if(int.diff1<0) stop("Something is wrong with my intensity difference for the right extension")
        

        if(!(leftExt>=row.id.maxExt[1] & (int.diff1>noise | int.diff1>valleyEdgeInt[1]))){
          
          ##We do not extend the cluster
          leftExt<-row.id[1]
          
        }   
        
      }else{##There is no peak between the left valley and cluster edge
        
        ##..First find a peak
        ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) leftExt1=row.id0[1]      
        extRegion<-c(row.id.maxExt[1]:row.id0[1])
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the peak on the left extendable region
        ##If extRegion is of length 1, it will return 0
        peak1<-ifelse(any(slopePositive),max(which(slopePositive)),0)
        ##The possible extension for the left indice.
        leftExt1<-(extRegion[1]+peak1)
        ##The row indices of 'data.default' that correspond to the old valley at the left edge and the new peak
        new.indice1<-c(row.id0[minmaxInfo[valleyEnd.indice[1],"id"]],leftExt1)
        ##The intensity difference between the new peak and the valley near the left edge of a window
        int.diff1<-diff(data.default[new.indice1,idRef,2])
        
        ##..Now find a valley for the new peak
        ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) leftExt2=row.id0[1]      
        extRegion<-c(row.id.maxExt[1]:leftExt1)
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return numeric(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the peak on the left extendable region
        ##If extRegion is of length 1, it will return 0
        valley1<-ifelse(any(!slopePositive),max(which(!slopePositive)),0)
        ##The possible extension for the left indice.
        leftExt2<-(extRegion[1]+valley1)
        ##The row indices of 'data.default' that correspond to the new left peak and valley at the edge
        new.indice2<-c(leftExt2,leftExt1)
        ##The intensity difference between the new peak and valley near the left edge of a window
        int.diff2<-diff(data.default[new.indice2,idRef,2])
        
        if(int.diff2<0 | int.diff1<0){
          stop("Something is wrong with my intensity difference for the left extension")
        }
        
        if(((leftExt2>=row.id.maxExt[1]) & (leftExt2<leftExt1)) &
           (int.diff1>noise | int.diff2>noise | int.diff1>valleyEdgeInt[1] | int.diff2>valleyEdgeInt[1])){
          ##There is an improvement in the signal with the extension
          
          ##We should accept the 2nd (new) valley and not the 1st (old) valley
          leftExt<-leftExt2
          
        }else{##There is NO improvement in the signal with the extension
          
          ##We do not extend the cluster
          leftExt<-row.id[1]
          
        }   
      }
      
    }else{##There is no need to extend the cluster on the left
      
      ##Set the orginal left indice
      leftExt<-row.id[1]
      
    }
        
    
    ##==============================================================##
    ##==Extension applied to valleys near the right edges/boundary==##
    ##==============================================================##
    if(!is.na(valleyNear[2])){##The nearby valley on the right contribute to a weak signal
      
      ##The row indices of 'data.default' corresponding to the region bewteen the right edge and valley
      within<-row.id0[minmaxInfo[valleyNear[2],"id"]]:row.id0[length(row.id0)]
      
      ##========================================================================##
      ##==Decide what type of slope to anticipate at the right extended region==##
      ##========================================================================##
      if(length(within)==3){##There are three points in the region between the right valley and cluster edge.
###Changes 06032007(Start)        
        ##There is a peak in the region between the right valley and cluster edge: True(=Peak)/FALSE(=None)
        maxWithin<-all((diff(data.default[within,idRef,2])>0)==c(TRUE,FALSE))
###Changes 06032007(End)        
        
      }else{##There are only two points in the region between the right valley and cluster edge.
        
        if(length(within)!=2) stop("Something wrong with the nearby criterion.")
        
        ##There is no peak in the region between the right valley and cluster edge.
        maxWithin<-FALSE
        
      }
      
      ##================================================##
      ##==Steps to decide whether to extend the region==##
      ##================================================##
      
      if(maxWithin){##There is a peak between the right valley and cluster edge
        
        ##Locate the row indice of the peak in the region between the valley and the right edge
        peakTmp<-row.id0[(length(row.id0)-1)]
        
        ##..Find a valley
        ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) rightExt=row.id0[length(row.id0)]
        extRegion<-c(row.id0[length(row.id0)]:row.id.maxExt[2])
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the valley on the right extendable region
        valley<-ifelse(any(slopePositive),min(which(slopePositive)),length(extRegion))
        ##The possible extension for the right indice.
        rightExt<-(extRegion[1]+valley-1)
        ##The row indices of 'data.default' that correspond to the new right valley and old peak at the edge
        new.indice1<-c(rightExt,peakTmp)
        ##The intensity difference between the new peak and the valley near the right edge of a window
        int.diff1<-diff(data.default[new.indice1,idRef,2])
        
        if(int.diff1<0) stop("Something is wrong with my intensity difference for the right extension")
        
        if(!(rightExt<=row.id.maxExt[2] & (int.diff1>noise| int.diff1>valleyEdgeInt[2]))){
          
          ##We do not extend the cluster
          rightExt<-row.id[length(row.id)]
          
        }   
        
      }else{##There is no peak between the right valley and cluster edge
        
        ##..First find a peak
        ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) rightExt1=row.id0[length(row.id0)]
        extRegion<-c(row.id0[length(row.id0)]:row.id.maxExt[2])
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the peak on the right extendable region
        ##If extRegion is of length 1, it will return length(extRegion)
        peak1<-ifelse(any(!slopePositive),min(which(!slopePositive)),length(extRegion))
        ##The possible extension for the right indice.
        rightExt1<-(extRegion[1]+peak1-1)
        ##The row indices of 'data.default' that correspond to the old valley at the edge and the new right peak
        new.indice1<-c(row.id0[minmaxInfo[valleyEnd.indice[2],"id"]],rightExt1)
        ##The intensity difference between the new peak and the valley near the right edge of a window
        int.diff1<-diff(data.default[new.indice1,idRef,2])
        
        
        ##..Now find a valley for the peak
        ##The extendable region (extRegion) using the indices of 'data.default' assuming overlapping is not allowed.
        ##Note: Even if there is no region extendable (i.e. extRegion is of length 1) rightExt2=row.id0[length(row.id0)]
        extRegion<-c(rightExt1:row.id.maxExt[2])
        ##The first difference of the intensities in the extendable region.
        ##If extRegion is of length 1, it will return numeric(0)
        diff1.intensity <-diff(data.default[extRegion,idRef,2],lag=1,differences=1)
        ##Slopes that are positive in the extended region
        ##If extRegion is of length 1, it will return logical(0)
        slopePositive<-(diff1.intensity>0)
        ##Find the peak on the right extendable region
        ##If extRegion is of length 1, it will return length(extRegion)
        valley1<-ifelse(any(slopePositive),min(which(slopePositive)),length(extRegion))
        ##The possible extension for the right indice.
        rightExt2<-(extRegion[1]+valley1-1)
        ##The row indices of 'data.default' that correspond to the new valley at the edge and new right peak
        new.indice2<-c(rightExt2,rightExt1)
        ##The intensity difference between the new peak and valley near the right edge of a window
        int.diff2<-diff(data.default[new.indice2,idRef,2])

        if(int.diff2<0 | int.diff1<0) stop("Something is wrong with my intensity difference for the right extension")
            
        if(((rightExt2<=row.id.maxExt[2]) & (rightExt1<rightExt2)) &
           (int.diff1>noise | int.diff2>noise | int.diff1>valleyEdgeInt[2] | int.diff2>valleyEdgeInt[2])){
          ##There is an improvement in the signal with the extension
          
          ##We should accept the 2nd valley and not the 1st valley
          rightExt<-rightExt2
          
        }else{##There is NO improvement in the signal with the extension
          
          ##We do not extend the cluster
          rightExt<-row.id[length(row.id)]
          
        }   
      }
      
      
    }else{##There is no need to extend the cluster on the right
      
      ##Set the orginal right indice
      rightExt<-row.id[length(row.id)]
      
    }
    
    ##Updated row indices w.r.t. to 'data.default'.
    row.id<-c(leftExt:rightExt)
    
  }
  
  return(row.id)

}
