##NOTE (05/01/2007): Include the parametric approach. Included two new arguments to the function,
##                   'parPeak' and 'par.maxiter'. 'parPeak' is logical indicating whether parametric
##                   approach is used, while 'par.maxiter' sets the number of iteration within the
##                   EM-algorithm. It returns an additional component in the list output '$infoPA'.
##NOTE (08/01/2007): Remove typ.range from reduct.ProSpect
##
##NOTE (08/02/2007):
##   1. Added the new arguments specified in findPeaks into the function settings().
##   2. Added a new function here called peakOut.file that output .ARS and/or .PA files.
##
##NOTE (12/02/2007)
##   1. Make the peakOut.file as .ars and .pa files.
##
##
##NOTE(6/9/2007):
##   1. Put the option for wanting the diagnostic plots, using arguement plots.TF.
##
##NOTE(6/9/2007):
##   1. We made changes to ccl.file.merge and peakOut.file.
##      Now .ccl gives the mean mz of the peaks detected by ARS via peakOut.file
##
##NOTE(26/11/2010):
##   1. Using reduct.ProSpect object for quantifying peaks.
##
##NOTE(02/12/2010):
##   1. For those without Background, return loess.dat as reduct.dat.


"findPeaks" <-
function(fullFolderName=NULL,region=NULL,outputName=NULL,
                      saveSettings=TRUE,doBackground=TRUE,
                      smoother="rsmooth",bandwidth=5,subsample=10,
                      winSize=5,B.mse=10,B.Fprime=10,
                      mean=1,distr="chisq",
                      signCriterion="fdr.BH",
                      signLevel=0.05,
                      mz.limit=0.05,
                      symm.limit=1.3,
                      toler=0.000001,maxiter=20,msetol=0.975,
                      parPeak=TRUE,par.maxiter=100,plots.TF=TRUE) {

  if(is.null(fullFolderName) | is.null(region) | is.null(outputName))
    {stop(paste("Please use e.g.: findPeaks(\"C:/experiment/\",",
                "c(3,10),\"cclFileName\")"))}
  
  if(max(region*1000)> 190000){
    stop("The maximum for the choosen region is 190kDa. ")
  }else{region <- region*1000}  
                          #that the given range is in kDa instead of Da

  if(mean!=1 & mean!=2 & mean!=3 & mean !=4) {
    stop("please use for 'mean' one of following numbers: 1-4")
  }
  if(mean==1){name <- "median"; corr.name <- "median"}
  if(mean==2){name <- "arithmetic"; corr.name <- "median"}
  if(mean==3){name <- "arithmetic"; corr.name <- "arithmetic"}
  if(mean==4){name <- "median"; corr.name <- "arithmetic"}
  
  wd <- getwd()
  setwd(fullFolderName)  
                          #to be sure that the pdf are in the right
                          # (experiment) directory  
  
#  fullFolderName <- fullFolderName
#  outputName <- outputName
#  region <- region             #normLoess.ProSpect
#  subsample <- subsample       #normLoess.ProSpect  
#  B.mse <- B.mse             #modF
#  B.Fprime <- B.Fprime          #modFcorr.fdr
#  mean <- mean            #modFcorr.fdr, zoom.alpha
#  distr <- distr        #zoom.alpha
#  signCriterion <- signCriterion   #zoom.aplpha
#  signLevel <- signLevel
#  merge <- merge

  if(smoother=="rsmooth") {
    if(subsample>1){
      subsample = 1
#      warning(paste("For \"smoother=rsmooth\", only",
#                  "subsample=1 is allowed. Program sets subsample=1."))
    }
  }

##080207: Added doBackground, mz.limit, symm.limit, parPeak, par.maxiter at the end
##060907: Decide whether the diagnostic plot is wanted
  settings <- function(){
    t <- c("fullFolderName: ",fullFolderName,
           "outputName: ",outputName,
           "region: ",toString(region/1000),
           "smoother: ",smoother,
           "bandwidth: ",bandwidth,
           "subsample: ",subsample,
           "winSize: ",winSize,
           "B.mse: ",B.mse,
           "B.Fprime: ",B.Fprime,
           "mean: ",paste(mean," (",name,"/",corr.name,")",sep=""),
           "distr: ",distr,
           "signCriterion: ",signCriterion,
           "signLevel: ",signLevel,
           "toler:",toler,
           "maxiter:",maxiter,
           "msetol:",msetol,
           "doBackground:",doBackground,
           "mz.limit",mz.limit,
           "symm.limit",symm.limit,
           "parPeak",parPeak,
           "par.maxiter",par.maxiter,
           "plots.TF",plots.TF)
    write(t,paste(fullFolderName,outputName,".settings",sep=""),2)
  }
  
##020808: A function that produces the .ccl, .ARS and .PA files
  peakOut.file<-function(quantify.dat,read.dat,
                         file.name=paste(fullFolderName,outputName,sep=""),
                         pctspectra=10,forcemanual=TRUE,
                         masswindow=0.3, bmwfirst=TRUE, bmwsecond=TRUE,
                         bmwestimated=TRUE, bmwdelete=TRUE, snfirst=5,
                         snsecond=2){

    ##Extract the m/z and intensity via ARS or parametric approach
    peak<-peakInfoIntMz(quantify.dat,read.dat)

    ##The ARS approach
    datARS<-data.frame(matrix(c(peak$infoARS),ncol=2),
                       peak=rep(rownames(peak$infoARS),times=ncol(peak$infoARS)),
                       spectrum=rep(colnames(peak$infoARS),each=nrow(peak$infoARS)),
                       stringsAsFactors=FALSE)
    colnames(datARS)[1:2]<-dimnames(peak$infoARS)[[3]]

    ##Output the stuff
    write.table(datARS, file = paste(file.name,".ars",sep=""),
                append = FALSE, quote = FALSE, 
                sep = ",", na = "NA", dec = ".", row.names = FALSE, 
                col.names = TRUE, qmethod = c("escape", "double"))
    
    ##The PA approach
    if(!is.null(peak$infoPA)){
      datPA<-data.frame(matrix(c(peak$infoPA),ncol=2),
                        peak=rep(rownames(peak$infoPA),times=ncol(peak$infoPA)),
                        spectrum=rep(colnames(peak$infoPA),each=nrow(peak$infoPA)),
                        stringsAsFactors=FALSE)
      colnames(datPA)[1:2]<-dimnames(peak$infoPA)[[3]]

      ##Output the stuff
      write.table(datPA, file = paste(file.name,".pa",sep=""),
                  append = FALSE, quote = FALSE, 
                  sep = ",", na = "NA", dec = ".", row.names = FALSE, 
                  col.names = TRUE, qmethod = c("escape", "double"))
    }
    ##The .ccl file for ARS
    peak.mz.mean<-apply(peak$infoARS[,,1],1,mean)
    ccl.file.merge(peak.mz.mean,paste(file.name,".ccl",sep=""))
    
    return(NULL)
  }
  
    
  print("start to read data sets")
  read.dat <- read.ProSpect(fullFolderName)
#  clin.dat <- read.dat[[2]]
#  read.dat <- read.dat[[1]]
  print("start to reduce the data to region of interest")
 

  ##Remove typ.range
  reduct.dat <- reduct.ProSpect(dat=read.dat[[1]],range.mz=region)
  
 
  
  if(doBackground) {
    print("start to do baseline correction")
    loess.dat <- BgCorrection.ProSpect(dat=reduct.dat,smoother=smoother,bandwidth=bandwidth,subsample=subsample)
    print("start to calculate the F-statistics")
    mlm.dat <- mlm.ProSpect(loess.dat,num.points=winSize)
    modF.dat<-mod.F(mlm.dat,B=B.mse)
    modFcorrFdr.dat <- modFcorr.fdr(modF.dat,name=name,B=B.Fprime)
    print("start to find the singificant peaks")
    zoom.dat<- zoom.alpha(modFcorrFdr.dat,loess.dat,NULL,
                          corr.name=corr.name,pType=distr,sign.type=signCriterion,
                          alpha.level=signLevel)

####!!!!!CHANGED 191205: Still allow output despite of no significant windows
    if(is.null(zoom.dat)){
      print("There are no significant values at all. Please increase the level of significance")
      return(list(read.dat = read.dat[[1]],
                  clin.dat = read.dat[[2]], 
                  sample = loess.dat$sample,
                  reduct.dat = reduct.dat,
                  loess.dat = loess.dat,
                  mlm.dat = mlm.dat,
                  modF.dat = modF.dat, 
                  modFcorrFdr.dat = modFcorrFdr.dat,
                  zoom.dat = zoom.dat,
                  region = region,
                  folder = fullFolderName,
                  smoother = smoother, 
                  signLevel = signLevel))    
    }
    
    print("quantifyPeaks")
####....OLD CODE 201005 TCS (Keep the quantified peak output and allow change to arguments mz.limit and symm.limit)    
####quantifyPeaks(loess.dat=loess.dat, zoom.dat=zoom.dat,fullFolderName=fullFolderName,mz.limit=0.05, symm.limit=1.3)
####...050107: Added in two new agruments to quantifyPeaks
####...060907: Decide whether the diagnostic plot is wanted
    quantify.dat<-quantifyPeaks(loess.dat=loess.dat, zoom.dat=zoom.dat,fullFolderName=fullFolderName,
                                mz.limit=mz.limit, symm.limit=symm.limit,
                                toler=toler,maxiter=maxiter,msetol=msetol,
                                parPeak=parPeak,par.maxiter=par.maxiter,plots=plots.TF)
    
#    print("start to create the .ccl file")
#    ccl.file.merge(zoom.dat,paste(fullFolderName,outputName,".ccl",sep=""), pctspectra=10,forcemanual=TRUE,
#                   masswindow=0.3, bmwfirst=TRUE, bmwsecond=TRUE,
#                   bmwestimated=TRUE, bmwdelete=TRUE, snfirst=5,
#                   snsecond=2,merge=merge)
    if(saveSettings){
      print("start to create .settings file")
      settings()
    }
    
##020808: Produces the .ccl, .ARS and .PA files    
    print("start to create the .ccl, .ARS or/and .PA file")    
    peakOut.file(quantify.dat,read.dat,paste(fullFolderName,outputName,sep=""))
    
    print("finished")    
    setwd(wd)
                              #reset the working directory back to the
                              # previous one

    return(list(read.dat = read.dat[[1]],
                clin.dat = read.dat[[2]], 
                sample = loess.dat$sample,
                reduct.dat = reduct.dat, 
                loess.dat = loess.dat,
                mlm.dat = mlm.dat,
                modF.dat = modF.dat, 
                modFcorrFdr.dat = modFcorrFdr.dat,
                zoom.dat = zoom.dat,
                quantify.dat=quantify.dat,
                region = region,
                folder = fullFolderName,
                smoother = smoother, 
                signLevel = signLevel))

  }else{  
    print("start to calculate the F-statistics")
    mlm.dat <- mlm.ProSpect(reduct.dat)
    modF.dat<-mod.F(mlm.dat,B=B.mse)
    modFcorrFdr.dat <- modFcorr.fdr(modF.dat,name=name,B=B.Fprime)
    print("start to find the singificant peaks")
    zoom.dat<- zoom.alpha(modFcorrFdr.dat,reduct.dat,NULL,
                          corr.name=corr.name,pType=distr,sign.type=signCriterion,
                          alpha.level=signLevel)
    
####!!!!!CHANGED 191205: Still allow output despite of no significant windows
    if(is.null(zoom.dat)){
      print("There are no significant values at all. Please increase the level of significance")
      return(list(read.dat = read.dat[[1]],
                  clin.dat = read.dat[[2]], 
                  sample = loess.dat$sample,
                  reduct.dat = reduct.dat, 
                  mlm.dat = mlm.dat,
                  modF.dat = modF.dat, 
                  modFcorrFdr.dat = modFcorrFdr.dat,
                  zoom.dat = zoom.dat,
                  region = region,
                  folder = fullFolderName,
                  smoother = smoother, 
                  signLevel = signLevel))    
    }
    

    print("quantifyPeaks")
####....OLD CODE 201005 TCS (Keep the quantified peak output and allow change to arguments mz.limit and symm.limit)    
####quantifyPeaks(loess.dat=loess.dat, zoom.dat=zoom.dat,fullFolderName=fullFolderName,mz.limit=0.05, symm.limit=1.3)
####...050107: Added in two new agruments to quantifyPeaks    
####...060907: Decide whether the diagnostic plot is wanted
####...261110: Using reduct.dat instead  of loess.dat	  
    quantify.dat<-quantifyPeaks(loess.dat=reduct.dat, zoom.dat=zoom.dat,fullFolderName=fullFolderName,mz.limit=mz.limit, symm.limit=symm.limit,parPeak=parPeak,par.maxiter=par.maxiter)
    
#    print("start to create the .ccl file")
#    ccl.file.merge(zoom.dat,paste(fullFolderName,outputName,".ccl",sep=""), pctspectra=10,forcemanual=TRUE,
#                   masswindow=0.3, bmwfirst=TRUE, bmwsecond=TRUE,
#                   bmwestimated=TRUE, bmwdelete=TRUE, snfirst=5,
#                   snsecond=2,merge=merge)

    if(saveSettings){
      print("start to create .settings file")
      settings()
    }

##020808: Produces the .ccl, .ARS and .PA files        
    print("start to create the .ccl, .ARS or/and .PA file") 
    peakOut.file(quantify.dat,read.dat,paste(fullFolderName,outputName,sep=""))    
        
    print("finished")
    setwd(wd)
                          #reset the working directory back to the
                          # previous one

####....NEW ADDITION 201005 TCS (Included the remove the constant that made the intensity values nonnegative)
####...261110: Using reduct.dat instead  of loess.dat for sample
####...021210: For those without Background, return loess.dat as reduct.dat.

    return(list(read.dat = read.dat[[1]],
                clin.dat = read.dat[[2]], 
                sample = reduct.dat$sample,
                reduct.dat = reduct.dat,
                loess.dat = reduct.dat, 
                mlm.dat = mlm.dat,
                modF.dat = modF.dat, 
                modFcorrFdr.dat = modFcorrFdr.dat,
                zoom.dat = zoom.dat,
                quantify.dat=quantify.dat,
                region = region,
                folder = fullFolderName,
                smoother = smoother, 
                signLevel = signLevel))    
    
  }

}#// findPeaks

