##NOTE(26/11/2010):
##   1. Remove the browser().


"locate.minmax" <-
function(max.spect,mz.limit=0.05,symm.limit=1.2,neigh=1,warning.path,cluster.id,idRef){

  ##NEW: point.criterion is used
  ##NOTE: 'max' here means peak while 'min' means valley
  
  ##Convert the percentage to a proportion
  mz.limit<-mz.limit/100
  ##diff1.intensity: A vector of the first difference
  diff1.intensity<-diff(max.spect[,2],lag=1,differences=1)
  
  ##==The sign of the slope (1=positive, 0=negative)
  sign.diff1<-diff1.intensity>0

  ##==The difference of sign of slope (-1=local max, 1=local min)
  diff.sign<-diff(sign.diff1,lag=1,differences=1)
  
  ##Using the median absolute value of the first differences to define noise
  ##This is needed in step 2
  noise<-median(abs(diff1.intensity))
  
  ##==Identify the places where max and min occur
  if(length(which(diff.sign==-1))>0){##At least 1 local max
    max.tmp<-which(diff.sign==-1)+1
  }else{##No local max. Let the largest point be a peak and
    ##the smallest point be a valley then exit from the program.

    ##Find the maximum point
    max.tmp<-which.max(max.spect[,2])
    x.max<-c(max.tmp,1,max.spect[max.tmp,1],max.spect[max.tmp,2],NA,NA)
    ##Find the minimum point
    min.tmp<-which.min(max.spect[,2])
    x.min<-c(min.tmp,0,max.spect[min.tmp,1],max.spect[min.tmp,2],NA,NA)
    ##Combine the two into 1 matrix
    x<-rbind(x.max,x.min)
    colnames(x)<-c("id","max","mz","intensity","noise","noise.ratio")
    ##Sort the rows by mz
    x<-x[order(x[,"mz"],decreasing = FALSE),]
    
    ##Output a warning to the file
    write(paste("locate.minmax(Step 1)","cluster.id:",cluster.id,"spectrum.id:",idRef),
          file=paste(warning.path,"warning.txt",sep=""),append=TRUE)
    print(paste("warning: Maximum and minimum could not be detected for cluster id ",
                cluster.id," by the function locate.minmax() at step 1",sep=""))
    return(as.data.frame(x))
  }
  
  if(length(which(diff.sign==1))>0){##At least 1 local min
    min.tmp<-which(diff.sign==1)+1
  }else{##No local min
    min.tmp<-NA
  }
  
  ##==A maxtrix containing location of the min and max 
  minmax<-rbind(cbind(max.tmp,1),cbind(min.tmp,0))
  ominmax<-order(minmax[,1])
  minmax<-minmax[ominmax,]
  
  x<-cbind(minmax,max.spect[minmax[,1],],noise)
  colnames(x)<-c("id","max","mz","intensity","noise")
  
  ##==Create artifical min points for max points at the start and end
  ##Remove the NA from the x (due to no valley detected)
  x<-x[!is.na(x[,1]),]
  ##Identify the max through the x matrix
  if(is.null(nrow(x))){##When there is only max point
    x.max<-which(x["max"]==1)
  }else{##When there is multiple max point
    x.max<-which(x[,"max"]==1)
  }
  
  ##==If the max is at the start
  ##  noise column for the fake min is given zero
  if(any(x.max%in%1)){
    x.max.start<-c(1,0,max.spect[1,],0)
  }else{
    x.max.start<-NULL
  }
  
  ##==If the  max is at the end
  ##  noise column for the fake min is given zero
  if(any(x.max%in%nrow(x)) | is.null(nrow(x))){
    x.max.end<-c(nrow(max.spect),0,max.spect[nrow(max.spect),],0)
  }else{
    x.max.end<-NULL
  }
  
  ##Including the start/end points with artifical min
  x<-rbind(x.max.start,x,x.max.end)
  colnames(x)<-c("id","max","mz","intensity","noise")
  rownames(x) <- NULL
  
  ##STEP 2
  ##2. Using the median absolute value of the first differences
  ## to define noise, eliminate all local maxima whose distance
  ## to the nearest local minimum is less than the noise.
  ##2b.Those max points near each other and share the same min point
  ## but are below the noise, the min point is dropped instead and
  ## the max of the max point is kept.
  
  ##==identify the points that are max from the x matrix
  max.i<-which(x[,"max"]==1)
  
  if(all(diff(max.i))){##between a max point is a min point
    ##Absolute difference in the m/z
    diff.mz<-abs(diff(x[,"mz"],lag=1,differences=1))
    
    ##=================================##
    ##Identify max with its nearest min##
    ##=================================##
    
    ##The diff.mz.mat:
    ## the cols correspond to the interior max,
    ## 1st row: abs diff of min m/z (left of max) with max m/z
    ## 2nd row: abs diff of min m/z (right of max) with max m/z
    ## For the ith max point, the (i-1)th diff.mz is the left slope
    ## while the ith diff.mz is the right slope.
    ## Those fake min point are removed from the evaluation  
    diff.mz.mat<-matrix(diff.mz,nrow=2,byrow=FALSE)
    
    ##==Indicator which of the min m/z is nearest to the max m/z
    ## Following the diff.mz.mat format
    diff.mz.min<-rbind(diff.mz.mat[1,]<=diff.mz.mat[2,],diff.mz.mat[1,]>diff.mz.mat[2,])
    ##Identify which of the m/z is nearest to the max m/z
    mz.min<-c(1:length(diff.mz))[c(diff.mz.min)]
    ##Identify the max (1st col) corresponding to its nearest min (2nd col)
    ##Note:
    ##-If the mz.min[i] is odd, then the location of the nearest min is to the left
    ## and that is the location of the min in the x matrix.
    ##-If the mz.min[i] is even, then the location of the nearest min is to the right
    ## and the location of the min in the x matrix is mz.min[i]+1
    max.w.min<-cbind(max.i,ifelse(mz.min%%2==1,mz.min,mz.min+1))
    
    if(x[1,"noise"]==0 & x[(max(max.i)+1),"noise"]==0 & nrow(x)==3){
      ##If there is only one max point with no natural valleys, then just keep things as it is
    }else{##If the min pt at the ends are fake min replace with the real min
      if(x[1,"noise"]==0){##If fake min is at the start
        max.w.min[1,2]<-3
      }
      if(x[(max(max.i)+1),"noise"]==0){##If fake min is at the end
        max.w.min[nrow(max.w.min),2]<-(max(max.i)-1)
      }
    }
    
    ##Indicate the max above the noise level
    max.noise.i<-abs(x[max.w.min[,1],"intensity"]-x[max.w.min[,2],"intensity"])>(max(x[,"noise"]))
    
    ##Identify the max points are the below noise
    ## (i.e. max.noise.i is FALSE)
    which.max.noise.i<-which(max.noise.i==FALSE)*2
    
    if(length(which.max.noise.i)==0){##No peaks to remove
      ##The noise ratio
      noise.ratio<-rep(NA,nrow(x))
      noise.ratio[max.i]<-1
      
      x<-cbind(x,noise.ratio)
      x.2<-data.frame(x,row.names = NULL)
    }else{##Some peaks to remove or some min pt to remove
      
      ##================================================================##
      ##==Check which valleys have two peaks nearest to them. If these==##
      ##==two peaks are very close, merge them.                       ==##
      ##================================================================##
      
      ##==Identify those min point that share the same max point
      ##Check that the min point are sorted and should be
      if(is.unsorted(max.w.min[,2])){
        stop("the min points are unsorted: Not possible")
      }
      
      ##==Identify the second occurence of dulipcate min point (dup.min2).
      ##== After which identify the 1st min point (dup.min1).
      dup.min2<-which(duplicated(max.w.min[,2]))
      dup.min1<-dup.min2-1
        
      ##==========================================================================
      ##==Identify those min point to drop and the max point to keep after merging
      ##==========================================================================
      no.dup<-length(dup.min2)
      
      if(no.dup!=0){##At least there is 1 duplicate min point
        
        iterDup<-0
        merge.max<-NULL
        while((length(which.max.noise.i)==length(max.noise.i) & iterDup==1 & is.null(merge.max)) | iterDup==0){
          
          if(no.dup==1){##Exactly 1 duplicate min point
            ##Identify the min point to remove from the x matrix
            ## (i.e. the intensity difference between the max & common min is < mz.limit & both slopes <noise)
            ##and the max to keep if the two max are close

            if(iterDup==0){
              nearTF<-(diff(x[c(2*dup.min1,2*dup.min2),"mz"])/x[2*dup.min1,"mz"]<mz.limit)
            }else{
              nearTF<-(2*(dup.min2-dup.min1)<3)
            }
            
            if(nearTF & sum(max.noise.i[dup.min1:dup.min2])==0){
              ##The max pts with common min pt within some small distance and below the noise level.
              ##Thus need to drop a min pt instead and merge the max point.
              keep.max<-c(2*dup.min1,2*dup.min2)[which.max(x[c(2*dup.min1,2*dup.min2),"intensity"])]
              ##The min point to drop, the max point to keep and whether to drop it
              merge.max<-c(max.w.min[dup.min1,2],keep.max)
            }else{
              merge.max<-NULL
            }
          
          }else{##More than 1 duplicate min point
            
            merge.max<-NULL
            for(i in 1:no.dup){
              
              if(iterDup==0){
                nearTF<-(diff(x[c(2*dup.min1[i],2*dup.min2[i]),"mz"])/x[2*dup.min1[i],"mz"]<mz.limit)
              }else{
                nearTF<-2*(dup.min2[i]-dup.min1[i])<3
              }
              
              ##Identify the min point to remove and the max to keep if the two max are close
              if(nearTF & sum(max.noise.i[dup.min1[i]:dup.min2[i]])==0){
                ##The max pts with common min pt within some small distance and below the noise level.
                ##Thus need to drop a min pt instead and merge the max point.
                keep.max<-c(2*dup.min1[i],2*dup.min2[i])[which.max(x[c(2*dup.min1[i],2*dup.min2[i]),"intensity"])]
                ##The min point to drop, the max point to keep and whether to drop it
                merge.max<-rbind(merge.max,c(max.w.min[dup.min1[i],2],keep.max))
              
              }
            }
          }
          
          ##Put increase the iterative duplication indice
          iterDup<-iterDup+1
          
        }##End of while loop
        
      }else{##No duplicate min point
        
        merge.max<-NULL
        
      }
      
     
      ##==Identify the max points to remove and the min point to remove
      if(is.matrix(merge.max)){##If there are lots of max point & min point to remove together
        ##The max point to keep
        keep.max<-which.max.noise.i%in%merge.max[,2]
        ##The max point to remove
        if(any(!keep.max)){
          rm.max<-which.max.noise.i[!keep.max]
        }else{
          rm.max<-NULL}
        ##the min point to remove
        rm.min<-merge.max[,1]
      }else{##If there is potentially 1 max point & 1 min point to remove
        if(!is.null(merge.max)){##1 max point & 1 min point to remove
          ##The max point to keep
          keep.max<-which.max.noise.i%in%merge.max[2]
          ##The max point to remove
          if(any(!keep.max)){
            rm.max<-which.max.noise.i[!keep.max]
          }else{
            rm.max<-NULL}
          ##the min point to remove
          rm.min<-merge.max[1]
        }else{##only have max to remove
          rm.max<-which.max.noise.i
          rm.min<-NULL
        }
      }
      
     
      ##Remove the peaks or valleys
      if(is.null(rm.max) & is.null(rm.min)){##No max & min to remove
        ##Create the noise ratio
        noise.ratio<-rep(NA,nrow(x))
        noise.ratio[max.i]<-1
        x.2<-data.frame(cbind(x,noise.ratio),row.names = NULL)
        
      }else{##There are max & min to remove
        ##Create the noise ratio
        noise.ratio<-rep(NA,nrow(x))
        noise.ratio[max.i]<-1
        
        x.2<-x[-c(rm.max,rm.min),]
        x.2<-cbind(x.2,noise.ratio=noise.ratio[-c(rm.max,rm.min)])
        x.2<-data.frame(x.2,row.names = NULL)
        
        if(all(x.2[,"max"]==0)){
          ##If x.2 is empty or no maximum (This prevents no peak returned)
          
          if(nrow(x)>=3){##The original x without the noise criterion has a proper peak

              
            ##Output a warning to the file
            write(paste("locate.minmax(Step2): Returned peak and valley in Step1.",
                        "cluster.id:",cluster.id,"spectrum.id:",idRef),
                  file=paste(warning.path,"warning.txt",sep=""),append=TRUE)
            print(paste("warning: Maximum could not pass noise criterion for cluster id ",
                        cluster.id," by the function locate.minmax() at step 2",sep=""))
            
            ##Return the unchanged x with the noise ratio (determine the quality of the template peaks)
            noise.ratio<-rep(NA,nrow(x))
            noise.ratio[max.i]<-abs(x[max.w.min[,1],"intensity"]-x[max.w.min[,2],"intensity"])/(max(x[,"noise"]))
            return(as.data.frame(cbind(x,noise.ratio))) 
            
          }else{##The original x does not have a proper peak therefore return the max and min point
            
            ##Output a warning to the file
            write(paste("locate.minmax(Step2): Returned max and min.",
                        "cluster.id:",cluster.id,"spectrum.id:",idRef),
                  file=paste(warning.path,"warning.txt",sep=""),append=TRUE)
            print(paste("warning: Maximum could not get a proper peak for cluster id ",
                        cluster.id," by the function locate.minmax() at step 2",sep=""))
            
            ##Find the maximum point
            max.tmp<-which.max(max.spect[,2])
            x.max<-c(max.tmp,1,max.spect[max.tmp,1],max.spect[max.tmp,2],NA,NA)
            ##Find the minimum point
            min.tmp<-which.min(max.spect[,2])
            x.min<-c(min.tmp,0,max.spect[min.tmp,1],max.spect[min.tmp,2],NA,NA)
            ##Combine the two into 1 matrix
            x.2<-rbind(x.max,x.min)
            colnames(x.2)<-c("id","max","mz","intensity","noise","noise.ratio")
            ##Sort the rows by mz
            x.2<-x.2[order(x.2[,"mz"],decreasing = FALSE),]
            return(as.data.frame(x.2))
          }    
        }
      }
    }
    
    
    ##STEP 3
    ##3. Combine local maxima that are separated by fewer
    ## than T clock ticks (or time intervals; the default value is
    ## T  3) or by less than M relative mass units (default
    ## value of M is 0.05% of the smaller mass). Retain the
    ## highest local maximum when combining nearby peaks.
    
    ##Initalizing the while loop
    max.noise<-subset(x.2,max==1)
    x.3<-data.frame(x.2,row.names = NULL)
    
   
    if(nrow(max.noise)>1){##When there are more than 1 max above noise level
      ##Indicate those whose max m/z are within 0.05%
      diff.mz<-abs(diff(max.noise[,"mz"],lag=1,differences=1))/max.noise[1:(nrow(max.noise)-1),"mz"]<mz.limit
      
      while(any(diff.mz)){##When there is at least 1 pair of max within 0.05%
        ##No of pairs of max that are with in 0.05%
        no.pair<-sum(diff.mz)
        
        if(no.pair==1){##Only 1 pair is within 0.05%
          pair.mz<-which(diff.mz)+c(0,1)
          ##Identify the max to remove
          rm.max.p<-which.min(max.noise[pair.mz,"intensity"])
          rm.max.p<-max.noise[pair.mz[rm.max.p],"id"]
        
        }else{##More than 1 pair within 0.05%
          ##the col are the number of pairs and only 2 rows for each pair
          pair.mz<-apply(matrix(which(diff.mz),ncol=1),1,
                         function(x){
                           return(x+c(0,1))
                         })
          
          ##Identify id of the max to remove (within 0.05% and have smaller intensity)
          rm.max.p<-apply(pair.mz,2,
                          function(x,max.noise){
                            ##the intensities of the paired max
                            tmp<-max.noise[x,"intensity"]
                            ##Identify which has the smallest intensity
                            return(c(tmp[1]<=tmp[2],tmp[1]>tmp[2]))
                          },max.noise=max.noise)
          
          pair.mz<-c(pair.mz)
          rm.max.p<-unique(max.noise[pair.mz[c(rm.max.p)],"id"])
          
        }
        
        ##Remove max that are within 0.05% and have smaller intensity
        x.3<-subset(x.3,id%in%rm.max.p==FALSE)
        
        ##==Variables needed to determine whether to do a loop
        max.noise<-subset(x.3,max==1)
        ##Indicate those whose max m/z are within 0.05%
        diff.mz<-abs(diff(max.noise[,"mz"],lag=1,differences=1))/max.noise[1:(nrow(max.noise)-1),"mz"]<mz.limit
      }
      
    }else{#when there is only 1 max

      x.3<-data.frame(x.2,row.names = NULL)
    }

    ##STEP 4
    ##4a.Refine the locations of the nearest local minima to both
    ##   the left and right of each of local maximum. Because of
    ##   flat regions, the right-hand minimum for one peak may
    ##   be at a different location than the left-hand minimum
    ##   of the next peak.
    ##4b.Ensure there there is symmetry in the valleys selected.
    ##   First by selecting the valley with the least signal.
    ##   Second by selecting a neighbouring valley for the valley with the least signal.
    ##   Third check that the intensity are reasonable too.
    
    ##==Identify the appropriate min and max from x.3
    diff.max<-diff(x.3[,"max"],lag=1,differences=1)
    if(any(diff.max==0)){##there is at least 1 excess min (make sense to search for a better peak)
      ##Identify the rows of x.3 that are the max with nearest min point
      max.x.3<-which(x.3[,"max"]==1)
      min.x.3L<-max.x.3-1
      min.x.3R<-max.x.3+1

      ##Difference in intensity, top row is the difference in intensity w.r.t to the left of the peak
      ##and the bottom row is the difference in intensity w.r.t the right of the peak.
      diff.int<-rbind((x.3[max.x.3,"intensity"]-x.3[min.x.3L,"intensity"]),
                      (x.3[max.x.3,"intensity"]-x.3[min.x.3R,"intensity"]))
      
      ##Check point for programmer
      if(!all(diff.int>0))
        stop("somthing wrong in Step4 with the difference of intensity")
      
      ##Identify points that have weaker signal and search for a better min if possible
      minStrong<-(diff.int[1,]>diff.int[2,])*min.x.3L+(diff.int[2,]>diff.int[1,])*min.x.3R
      minWeak<-(diff.int[1,]<diff.int[2,])*min.x.3L+(diff.int[2,]<diff.int[1,])*min.x.3R
      
      ##Identify the rows of x.3 that have extra min point
      extra.min<-which(diff.max==0)
      extra.min<-sort(unique(c((extra.min+1),extra.min)))
      
      ##Indicate potential extra min to consider (because it is at extra.min+1or-1)
      extra.i<-minWeak%in%extra.min

      if(length(extra.i)==0){##no potential extra min to improve symmetry for those min with weak signal
        ##Hence the nearest min are the the only alternative
        x.4<-data.frame(x.3[sort(unique(c(min.x.3L,max.x.3,min.x.3R))),],check.names = FALSE, row.names = NULL)
      }else{##there are extra min that could improve symmetry for those min with weak signal
        
        ##Shortlisted max (1st row) with extra min and its corresponding nearest min with weak signal (2nd row) and nearest strong signal (3rd row)
        shortlist<-rbind(max.x.3[extra.i], minWeak[extra.i], minStrong[extra.i])
        
        ##==========================================================================================
        ##Among the shortlist, check which do not fufill the symmetric by intensity criterion (main)
        ##but compute the m/z criterion (secondary) too.
        ##==========================================================================================
        ##intensity (max-min weak) and (max-min strong)
        diff.int<-rbind((x.3[shortlist[1,],"intensity"]-x.3[shortlist[2,],"intensity"]),
                        (x.3[shortlist[1,],"intensity"]-x.3[shortlist[3,],"intensity"]))
        ##m/z
        diff.mz<-rbind(abs(x.3[shortlist[1,],"mz"]-x.3[shortlist[2,],"mz"]),
                       abs(x.3[shortlist[1,],"mz"]-x.3[shortlist[3,],"mz"]))

        ##The one with max intensity difference is the numerator and the min the denominator
        diff.intRatio<-apply(diff.int,2,"max")/apply(diff.int,2,"min")
        
        if(any(diff.intRatio<0)){
          stop("THere is something wrong with the intensity criterion in step 4")
        }
        ##Indicate which max points do not satisfy the symmetry criterion
        not.symm<-(diff.intRatio>symm.limit)
        
        ##The one with max m/z difference is the numerator and the min the denominator
        diff.mzRatio<-apply(diff.mz,2,"max")/apply(diff.mz,2,"min")


        if(all(!not.symm)){
          ##All the max points satisfy the symmetry criterion and do not have excess min
          ##Hence the nearest min are the the only alternative
          x.4<-data.frame(x.3[sort(unique(c(min.x.3L,max.x.3,min.x.3R))),],check.names = FALSE, row.names = NULL)
        }else{##Some of the max points do not satisfy the symmetry criterion and have excess min
####...261110: Removing browser()	  		
#browser()
          ##Find the rows of x.3 of the extra minmium of those that do not satisfy the symmetry criterion
          if(sum(not.symm)>1){##more than 1 max pt do not satisfy the symmetry criterion
            extraList<-apply(shortlist[,not.symm],2,
                             function(x,extra.min,neigh){
                               ##Find extra min pt that are within some neighbourhood of a weak signal that did
                               ##not pass the symmetry criterion.
                               tmp<-extra.min[abs(extra.min-x[2])<=neigh]
                               ##Remove those extra min that are the weak signal
                               return(tmp[!tmp%in%x[2]])
                             },extra.min=extra.min,neigh=neigh)
            ##A row vector telling us which extra min pt to try
            extraList<-matrix(extraList,nrow=1)
            
          }else{##exactly 1 max pt does not satisfy the symmetry criterion
            ##Find extra min pt that are within some neighbourhood of a weak signal that did
            ##not pass the symmetry criterion.
            tmp<-extra.min[abs(extra.min-shortlist[2,not.symm])<=neigh]
            extraList<-matrix(tmp[!tmp%in%shortlist[2,1]],ncol=1)
          }
          
          ##====================================
          ##USing the intensity criterion (main)
          ##====================================
          ##Compute the difference in intensity for the extra min with the max.
          diff.intReplace<-apply(matrix(1:ncol(extraList),nrow=1),2,
                                 function(i,x.3,extraList,max.x.3,extra.i,not.symm){
                                   x.3[max.x.3[extra.i],"intensity"][not.symm][i]-x.3[extraList[,i],"intensity"]
                                 },x.3=x.3,extraList=extraList,max.x.3=max.x.3,extra.i=extra.i,not.symm=not.symm)


          ##Top row is the difference in intensity for the excess min (max-excess min) and
          ##the bottom row is the strong signal min (max-strong min)
          diff.intReplace<-rbind(diff.intReplace,
                                 x.3[shortlist[1,][not.symm],"intensity"]-x.3[shortlist[3,][not.symm],"intensity"])
                                 

          ##The ratio of the intensity(max of the difference in intensity is in the numerator and
          ##the min in the denominator)
          diff.intReplaceRatio<-apply(diff.intReplace,2,"max")/apply(diff.intReplace,2,"min")
          
          ##Indicator of those whose symmtry has improved
          improve.symm<-(diff.intReplaceRatio<diff.intRatio[not.symm])


          ##The sign of the difference in intensity should be the same since max-excess min and max-strong min          
          diff.intSignTest<-apply(diff.intReplace<0,2,"any")
          
          ##===================================
          ##USing the m/z criterion (Secondary)
          ##===================================
          ##Compute the difference in mz for the extra min with the max.
          diff.mzReplace<-apply(matrix(1:ncol(extraList),nrow=1),2,
                                function(i,x.3,extraList,max.x.3,extra.i,not.symm){
                                  abs(x.3[extraList[,i],"mz"]-x.3[max.x.3[extra.i],"mz"][not.symm][i])
                                },x.3=x.3,extraList=extraList,max.x.3=max.x.3,extra.i=extra.i,not.symm=not.symm)

          ##Combine the difference in mz for the excess min and the strong signal min
          diff.mzReplace<-rbind(diff.mzReplace,diff.mz[2,not.symm])

          ##Check the ratio
          diff.mzReplaceRatio<-apply(diff.mzReplace,2,"max")/apply(diff.mzReplace,2,"min")

          ##Indicator of those that are symmtric under the mz criterion
          symm.mz<-(diff.mzReplaceRatio<=symm.limit)

          ##Indicator of those whose symmtry has improved but not symmetric under the  mz criterion
          improve.symm.mz<-(diff.mzReplaceRatio<diff.mzRatio[not.symm])

          ##Indicator of those that are more symmtric under the intensity or m/z criterion
          symmFinal<-(((diff.intReplaceRatio<=symm.limit | improve.symm) | symm.mz | improve.symm.mz)
                      & !diff.intSignTest)
          
          ##Indicator of which weak min to remove
          final.replace<-symmFinal

          ##Initialize the miWeakNew
          minWeakNew<-minWeak
          ##Where there exist 1 weak signal to replace
          if(!all(final.replace==FALSE)){  
            minWeakNew[extra.i][not.symm][final.replace]<-extraList[,final.replace]
          }
          
          ##==To update the rows that improve symmetry
          ##Get the range of the row indices of the left and right valleys
          valid.id<-apply(cbind(minWeakNew,minStrong),1,"range")
          ##Include the max in the middle row.
          valid.id<-c(rbind(valid.id[1,],max.x.3,valid.id[2,]))
          ##Remove duplicated row indices
          valid.id<-valid.id[!duplicated(valid.id)]
          
          ##Get the data with improved symmetry
          x.4<-data.frame(x.3[valid.id,],check.names = FALSE, row.names = NULL)
        }
      }
    }else{##There is no excess min points
      x.4<-x.3
    }
    

    ##STEP 5
    ##5. Compute the slopes from the left-hand local minimum
    ##up to the local maximum and from the local maximum
    ##down to the right-hand local minimum. Eliminate
    ##peaks where both slopes are less than half the value of
    ##the noise.
    
    ##Identify the max points
    max.x4.i<-which(x.4[,"max"]==1)
    
    ##Identify the left(1st row) and right(3rd row) of the max(2nd row)
    slope.i<-rbind(max.x4.i-1,max.x4.i,max.x4.i+1)

    ##==indicate peaks whose slope are less than half the value of the noise
    peak.rm<-apply(slope.i,2,
                   function(s,x.4){
                     ##The slope of the left and right of the peak
                     slope.tmp<-diff(x.4[s,"intensity"],lag=1,differences=1)/diff(x.4[s,"mz"],lag=1,differences=1)
                     ##peaks where both slopes are less than half the value of the noise
                     return(all(abs(slope.tmp)<(max(x.4[1,"noise"])/2)))
                   },x.4=x.4)

    ##Peaks identified by the simple method
    if(any(peak.rm)){##at least 1 peak has both slope less than half the noise
      x.5<-data.frame(x.4[-max.x4.i[peak.rm],],check.names=FALSE)
      
      
      if(all(x.5[,"max"]==0)){##If all the max points are removed, return x.4 instead
        ##Do not remove the max points & let it pass
        ##Output a warning to the file
        write(paste("locate.minmax(step5). Returned Step4 instead.","cluster.id:",cluster.id,"spectrum.id:",idRef),
              file=paste(warning.path,"warning.txt",sep=""),append=TRUE)
        print(paste("warning: Maximum and minimum could not be detected for cluster id ",
                    cluster.id," by the function locate.minmax() at step 5.",sep=""))
        return(x.4)
      }else{##Not all the max points are removed
        ##==Identify the appropriate min and max from x.5
        diff.max<-diff(x.5[,"max"],lag=1,differences=1)
        
        if(any(diff.max==0)){##there is at least 1 excess min
          ##Identify the rows that are valid
          valid<-which(diff.max!=0)
          valid<-sort(unique(c((valid+1),valid)))
          x.5<-data.frame(x.5[valid,],check.names = FALSE)
        }
      }
      
    }else{##there is no excess min
      x.5<-x.4
    }
    
    return(x.5)
  }else{
    stop("between two max, there is no min point")
  }
  
}

