"mlm.ProSpect" <-
function(loessProSpect,num.points=10) {
   
  #!!!NOTE THAT THE WAY IT IS DONE IS WITH REFERENCE TO THE POINTS
  # (order of TOF) AND NOT THE MZ VALUES.
  
  #Initializing the F-stat, its p-value, MSE(Mean Square Error)
  # for each return value of lm.ProSpect() during the iteration.
  statF<-NULL;
  pValueF<-NULL;
  mseF<-NULL;

  loess.dat<-loessProSpect$loess.dat
                          #extracting the array that contains the
                          # intensities and mz

  # The summary statistics of each iteration's mz and intensity
  #  (useful for identifying regions that are significant)
  mz<-NULL;
  max.intensity<-NULL

  max.window<-ceiling(nrow(loess.dat)/num.points)
                          #nrow(loess.dat): number of rows in 3dim
                          # array loess.dat
                          #max.window: the max number of possible
                          # windows

  # quality check: To avoid that a window with only one mz value per
  #  spectrum exists it is better to use (max.window-1) instead of
  #  max.windows.
  if ( (max.window-1)*num.points+1 ==  nrow(loess.dat) )
    usedCount.window <- max.window-1
  else 
    usedCount.window <- max.window
  
  #===========================================
  # Doing one-way ANOVA to a sequential and 
  # non-overlapping windows of proteomics data
  #===========================================

  #*******************************************************************
  #Creating of optimized anova
  #===========================
  #
  #What the function does: It does a one-way ANOVA on a balance
  # design experiment, i.e. each factor level has the same number
  # of observations
  #Y is the outcome matrix
  #X is the corresponding design matrix: with intercept and a factor
  # level as reference
  #NOTE: THIS FUNCTION IS SELF CONTAINED
  
  optim.lm<-function(Y,X){

    dim.X<-dim(X)
    num.level<-dim.X[2]
    num.obs<-dim.X[1]/dim.X[2]
                          #The number of factor levels (i.e. number
                          # of spectra) and the number of observations
                          # for each factor level    
                                        
    hat.matrix<-solve(t(X)%*%X)%*%t(X)
                          #The hat matrix: (X'X)X'

    beta<-hat.matrix%*%Y
                          #The estimates of beta

                      
    Y.bar<-t(Y)%*%matrix(1,ncol=1,nrow=(num.obs*num.level))/
           (num.obs*num.level)
                          #The overall mean for each window
    
    #To compute the MSE: b'X'Xb
    BXXB<-apply(beta,2,
                function(beta.hat,XX){
                  beta.hat<-matrix(beta.hat,ncol=1)
                  return(t(beta.hat)%*%XX%*%beta.hat)
                },XX=(t(X)%*%X))
    df1<-(num.level-1)
    MSR<-c((BXXB-(num.obs*num.level)*(Y.bar^2))/df1)

    #To compute MSE
    residual<-Y-X%*%beta
    df2<-(num.obs*num.level-num.level)
    MSE<-c(colSums(residual*residual)/df2)
  
    F.stat<-MSR/MSE

    optim.lm.out<-cbind(F.stat,rep(df1,times=length(MSR)),
                        rep(df2,times=length(MSE)),
                        pf(F.stat,df1=df1,df2=df2,lower.tail=FALSE),
                        MSE,MSR)
    colnames(optim.lm.out)<-c("statF","F.df1","F.df2","pValueF",
                              "mseF","msrF")
    return(optim.lm.out)
  }
#  
#*******************************************************************
 
  #=================================
  #For the first (usedCount.window-1)th windows
  #=================================
  #For testing purposes
  #usedCount.window<-20
  #loess.dat<-loessProSpect$loess.dat[1:(usedCount.window*num.points),,]
  #num.points<-5
  
  num.spectra<-ncol(loess.dat)
                          #number of factor levels (number of spectra)
    
  range.row<-matrix(1:((usedCount.window-1)*num.points),
                    ncol=num.points,byrow=TRUE)
                          #The rows are the windows and its row is the
                          # row index of loess.dat that correspond to the
                          # window
  
  X<-cbind(rep(1,times=num.points*num.spectra),
           apply(diag(1,nrow=(num.spectra-1),ncol=num.spectra),1,
                 "rep",each=num.points))
                          #The design design matrix (X): There is an
                          # intercept with the last spectrum as the
                          # reference spectrum
 
  Y<-apply(range.row,1,
           function(row.index,intensity){
             return(c(intensity[row.index,]))
           },intensity=loess.dat[,,2])
                          #The outcome vectors
                          # Y=cbind(Y_1,Y_2,...,Y_{usedCount.window-1})
  
  mlmF.dat<-optim.lm(Y,X)
                          #A matrix where the rows corresponds to the
                          # windows and the columns correspond to
                          #F-statistics, df1, df2, p-value, MSE and MSR
  

  MZ<-apply(range.row,1,
            function(row.index,mz.matrix){
              return(c(mz.matrix[row.index,]))
            },mz.matrix=loess.dat[,,1])
                          #The m/z vectors
                          # MZ=cbind(MZ_1,MZ_2,...,MZ_{usedCount.window-1}).
                          #NOTE it is the same as for Y
  
  mz<-cbind(colMeans(MZ),sd(MZ),
            t(apply(MZ,2,"range")))
                          #A matrix consisting of the various
                          # statistics (col: mean, ) of the m/z in
                          # each window (rows)
  
  #==========================================
  #For the last window i.e. (usedCount.window)th windows
  #==========================================
  
  range.row<-matrix(c(((usedCount.window-1)*num.points+1):(dim(loess.dat)[1])),
                    nrow=1,byrow=TRUE)
                          #The row index of loess.dat that correspond to
                          # the last window with 1 row and  col
  
  #The design matrix (X): There is an intercept with the last spectrum as the reference spectrum

  num.points.last<-ncol(range.row)
                          #number of points in the last window for each spectrum
  X<-cbind(rep(1,times=(num.spectra*num.points.last)),
           apply(diag(1,nrow=(num.spectra-1),ncol=num.spectra),1,
                 "rep",each=num.points.last))
  Y<-apply(range.row,1,
           function(row.index,intensity){
             return(c(intensity[row.index,]))
           },intensity=loess.dat[,,2])
                          #The outcome vector Y=Y_{usedCount.window}
  mlmF.dat<-rbind(mlmF.dat,optim.lm(Y,X))
                          #A matrix where the rows corresponds to the
                          # windows and the columns correspond to
                          #F-statistics, df1, df2, p-value, MSE and MSR
  MZ<-apply(range.row,1,
            function(row.index,mz.matrix){
              return(c(mz.matrix[row.index,]))
            },mz.matrix=loess.dat[,,1])
                          #The m/z vector MZ=MZ_{usedCount.window}.
                          #NOTE it is the same as for Y
  mz<-rbind(mz,c(mean(MZ),sd(MZ),range(MZ)))
                          #A matrix consisting of the various
                          # statistics (col: mean, ) of the m/z
                          # in each window (rows)

  #===================================
  #Giving col/row names to the outputs
  #===================================
    
  dimnames(mlmF.dat)<-list(c(1:usedCount.window),
                         c("statF","F.df1","F.df2","pValueF",
                           "mseF","msrF"))
                          #the statistics related to the 1 way ANOVA

  dimnames(mz)<-list(c(1:usedCount.window),
                     c("mean.mz","sd.mz","min.mz","max.mz"))
                          #the various statistics for the mz

  return(list(mlmF.dat=mlmF.dat, mz=mz,num.points=num.points,
              expt=loessProSpect$expt,sample=loessProSpect$sample,
              range=loessProSpect$range))
                          #NOTE that only loess.ProSpect will have a
                          # range

  #Output of mlm.ProSpect:
  #It outputs a list with
  #-mlmF.dat is a matrix with information on the one-way ANOVA. The
  # row corresponds to the window sequence while the columns are
  # F-statistics(statF), its numerators df (F.df1), denominator
  # df (F.df2), p.value (pValueF), MSE (mseF) and MSR (msrF)
  #-mz is a matrix with information on each window's m/z summary
  # statistics. The row corresponds to the window sequence while the
  # columns are the mean mz (mean.mz), sd mz (sd.mz), min mz
  # (min.mz) and max mz (max.mz).
  #-max.intensity is a matrix with information on each window's
  # maximum intensity mz (1st col)and maximum intensity value
  # (2nd col).The row corresponds to the window sequence.
  #-num.points is a scalar that tells the number of paired
  # observations (m/z,intensity) for each window's spectrum.
  #-expt is a string that is the name of the experiment
  #-sample is a vector of strings and each entry correspond to the
  # name of a spectrum.
  #-range is a vector that indicate the row values corresponding to
  # the output of read.ProSpect that are used in the analysis
  #          (!!!!only lowess corrected intensities should have this)


  
}

