"mod.F" <-
function(mlmProSpect,B=10,t.prop=.1) {

  #============================================
  #smoothed or the local mean/median of the MSE
  #============================================

  #Note: smooth.mse2.R must be loaded before
  s.mse<-smooth.mse(mlmProSpect,B,t.prop)

  #======================================
  #The F-statistics with the smoothed MSE
  #======================================

  F.msr = mlmProSpect$mlmF.dat[,"msrF"]
                          #Prepartion to get the modified F-statistics  
                          #msr=mean square regression
  df1<-mlmProSpect$mlmF.dat[1,"F.df1"]
                          #The numerator degrees of freedom
  mF.med<-F.msr/s.mse$med
                          #To get the modified F by median run
  mF.arithmetic<-F.msr/s.mse$arithmetic[,"normal.mean"]
                          #To get the modified F by mean run
  mF.arithmetic<-cbind(mF.arithmetic,s.mse$arithmetic[,"satter.df"])
  colnames(mF.arithmetic)<-c("mF.arithmetic","satter.df")
  mF.tukey<-F.msr/s.mse$tukey[,"tukey.mean"]
                          #To get the modified F by robust mean run 
  mF.tukey<-cbind(mF.tukey,s.mse$tukey[,"satter.df"])
  colnames(mF.tukey)<-c("mF.tukey","satter.df")
  mF.trimmed<-F.msr/s.mse$trimmed[,"trimmed.mean"]
                          #To get the modified F by trimmed mean run 
  mF.trimmed<-cbind(mF.trimmed,s.mse$trimmed[,"satter.df"])
  colnames(mF.trimmed)<-c("mF.trimmed","satter.df")
  
  mod.F<-list(median=mF.med,
              arithmetic=mF.arithmetic,
              tukey=mF.tukey,
              trimmed=mF.trimmed,
              df1=df1,
              mz=mlmProSpect$mz,
              trimmed.sMse=s.mse$trimmed,
              tukey.sMse=s.mse$tukey,
              arithmetic.sMse=s.mse$arithmetic,
              med.sMse=s.mse$med,
              num.run.sMse=s.mse$num.run,
              B.sMse=s.mse$B.mse,
              trimmed.propotion.sMse=s.mse$trimmed.propotion.mse,
              expt=mlmProSpect$expt,
              num.points=mlmProSpect$num.points,
              sample=mlmProSpect$sample,
              range=mlmProSpect$range)
  return(mod.F)

  #Output of modified.F:
  #It outputs a list with
  #-$median is a vector of the MSR/local median of MSE
  #-$arithmetic is a matrix with 1st col: MSR/local mean and 2nd col:
  # satter.df
  #-$tukey is a matrix with 1st col: MSR/tukey mean and
  # 2nd col: satter.df
  #-$trimmed is a matrix with 1st col: MSR/local trimmed mean and
  # 2nd col: satter.df
  #-$df1 is the df for the numerator of the modified F-statistics (F')
  #-$mz is a matrix containing all the summary statistics of m/z of
  # the various windows.
  #-$num.run.mse is a vector indicating the number of neighbours used
  # to calculate mean or median
  #-$B.mse is a 2dim vector: 1st entry is the percentage of neighbours
  # and 2nd entry is the actual number of neighbours
  #-$trimmed.proportion.mse is a 2dim vector: 1st entry is the proportion to trim
  # and 2nd entry is the actual number trimed off
  #-$expt is a string that is the name of the experiment
  #-$num.points is a scalar that tells the number of paired
  # observations (m/z,intensity) for each window's spectrum.
  #-$sample is a vector of strings and each entry correspond to the
  # name of a spectrum.
  #-$range is a vector that indicate the row values corresponding to
  # the output of read.prot that are used in the analysis
  
}

