peakInfoIntMz<-function(quantify.dat,read.dat){

  ##==Purpose: To extract the mz and intensity of the peaks via ARS and parametric approach
  ##
  ##==Input:
  ##    -quantify.dat: Object from the function quantifyPeaks().
  ##    -read.dat: Object from the function read.ProSpect().
  ##
  ##==Output:
  ##    A list of arrays $infoARS and $infoPA corresponds to the ARS and parametric approach:
  ##    -array with rows corresponding to peaks, columns correpsonding to spectra and
  ##     the 3rd dimension corresponding to c("mz","intensity"). There converge
  ##     indicate whether the EM-aglorithm has converged and NA if no applicable.
  ##
  ##==Comments:
  ##  080207:- It faciliates the process of exporting peaks information out.
  ##           Used in findPeaks()
  ##
  ##NOTE (12/02/2007)
  ##   1. read.dat is a list and the first component of this object is the intensity and m/z array
  ##      Therefore, in assigning the column names of read.dat to infoARS, use read.dat[[1]].
  
  ##...Gather the mz and intensities of the ARS approach
###12022007: read.dat is a list and not a matrix. Specify read.dat[[1]]
  infoARS<-array(NA,dim=c(dim(quantify.dat$arrayInfo)[1:2],2),
                 dimnames=list(rownames(quantify.dat$arrayInfo),
                   colnames(read.dat[[1]]),
                   c("mz","intensity")))
  infoARS[,,"mz"]<-quantify.dat$arrayInfo[,,"max.mz"]
  infoARS[,,"intensity"]<-quantify.dat$arrayInfo[,,"max.ampMix"]+quantify.dat$arrayInfo[,,"min.int"]
  
  ##...Gather the mz and intensities of the parametric approach
  if(!is.null(quantify.dat$infoPA)){##Check if the parametric approach was computed
    infoPA<-arrayInfoPA(quantify.dat)[,,c("mz","intensity")]
    colnames(infoPA)<-colnames(infoARS)
  }
  
  return(list(infoARS=infoARS,infoPA=infoPA))
  
}
