"pickSpect" <-
function(data.cluster,warning.path,cluster.id,plot.indicator=FALSE){
  
  ##The number of points in each cluster
  no.pt<-nrow(data.cluster)
  ##A QC scalar telling whether to consider additional slopes because of
  ##the number of points in the cluster
  slopeQC<-ifelse(no.pt>10,1,0)
  
  ##================================================================##
  ##Shortlist spectra that are good candidates for template spectrum##
  ##================================================================##
  ##==Compute the difference of intensity range
  amp.inten<-apply(data.cluster[,,2],2,
                   function(x){
                     return(diff(range(x),lag=1,differences=1))
                   })
  
  ##==Indicate the indices that have the max range difference to the smallest
  order.amp<-order(amp.inten,decreasing = TRUE)
  
  ##===============================================================================##
  ##Locate and verify if the spectrum with large difference in intensity difference##
  ##has a large max intensity value that is a good candidate for peak              ##
  ##===============================================================================##
  not.max<-TRUE##The max spectrum max point is not a good candidate.
  not.max.indice<-1##The spextrum number corresponding to the spectrum being tested as a good candidate.
                   ##The first natural choice the one with the largest difference in intensity range.
  
  while(not.max & not.max.indice<=ncol(data.cluster)){
    ##==The spectrum with the max range difference with m/z and intensity
    max.spect<-data.cluster[,order.amp[not.max.indice],]

    ##================================================================##
    ##Locate the max intensity of the spectrum that is not at the edge##
    ##(i.e. last two points in either ends)                           ##
    ##================================================================##
    
    ##==Identify the entry where the max intensity is in the spectrum
    peak.loc<-which.max(max.spect[,2])
    
    ##==Ensure the next largest intensity value are not at the edge of
    ##==the window.
    ##no of points in a spect
    no.pt.spect<-nrow(max.spect)
    ##If the peak location is at the edge look at the ith large intensity
    i<-2
    order.max<-order(max.spect[,2],decreasing=TRUE)
    while(peak.loc%in%c(1:(1+slopeQC),(no.pt.spect-slopeQC),no.pt.spect)){
      peak.loc<-order.max[i]
      i<-i+1
    }
    
####==Plot the template spectrum
if(plot.indicator==TRUE){
  plot(max.spect,ylab="intensity",xlab="m/z",type="l")
  abline(v=max.spect[peak.loc,1])
}

    ##==============================================================##
    ##==Verify if the peak located (peak.loc) is a reasonable peak  ##
    ##==by using first differences between successive time points.  ##
    ##==If the two slopes on its left is positive and the two slopes##
    ##==on its right is negative then it is a good peak.            ##
    ##==============================================================##
    
    ##Create the first difference of the spectrum with max range difference
    diff1.max<-diff(max.spect[,2],lag=1,differences=1)
        
    ##For the ith point of the first difference vector (i.e. diff1.max),
    ##it is difference of the (i-1)th and ith intensity.
    
    ##Indicate those slopes on the left that are positive
    left.slope<-diff1.max[1:(peak.loc-1)]>0
      
    ##Indicate those slopes on the right that are negative
    right.slope<-diff1.max[peak.loc:(no.pt.spect-1)]<0
    
    ##The nearest 2 slopes on the right and left are positive and negative respectively
    right.near<-1:(1+slopeQC)
    left.near<-peak.loc-right.near
    
    ##When the nearest 2 slopes on the left and right are not positive and negative respectively
    no.slope.l<-sum(left.slope[left.near])
    no.slope.r<-sum(right.slope[right.near])

    if((no.slope.l+no.slope.r)<ifelse(slopeQC==0,2,4)){##Seems that the present max point may not be the best
     
      ##More local measure to determine if the peak is reasonable
      ##Need to consider more points
      if(slopeQC==1){##When there is enough points in the spectrum to see if
        ##the additional slope will improve  the situation
        
        i<-3 ##Now consider 3 slopes instead of 2
        
        ##==The side that has 1 slope that does not agreed.
        ##==Increase by another slope to see if it is reasonable
        ##Left: To check if need to increase this side by 1 slope
        if(i<=length(left.slope)){##Enough slopes in the left to increase
          no.slope.li<-ifelse(no.slope.l==2,2,i)
        }else{##Not enough slopes to increase
          no.slope.li<-NULL
        }
        ##Right: To check if need to increase this side by 1 slope
        if(i<=length(right.slope)){##Enough slopes in the right to increase
          no.slope.ri<-ifelse(no.slope.r==2,2,i)
        }else{##Not enough slopes to increase
          no.slope.ri<-NULL
        }
        
        ##==Check if increasing the slope by 1 improves the situation
        if(is.null(no.slope.li) | is.null(no.slope.ri)){##There isn't enough points for both the left and right slopes
          ##Go to the next spectrum with next large intensity difference
          not.max.indice<-not.max.indice+1          
        }else{
          ##Increase the number of points
          left.near2<-(peak.loc-no.slope.li):(peak.loc-1)
          right.near2<-1:no.slope.ri
        
          left.imp<-ifelse(no.slope.l==2,TRUE,sum(left.slope[left.near2])>no.slope.l)
          right.imp<-ifelse(no.slope.r==2,TRUE,sum(right.slope[right.near2])>no.slope.r)
        
          if(left.imp & right.imp){##The spectrum picked has a reasonable peak
            ##This spectrum is selected as the template peak
            not.max<-FALSE
          }else{##The spectrum picked does not have a reasonable peak
            ##Go to the next spectrum
            not.max<-TRUE
            not.max.indice<-not.max.indice+1
          }
        }
      }else{##The sepctrum has too little points and does not have a reasonable peak
        ##Go to the next spectrum
        not.max<-TRUE
        not.max.indice<-not.max.indice+1
      }
      
    }else{##The spectrum picked has a reasonable peak
      ##This spectrum is selected as the template peak
      not.max<-FALSE
    }
  }
  
  
  ##==When the template spectrum is selected
  if(not.max.indice<=ncol(data.cluster)){
    idRef<-order.amp[not.max.indice]
    rankRef<-not.max.indice
  }else{##Since none of the template is selected, use the one with the largest difference in intensity
    idRef<-order.amp[1]
    rankRef<-NA
    ##Output a warning to the file
    write(paste("pickSpect","cluster.id:",cluster.id,"spectrum.id:",NA),
          file=paste(warning.path,"warning.txt",sep=""),append=TRUE)
    print(paste("warning: Template spectrum was not selected for cluster id ",
                cluster.id," by the function pickSpect()",sep=""))
  }
  
  ##Return the spectrum number which has large intensity difference,
  ##has a peak not at the edge of the window with neighbouring left
  ##slope that are positive and right slope negative. If no such
  ##spectrum exist, then just choose the one with the largest
  ##intensity difference.
  return(list(idRef=idRef,rankRef=rankRef))
}

