"plotPeaks" <-
function(dat=NULL,plotWhat=NULL,fileName=NULL,region=NULL,winSizePlot=0.5,grouping=NULL,colour=NULL,device=c("default","pdf"),ask=FALSE)
{

  # 'region' can be set individually -> to get it nice working within
  # tkrplot,if argument is not used, the default dat$region is used 

  #===========================
  # Security checks
  #==================  
  
  if(is.null(dat) | is.null(plotWhat) )
  {
    stop(paste("Function \"plotPeaks\" returns error.","\n",
               "Reason: One of the arguments (dat, plotWhat) is \"NULL\"."
               )
         )
  }
  
  if(plotWhat!="raw" & plotWhat!="BGcorrected" &plotWhat!="rawBG" & plotWhat!="significant")
  {
    stop(paste("Function \"plotPeaks\" returns error.","\n",
               "Reason: Argument plotWhat is not \"raw\", \"BGcorrected\" or\"significant\"."
               )
         )
  }


  #===========================
  # Variables
  #============

  device <- tolower(device)
                          #character vector contains only lower case elements now
                          # Reason: to be not case-sensitive
  
  winSizePlot=winSizePlot*1000 
  if(winSizePlot>(dat$region[2] - dat$region[1]) | winSizePlot<=0)
  {
    stop(paste("please choose a value for winSizePlot which is between",
               "0 and the hole choosen region in kDa"))
  }


#Window popped up
  
#  old.par <- par(no.readonly = TRUE)
                          #all par settings which
                          # could be changed.
                          #if has to be set otherwise pops a empty plot
                          #screen up    

    
  wd <- getwd()
  setwd(dat$folder)
                          #to be sure that exporting files like pdf's are in the right
                          # (experiment) folder    
    


  
  #==============================
  # Preparation for Plotting
  #===========================
    
#  #Security check if GUI is used or not
#  t <- search()
#  noPlotScreen <- "package:ProSpectGUI" %in% t  
#   
#  if(!noPlotScreen){old.par <- par(no.readonly = TRUE)}
                          #all par settings which
                          # could be changed.
                          #if has to be set otherwise pops a empty plot
                          #screen up  
    
#  wd <- getwd()
#  setwd(dat$folder)
                          #to be sure that exporting files like pdf's are in the right
                          # (experiment) folder


  
  if(plotWhat=="raw" || plotWhat=="BGcorrected" || plotWhat=="rawBG")
  {
    if(is.null(dat$loess.dat))
    {
      stop("Error: For this R object was no normalization step used.")
    }
    if("default" %in% device)
    {
#      old.par <- par(no.readonly = TRUE)
#      if(plotWhat=="rawBG")
#      {
#        par(mfrow=c(2,1))
#      }
      rawBG.plot(dat=dat, smoother=dat$smoother, type=plotWhat)
#      par(old.par)
      
    }
    if(!is.null(fileName))
    {
      if("pdf" %in% device)
      {
        name <- paste(fileName,".pdf",sep="")
        pdf(name)
#        if(TRUE){old.par <- par(no.readonly = TRUE)}
#        if(plotWhat=="rawBG")
#        {
#          par(mfrow=c(2,1))
#        }        
        rawBG.plot(dat=dat, smoother=dat$smoother, type=plotWhat)
        dev.off()
      }     
    }  
  }

#  significant.plot <- function(dat=NULL,
#                             winSizePlot=500,
#                             clin.dat=FALSE,                             
#                             grouping=NULL,
#                             colour=NULL  

  if(plotWhat=="significant")
  {
    #Security check:
    if(!is.null(grouping) &&  dat$clin.dat==FALSE)
      {warning("No grouping possible because no clinical data file detected!")
       grouping=NULL
     }

    
    if("default" %in% device)
    {
#      par(oma=c(3,1,0,0),mar=c(0,0,0.5,0),cex.axis=1,ask=ask)
                          #oma = spaces inner circle
                          #mar = spaces outer circle
#      layout(matrix(c(2,2,1,1),2,2,byrow=TRUE), c(1,4), c(1,4), TRUE)
      significant.plot(dat=dat,region=region,winSizePlot=winSizePlot,grouping=grouping,colour=colour,ask=ask)
#      par(old.par)
#      dev.off()
    }
    if(!is.null(fileName))
    {
      if("pdf" %in% device)
      {    
        name <- paste(fileName,".pdf",sep="")
        pdf(name)      
#        par(oma=c(3,1,0,0),mar=c(0,0,0.5,0),cex.axis=1)
#        layout(matrix(c(2,2,1,1),2,2,byrow=TRUE), c(1,4), c(1,4), TRUE)
        significant.plot(dat=dat,region=region,winSizePlot=winSizePlot,grouping=grouping,colour=colour)
#        par(old.par)
        dev.off()
      }
    }
  }

  setwd(wd)
#  on.exit(par(old.par))
}

