##Comments:
##  This function allows for the extension of the windows by
##  1. assuming clusters do not overlap, and,
##  2. extending the windows by using the difference/change in sign
##NOTE: This function is being modifed at 2. at quantifyPeaksPCA2.R.
##      For this case, we have used the difference or change in signs to determine the extension.
##      This function is being modifed from version 3. It now has the restriction on the shift too.
##NOTE (29/12/2006): Include the parametric approach. Included two new arguments to the function,
##                   'parPeak' and 'par.maxiter'. 'parPeak' is logical indicating whether parametric
##                   approach is used, while 'par.maxiter' sets the number of iteration within the
##                   EM-algorithm. It returns an additional component in the list output '$infoPA'.
##NOTE (05/1/2007): Allow printing of warning if convergence in the EM algorithm of the parametric
##                  approach is not met.
##                    
##NOTE (08/2/2007): Need to ensure parPeak=TRUE before returning the list because infoPA is NULL.
##                  If not, when parPeak=FALSE, there will be a problem there.
##
##NOTE (26/2/2007): Note that the last window may have less points than specified. Thus
##                  we included min(.,nrow(data.default))) on top of "." to the R object
##                  "row.id.maxExt".
##
##NOTE (08/2/2007): We hope to check out why the dominant spectra detection is terrible for
##                  cluster 22, 56 and 65
##
##                  Need to execute the following statements:
##                  zoom.dat<-zoom.datAll[[i]];loess.dat<-loess.datAll[[i]];FolderName<-wk.dir
##
##NOTE (14/3/2007): Need to execute the following statements for the folder data 12032007
##                  fullFolderName<-wk.dir
##
##NOTE (03/4/2007): Need to find out why cluster 283 is giving problem in peakPA
##                 
##                  Need to execute the following statements:
##                  loess.dat<-reduct.dat;fullFolderName<-wk.dir
##                  Mysteriously it disappear.
##
##NOTE (03/4/2007): Need to find out why cluster 520 is giving problem in extendDomSpect
##
##                  Need to execute the following statements:
##                  loess.dat<-reduct.dat;fullFolderName<-wk.dir
##
##                  There was a problem because the original cluster region is a dip and not a peak.
##                  Therefore, I remove the stop() function and return the original window
##
##NOTE (06/9/2007): Put the option for wanting the diagnostic plots, using argument "plots".

"quantifyPeaks" <-
function(loess.dat=NULL,zoom.dat=NULL,fullFolderName=NULL,mz.limit=0.05,symm.limit=1.3,
         toler=0.000001,maxiter=20,msetol=0.975,parPeak=TRUE,par.maxiter=100,plots=TRUE){
  
  ##Get the proteomic array
  data.default <- loess.dat$loess.dat
  zoom.dat <- zoom.dat
  folder <- fullFolderName

  ##Create warning and diagnostic directories
  warning.path<-paste(folder,"warning/",sep="")
  dir.create(warning.path,showWarnings = TRUE)
  diagnostic.path<-paste(folder,"diagnostic/",sep="")
  dir.create(diagnostic.path,showWarnings = TRUE)
  
  ##Using merged contingious windows and termed these as clusters
  merge.win.info<-windows.merge(zoom.dat$mz,zoom.dat$id)
  interest.all<-merge.win.info$win.id
  ##No. of points
  no.pt<-zoom.dat$num.points
  ##No of spectra in the experiment
  no.spectra<-ncol(data.default)
  ##This is to collect all the information of the template spectrum's peaks fitted to the other spectra
  collect.Info<-NULL
  ##This is to collect all the information of the template profile's; both mz and intensity
  templateInfo<-list(NULL)
  ##This is to collect all the information of the quality of the peaks
  qualityTemplate<-NULL
  ##This is to collect all the information of the parametric way of getting the estimates
  infoPA<-NULL
  

  
  ##========================================##
  ##Peak quantify algorithm for each cluster##
  ##========================================##
  for(cluster.id in 1:nrow(interest.all)){
    
###...........................................###
###Start: For integrate.test.R: Set cluster.id###
###...........................................###
#  for(cluster.id in 1:100){
#    cluster.id<-520 ##One peak
#    cluster.id<-34 ##Two peak cluster
#    cluster.id<-22    
    
###.........................................###
###End: For integrate.test.R: Set cluster.id###
###.........................................###
    
###26022006(Start): Make changes here
    ##The row indices of the clusters that correspond to the intensities position
    row.id<-((interest.all[cluster.id,1]-1)*no.pt+1):min((interest.all[cluster.id,2]*no.pt),nrow(data.default))
###26022006(End): Make changes here

    ##=================================================================================##
    ##==STEP A:                                                                      ==##
    ##==Determine the maximum extension permissible for each cluster and yet ensuring==##
    ##  they do not overlap.                                                         ==##
    ##!!NOTE: I am ensuring that the clusters do not overlap when doing the extension!!##
    ##=================================================================================##
    
    ##The maximum number of windows created 
    maxWin<-nrow(zoom.dat$select.F)
    ##==The maximum extension on the left and right side of the extension of the cluster
    ##The furtherest window extendable to the left before overlapping another cluster
    cluster.idLeft<-ifelse(cluster.id==1,
                           (interest.all[cluster.id,1]-1),interest.all[(cluster.id-1),2])
    ##To check if the cluster.id is the first one, if not, then extendable to the first window
    cluster.idLeft<-ifelse((cluster.id==1 & cluster.idLeft>=0),0,cluster.idLeft)
    
    ##The furtherest window extendable to the right before overlapping another cluster
    cluster.idRight<-ifelse(cluster.id==nrow(interest.all),
                            (interest.all[cluster.id,2]+1),interest.all[(cluster.id+1),1])
    ##To check if the cluster.id is the last one, if not, then extendable to the last window
    cluster.idRight<-ifelse((cluster.id==nrow(interest.all) & (cluster.idRight<=(maxWin+1))),
                            (maxWin+1),cluster.idRight)
    
###26022006(Start): Make changes here
    ##The extreme row indices extendable for the left and right row.id
    ##(note that due to the possibility that the last window may have less no. of pt. for the specified windows,
    ## therefore, we need to use min(.,nrow(data.default)))
    row.id.maxExt<-c((cluster.idLeft*no.pt+1),min(((cluster.idRight-1)*no.pt),nrow(data.default)))
###26022006(End): Make changes here

    ##======================================================##
    ##==Prepare the proteomic data of a particular cluster==##
    ##======================================================##

    ##Restrict the proteomic data that correspond to the cluster
    data.cluster<-data.default[row.id,,]
    
    
    ##============================================================================
    ##==Step B: Find a spectrum with the highest amplitude as a peak template.  ==
    ##==        It is called the dominant spectrum and used to determine the no.==
    ##==        of peaks in the region. Assume the difference of the range as   ==
    ##==        an indication of amplitude.                                     ==
    ##============================================================================
    
    ##ID of the template spectrum
    pickResult<-pickSpect(data.cluster,warning.path=warning.path,cluster.id=cluster.id)
    idRef<-pickResult$idRef
    
    ##The template/reference spectrum with m/z and intensity
    refSpect<-data.cluster[,idRef,]##Rename max.spect to refSpect
      
    
    ##================================================================================##
    ##Locate the minimum and maximum points in the dominant(old: template) spectrum.  ##
    ## Modified algorithm by Coombes et.al. (2003) Clinical Chemistry 49:10 1615-1623 ##
    ## Quality Control and Peak Finding for Proteomics Data Collected from Nipple     ##
    ## Aspirate Fluid by Surface-Enhanced Laser Desorption and Ionization             ##
    ##================================================================================##
      
    ##The peak and valley of the dominant(old: template) spectrum
    minmaxInfo<-locate.minmax(refSpect,mz.limit=mz.limit,symm.limit=symm.limit,
                              warning.path=warning.path,cluster.id=cluster.id,idRef=idRef)

    ##Initializing the "old" id and row.id of the dominant spectrum
    row.id0<-c(0,row.id)

    while(!(identical(row.id,row.id0))){
      
      ##Save the dominant spectrum previous row.id indices
      row.id0<-row.id
      ##Extend the dominant spectrum
      row.id<-extendDomSpect(idRef,data.default,row.id,minmaxInfo,row.id.maxExt,
                              warning.path=warning.path,cluster.id=cluster.id)
      
      ##===================================##
      ##==Update the new extended cluster==##
      ##===================================##
      ##Restrict the proteomic data that correspond to the extended cluster
      data.cluster<-data.default[row.id,,]
            
      ##===========================================================================
      ##==Step1: Find a spectrum with the highest amplitude as a peak template.  ==
      ##==       It is called the dominant spectrum and used to determine the no.==
      ##==       of peaks in the region. Assume the difference of the range as   ==
      ##==       an indication of amplitude.                                     ==
      ##===========================================================================
      
      ##The template/reference spectrum with m/z and intensity
      refSpect<-data.default[row.id,idRef,]##Rename max.spect to refSpect
  
    
      ##================================================================================##
      ##Locate the minimum and maximum points in the dominant(old: template) spectrum.  ##
      ## Modified algorithm by Coombes et.al. (2003) Clinical Chemistry 49:10 1615-1623 ##
      ## Quality Control and Peak Finding for Proteomics Data Collected from Nipple     ##
      ## Aspirate Fluid by Surface-Enhanced Laser Desorption and Ionization             ##
      ##================================================================================##
      
      ##The peak and valley of the dominant(old: template) spectrum
      minmaxInfo<-locate.minmax(refSpect,mz.limit=mz.limit,symm.limit=symm.limit,
                                warning.path=warning.path,cluster.id=cluster.id,idRef=idRef)
    }
    
    ##Locate the max points of the dominant (old: template) in minmaxInfo
    locate.max<-which(minmaxInfo[,"max"]==1)
    
    ##Locate the min points of the dominant spectrum from minmaxInfo.
    ##Locate the row indices related to the dominant peak mz range for all spectra
    if(nrow(minmaxInfo)>2){##When there is a proper peak identified
      ##locate the min/valley's mz values
      locate.mz.min<-cbind(minmaxInfo[locate.max-1,"mz"],minmaxInfo[locate.max+1,"mz"])
      colnames(locate.mz.min)<-c("mz.left","mz.right")
      ##The mz range of the dominant peak
      peakRange<-list(NULL)
      no.peak<-ifelse(is.matrix(locate.mz.min),nrow(locate.mz.min),1)
      for(i in 1:no.peak){
        peakRange[[i]]<-c(locate.mz.min[i,"mz.left"],locate.mz.min[i,"mz.right"])
      }
      names(peakRange)<-locate.max
      
    }else{
      ##locate the min/valley's mz values
      locate.mz.min<-matrix(0,nrow=1,ncol=2)
      locate.mz.min[1,1]<-ifelse(minmaxInfo[1,"max"]==0,minmaxInfo[1,"mz"],NA)
      locate.mz.min[1,2]<-ifelse(minmaxInfo[2,"max"]==0,minmaxInfo[2,"mz"],NA)
      colnames(locate.mz.min)<-c("mz.left","mz.right")
      
      ##Locate the row indices related to the peak template
      peakRange<-list(NULL)
      peakRange[[1]]<-c(minmaxInfo[1,"mz"],minmaxInfo[2,"mz"])
      names(peakRange)<-locate.max
      
    }

    
    ##===========================================##
    ##==Plot the dominant spectra peaks profile==##
    ##===========================================##
###==Start 06092007: Decide whether the diagnostic plot is wanted
    if(plots){
      ##Plot out the peak and valley of the template spectrum
      pdf(paste(diagnostic.path,"dominant.cluster.id.",cluster.id,".mz.",
                round(min(data.cluster[,,1])),"to",round(max(data.cluster[,,1])),".pdf",sep=""))
      row.idOldClust<-((interest.all[cluster.id,1]-1)*no.pt+1):(interest.all[cluster.id,2]*no.pt)
      plot(refSpect,xlab="m/z",ylab="intensity",type="l",main=paste("Cluster:",cluster.id," Spectrum:",idRef),
         sub=paste("New no.pt: ",nrow(data.cluster)," (",length(row.idOldClust),")",sep=""))
      points(data.default[row.idOldClust,idRef,],pch="*")
      abline(v=unlist(subset(minmaxInfo,max==1,"mz")),lty=2)
      points(minmaxInfo[,3:4],pch=ifelse(minmaxInfo[,"max"]==1,19,24))
      legend(min(refSpect[,1]),max(refSpect[,2]),c("peak","valley"),pch=c(19,24))
      dev.off()
    }
###==End 06092007: Decide whether the diagnostic plot is wanted

    ##==================================================================##
    ##Step 2: Align the spectra to the dominant spectrum's peak mz range##
    ##        to get a template.                                        ## 
    ##==================================================================##
    
    ##To arrange the information of the intensities into a matrix
    ##1st: spectrum id, 2nd: max pt, 3rd: grid, 4th: step, 5th: scale
    alignInfoMulti<-list(NULL)
    ##To store the 'results' of the PCA for the cluster
    resultsAll<-list(NULL)
    ##To store the information of the valley and peaks
    tempPeakValleyAll<-list(NULL)
    ##To store the template information
    templateAll<-list(NULL)
    ##To store the information of the adjusted mz values of the spectra
    adjMzAll<-list(NULL)
    
    for(peak.id in 1:length(peakRange)){
    
      ##Identify the points that are within the peak range
      withinRange<-(data.default[,,1]>=peakRange[[peak.id]][1] & data.default[,,1]<=peakRange[[peak.id]][2])
      ##The row indices which is within the peak range
      row.idPeakPCA<-which(apply(withinRange,1,any))
      ##The range of the row indices for each spectrum
      row.idRangeSpec<-apply(withinRange,2,function(x) range(which(x)))
      ##The no of excess points (1st row=left, 2nd row=right) for each spectrum (columns) which are out of the mz range
      excessPt<-ifelse(row.idRangeSpec[1,]>row.idPeakPCA[1],row.idRangeSpec[1,]-row.idPeakPCA[1],0)
      excessPt<-rbind(excessPt,ifelse(row.idRangeSpec[2,]<row.idPeakPCA[length(row.idPeakPCA)],
                                      row.idPeakPCA[length(row.idPeakPCA)]-row.idRangeSpec[2,],0))
      
      if(length(row.idPeakPCA)>2){##There are enough of points in the spectra for PCA method and to give a profile of a max point profile (3 points).
        
        results<-onePeakPCA(data.default,row.idPeakPCA,excessPt,tol=toler,maxit=maxiter,tolCon=tolConMain,
                            mz.limit=mz.limit,symm.limit=symm.limit,warning.path=warning.path,cluster.id=cluster.id)

        ##Store the information of the results from PCA
        resultsAll[[peak.id]]<-results

        ##==============================================================================================================##
        ##Adjusting the mz values of the left valley, peak and right valley of each spectrum according to the shift term##
        ##Compute the variables "max.mz", "left.mz" and "right.mz".                                                     ##
        ##==============================================================================================================##
        adjMz<-apply(matrix(1:ncol(data.default),ncol=1),1,
                     function(i,data.default,row.idPeakPCA,results){
                       ##The mz/ values of the cluster
                       y<-data.default[row.idPeakPCA,i,1]
                       ##The row indices of the cluster to 'data.default'
                       x<-row.idPeakPCA
                       ##Model the qaudratic relationship between the mz and row indice
                       model<-lm(y~x+x2,data=data.frame(y=y,x=x,x2=x^2))
                       ##The new row indice due to shift
                       xNew<-(row.idPeakPCA+results$shift[i])
                       ##The predicted mz of the peak location for the spectrum
                       predict(model,newdata=data.frame(x=xNew,x2=xNew^2))
                     },data.default=data.default,row.idPeakPCA=row.idPeakPCA,
                     results=results)
        
        ##Store the adjusted mz valued for the spectra
        adjMzAll[[peak.id]]<-adjMz
               
        ##Extract the template intensity profiles from PCA. The mz of the points correspond to the average corrected mz.
        template<-cbind(rowMeans(adjMz),results$PC1)       
        ##To determine the peak and the valleys of the template
        minmaxTemp<-locate.minmax(template,mz.limit=mz.limit,symm.limit=symm.limit,
                                  warning.path=warning.path,cluster.id=cluster.id,idRef="template")
        
        if(nrow(minmaxTemp)>2){##There is only at least 2 valleys for each peak
          
          ##Identify the row indices of minmaxTemp that corresponds to the peak of the template
          tempPeak<-which(minmaxTemp[,"max"]==1)
          
          if(length(tempPeak)>1){##There are more than 2 peaks
            
            ##Choose the max pt with the largest intensity as the peak
            ##(i.e. row indices of 'minmaxTemp' corresponding to the Peak)
            tempPeak1<-which.max(minmaxTemp[,"intensity"])
            ##The mz and intensity values of the left valley, peak and right valley.
            tempPeakValley<-minmaxTemp[rep(tempPeak1,3)+c(-1,0,1),c("mz","intensity","id")]
            
            print(paste("There is a case with more than 1 peaks in the template at cluster",cluster.id))
            ##The quality control indicator of the template is 2 (i.e. with more than 1 peak available)
            qc1<-2
            
          }else{##There are only 1 peak with 2 valleys
            
            ##The mz and intensity values of the left valley, peak and right valley.
            tempPeakValley<-minmaxTemp[rep(tempPeak,3)+c(-1,0,1),c("mz","intensity","id")]
            ##Row indices of 'minmaxTemp' corresponding to the Peak
            tempPeak1<-2
            
            ##The quality control indicator of the template is 1 (i.e. with 1 peak and 2 valleys)
            qc1<-1
          }
          
        }else{##There is only 1 valley for each peak
          ##Identify the row indices of minmaxTemp that corresponds to the peak of the template
          tempPeak<-which(minmaxTemp[,"max"]==1)
          
          tempPeakValley<-NULL          
          if(tempPeak==1){##The right valley is known and not the left
            tempPeakValley<-rbind(rep(NA,3),minmaxTemp[,c("mz","intensity","id")])
            ##Row indices of 'minmaxTemp' corresponding to the Peak
            tempPeak1<-1
          }else{##The left valley is known and not the right
            tempPeakValley<-rbind(minmaxTemp[,c("mz","intensity","id")],rep(NA,3))
            ##Row indices of 'minmaxTemp' corresponding to the Peak
            tempPeak1<-2
          }
          
          ##The quality control indicator of the template is 3 (i.e. with 1 peak and 1 valley)
          qc1<-3
        }
        
      }else{##There are not enough of points in the spectra for PCA method or less than 3 pts.
        qc1<-4
        stop("There are cases of these happening: not enough of points in the spectra for PCA method (i.e. less than 3 points)")
        
      }
      
      ##Store the valleys and peak information of the peak of a cluster
      tempPeakValleyAll[[peak.id]]<-tempPeakValley
      ##Store the template
      templateAll[[peak.id]]<-template

      ##====================================================##
      ##Step 2: Align the other spectra to the peak template##
      ##====================================================##
      refSpectInfo<-list(PC1=results$PC1,peakRange=peakRange[[peak.id]],
                         excessPt=excessPt,row.idPeakPCA=row.idPeakPCA,
                         peakValley=tempPeakValley,min.int=results$min.int)
      
      ##ID of non-template spectra
      idOth<-1:ncol(data.default)
      
      ##To arrange the information of the intensities into a matrix
      alignOthFit<-apply(matrix(idOth,ncol=1),1,"alignOptimMultiPeak",
                         refSpect=refSpectInfo,data.default=data.default,
                         warning.path=warning.path,cluster.id=cluster.id,peak.id=peak.id)
      ##The rows are the spectrum and columns the different quantities
      alignOthFit<-t(alignOthFit)
      
      ##quality of Peak
      qualityTemplate<-rbind(qualityTemplate,
                             c(cluster.id,peak.id,idRef,pickResult$rankRef,
                               minmaxTemp[tempPeak1,"noise.ratio"],
                               minmaxTemp[tempPeak1,"intensity"],qc1,results$converge,results$iter,
                               peakRange[[peak.id]][1],peakRange[[peak.id]][2],range(na.omit(tempPeakValley[,"id"]))))
      
      

      ##============================================================================================##
      ##Obtain the estimated amplitude of the m/z value of each spectrum according to the shift term##
      ##Compute the variables "max.amp"                                                             ## 
      ##============================================================================================##
      
      ##==Using the PCA argument to obtain a scale for the max point in the data
      ##The estimated intensity value of a peak
      max.ampPC<-results$amp*tempPeakValley[2,"intensity"]

      colnames.tmp<-c("max.mz","max.mzPC",
                      "step","df","left.mz","right.mz",
                      "max.ampMix","scaleMix","mseMix",
                      "max.ampPC","shiftPC","scalePC","left.mzPC","right.mzPC",
                      "min.int","excessPt")

      ##Consolidating the peak information of the spectra for the cluster
      alignInfo<-cbind(1:ncol(data.default),alignOthFit[,"max.mz"],adjMz[tempPeakValley[2,"id"],],
                       alignOthFit[,c("step","df","left.mz","right.mz")],
                       alignOthFit[,c("max.ampMix","scaleMix","mseMix")],
                       max.ampPC,results$shift,results$amp,
                       adjMz[tempPeakValley[1,"id"],],adjMz[tempPeakValley[3,"id"],],
                       results$min.int,alignOthFit[,"excessPt"])
      
      ##Name the columns of 'alignInfo'
      colnames(alignInfo)<-c("spectrum",colnames.tmp)
      
      ##Store the peak information of the spectra for the cluster
      alignInfoMulti[[peak.id]]<-alignInfo
      
      ##========================================================================##
      ##Step 3(a): Diagnostic plots on the alignment of the template spectrum's ##
      ##           peak(s) to the spectra, peak by peak                         ##
      ##========================================================================##
      ##The pdf file which the diagnostic 
###==Start 06092007: Decide whether the diagnostic plot is wanted
      if(plots){
        pdf(paste(diagnostic.path,"diagnostic.cluster.id.",cluster.id,".",peak.id,".mz.",
                  round(peakRange[[peak.id]][1]),"to",round(peakRange[[peak.id]][2]),".pdf",sep=""))
        par(mfrow=c(2,2))
        for(i in 1:ncol(data.default)){
          ##The raw initial intensities
          raw<-data.default[row.idPeakPCA,i,2]-results$min.int[i]
        
          ##The fitted intensities based on the PC approach
          fittedPC<-results$PC1*results$amp[i]
          if(!is.na(alignInfo[i,"step"])){
            ##Indices of the peak
            tmp<-tempPeakValley[,"id"]
            tmp<-range(tmp[!is.na(tmp)])
            ##The fitted intensities based on the mixture model approach
            fittedMix<-results$PC1[tmp[1]:tmp[2]]*alignInfo[i,"scaleMix"]
          
          }else{
            ##The fitted intensities based on the mixture model approach
            fittedMix<-NULL
          }

          ylim<-range(raw,fittedMix,fittedPC)
          ##Plot all three of the intensities
          plot(raw,ylim=ylim,type="l",
               main=paste("Spectrum",i),sub=paste("shift:",round(results$shift[i],3),"(No flag on MSE)"),col=1)
        
          lines(fittedPC,col=2)
          if(!is.na(alignInfo[i,"step"])){
            lines(fittedMix,col=3)
          }else{
            lines(rep(alignInfo[i,"max.mz"],2),c(ylim[1],alignInfo[i,"max.ampMix"]),col=2)
          }
          legend(1,max(raw),c("raw","fittedPC","fittedMix"),lty=rep(1,3),col=1:3)
        }
        dev.off()
      }
###==End 06092007: Decide whether the diagnostic plot is wanted
    }

    templateInfo[[cluster.id]]<-templateAll
    
    
    ##========================================================================##
    ##Step 3(b): Diagnostic plots on the alignment of the template spectrum's ##
    ##           peak(s) to the other spectra                                 ##
    ##========================================================================##
    
    ##The mz range used by MSE approach to detect peaks
    clusterMzRange<-range(sapply(alignInfoMulti,function(x) range(na.omit(c(x[,c("left.mz","max.mz","right.mz")])))))
    ##The mz range of the dominant peak
    peakRange.lim<-sapply(peakRange,range)
    ##The row indices
    clusterMzId<-which(apply((data.default[,,1]>=(min(clusterMzRange[1],peakRange.lim[1,1])) &
                              data.default[,,1]<=(max(clusterMzRange[2],peakRange.lim[2,ncol(peakRange.lim)]))),1,any))
    ##THe mz range used by PCA to detect peaks
    clusterMzRange<-range(clusterMzRange,data.default[clusterMzId,,1])
    
    
    ##==Produce the plots of the fitted peaks of the template spectrum to the other spectra
###==Start 06092007: Decide whether the diagnostic plot is wanted
      if(plots){
        pdf(paste(diagnostic.path,"diagnostic.cluster.id.",cluster.id,".mz.",
                  round(min(data.cluster[,,1])),"to",round(max(data.cluster[,,1])),".pdf",sep=""))
        oldpar<-par(mfrow=c(2,2))
        for(i in 1:ncol(data.default)){
      
          ##===============================================##
          ##==Obtain the cluster from the actual spectrum==##
          ##===============================================##
          ##The adjusted mz and intensity values of the fitted spectrum
          fitted<-list(NULL)
      
          for(peak.id in 1:length(peakRange)){

            ##==Preparing the objects for the plotting
            ##The PCA results
            results<-resultsAll[[peak.id]]
            ##The valleys and peaks information of the templates
            tempPeakValley<-tempPeakValleyAll[[peak.id]]
            ##The information
            alignInfo<-alignInfoMulti[[peak.id]]
        
            ##==Reconstruct the row indices of data.default that correspond to the cluster
            ##Identify the points that are within the peak range
            withinRange<-(data.default[,,1]>=peakRange[[peak.id]][1] & data.default[,,1]<=peakRange[[peak.id]][2])
            ##The row indices for 'data.default' which is within the peak range
            row.idPeakPCA<-which(apply(withinRange,1,any))
        
            if(!is.na(alignInfo[i,"scaleMix"])){
          
              ##Get the range of the template
              idTemplate<-range(na.omit(tempPeakValley[,"id"]))
              valid<-(row.idPeakPCA+alignInfo[i,"step"]+alignInfo[i,"excessPt"])[1:(diff(idTemplate)+1)]
              ##Get the profile of the fitted
              fitted[[peak.id]]<-cbind(data.default[valid,i,1],
                                       (alignInfo[i,"scaleMix"]*results$PC1[idTemplate[1]:idTemplate[2]]
                                        +results$min.int[i]))
            }else{
              fitted[[peak.id]]<-cbind(rep(alignInfo[i,"max.mz"],2),c(min(sapply(resultsAll,function(x) min(x$min.int))),
                                                                  (alignInfo[i,"max.ampMix"]+results$min.int[i])))
            }
          }
      
      
          ##The mz and intensity values of the plot region
          data.plot<-data.default[clusterMzId,i,]
      
          ##The range of the mz and intensity values in the clusters, fitted, corrected and original
          xylim<-sapply(fitted,function(x) apply(x,2,range))
          ##The range of the mz values in the cluster
          xlim<-range(xylim[1:2,],clusterMzRange,peakRange.lim)
          ##The range of the intensities values in the cluster
          ylim<-range(xylim[3:4,],data.plot[,2])
      
          ##Set the figure for the plots for plotting
          plot(data.plot,xlim=xlim,ylim=ylim,main=paste("Spectrum",i),sub="(No flag on MSE)",type="n")
      
          for(peak.id in 1:length(peakRange)){
            ##THe shaded region of orginal interest
            rect(peakRange[[peak.id]][1],ylim[1],peakRange[[peak.id]][2],ylim[2],border="white",col="gray")
          }
      
          for(peak.id in 1:length(peakRange)){
            ##Plot the fitted spectrum to the actual data
            lines(fitted[[peak.id]],col=peak.id,lwd=2)
            ##Plot the vertical line (amptitude) of the peak
            lines(rep(alignInfoMulti[[peak.id]][i,"max.mz"],2),
                  c(ylim[1],alignInfoMulti[[peak.id]][i,"max.ampMix"]+alignInfoMulti[[peak.id]][i,"min.int"]),
                  col=peak.id)
          }

          ##The points for plotting (based on the range of the mz values for all valleys in the clusters)
          lines(data.plot)

     
        }
        dev.off()
     }
###==End 06092007: Decide whether the diagnostic plot is wanted
    
    ##===============================================================================================##
    ##Arrange the information of the template spectrum's peaks fitted to the other spectra.          ##
    ##Row-combine the information before converting it into an array outside the cluster.id for-loop.##
    ##===============================================================================================##
    if((no.peaks<-length(alignInfoMulti))>1){##>1 peak for each template spectrum
      collect.tmp<-do.call("rbind", alignInfoMulti)
      collect.tmp<-cbind(cluster.id,peak.id=rep(1:no.peaks,each=no.spectra),collect.tmp)
      rowStart<-ifelse(is.null(collect.Info),0,nrow(collect.Info))
      rownames(collect.tmp)<-seq(rowStart+1,rowStart+nrow(collect.tmp))
      ##row combine the present peak info with the previous
      collect.Info<-rbind(collect.Info,collect.tmp)
      
    }else{
      collect.tmp<-cbind(cluster.id,peak.id=rep(1,nrow(alignInfoMulti[[1]])),alignInfoMulti[[1]])
      rowStart<-ifelse(is.null(collect.Info),0,nrow(collect.Info))
      rownames(collect.tmp)<-seq(rowStart+1,rowStart+nrow(collect.tmp))
      ##row combine the present peak info with the previous
      collect.Info<-rbind(collect.Info,collect.tmp)
    }
    

###..................................................................###
###....Start here to do the mixture modeling to quantify the peaks...###
###..................................................................###

    if(parPeak){
###==Start 06092007: Decide whether the diagnostic plot is wanted
      if(plots){
        pdf(paste(diagnostic.path,"diagnostic.Parametric.cluster.id.",cluster.id,".mz.",
                  round(min(collect.tmp[,c("max.mz","left.mz","right.mz")])),"to",
                  round(max(collect.tmp[,c("max.mz","left.mz","right.mz")])),".pdf",sep=""))
        par(mfrow=c(2,2))
      }
###==End 06092007: Decide whether the diagnostic plot is wanted

      infoPA.tmp<-lapply(matrix(1:ncol(data.default),ncol=1),"peakPA",
                         collect.tmp=collect.tmp,data.default=data.default,
                         maxiter=par.maxiter,toler=toler,warning.path=warning.path,
                         cluster.id=cluster.id,plots.TF=plots)
                         
###==Start 06092007: Decide whether the diagnostic plot is wanted
      if(plots){
        dev.off()
      }
###==End 06092007: Decide whether the diagnostic plot is wanted

      infoPA.tmp<-cbind(cluster.id=cluster.id,do.call("rbind",infoPA.tmp))
      rownames(infoPA.tmp)<-NULL

      infoPA<-rbind(infoPA,infoPA.tmp)
    }
    
###................................................................###
###....End here to do the mixture modeling to quantify the peaks...###
###................................................................###

    
  }##The end of the loop for a cluster id.
  
  ##===================================================================================================##
  ##Arrange the information of the template spectrum's peaks fitted to the other spectra into an array.##
  ##The dim of the array (i.e.arrayInfo) is c(peak,spectra,info).                                      ##
  ##===================================================================================================##
 
  ##Order collect.Info such that it is by spectrum, cluster then peak. This makes it easy to convert to an array.
  rowBySpect<-as.vector(matrix(1:nrow(collect.Info),ncol=no.spectra,byrow=TRUE))
  collect.Info<-collect.Info[rowBySpect,]
  arrayInfoName<-colnames(collect.Info)[-c(1:3)]
  arrayInfo<-array(c(collect.Info[,-c(1:3)]),
                   c(no.row<-nrow(collect.Info)/no.spectra,no.spectra,length(arrayInfoName)),
                   list(paste(collect.Info[1:no.row,"cluster.id"],collect.Info[1:no.row,"peak.id"],sep="."),
                        1:no.spectra,arrayInfoName)
                   )

  colnames(qualityTemplate)<-c("cluster.id","peak.id","idRef","rank","noise.ratio","tempPeakInt","qc1",
                               "convPer","convPC","iter","domPeakLeftMz","domPeakRightMz","idTempL","idTempR")

  ##Using the scaled MSE to flag those peak with terribly bad scale MSE
  mseCut<-quantile(na.omit(arrayInfo[,,"mseMix"]), probs = seq(msetol, msetol, length.out=1))

  ##Identify those peaks and spectrum with terrible scaled MSE
  setNA<-(arrayInfo[,,"mseMix"]>mseCut)

  ##Set up a new info matrix with the the quality control on the MSE
  arrayFinal<-arrayInfo
  ##Set the max mz and height to zero
  arrayFinal[,,"max.mz"][setNA]<-NA
  arrayFinal[,,"max.ampMix"][setNA]<-NA
  
  ##===================================================================================================##
  ##Arrange the information of peaks fitted to the other spectra into an array via parametric approach.##
  ##The dim of the array (i.e.infoPA) is c(peak,spectra,info).                                         ##
  ##===================================================================================================##
  
##080207: This option is valid if parPeak==TRUE  
  if(parPeak){
    ##Order infoPA such that it is by spectrum, cluster then peak. This makes it easy to convert to an array.
    rowBySpect<-order(infoPA[,"spectrum"],infoPA[,"cluster.id"],infoPA[,"peak.id"])
    infoPA<-infoPA[rowBySpect,]
    infoPAName<-colnames(infoPA)[-c(1:3)]
    infoPA<-array(c(infoPA[,-c(1:3)]),
                  c(no.row<-nrow(infoPA)/no.spectra,no.spectra,length(infoPAName)),
                  list(paste(infoPA[1:no.row,"cluster.id"],infoPA[1:no.row,"peak.id"],sep="."),
                       1:no.spectra,infoPAName)
                  )
  }
  
  return(list(arrayFinal=arrayFinal,arrayInfo=arrayInfo,qualityTemplate=qualityTemplate,
              templateInfo=templateInfo,infoPA=infoPA))
  
}


