"rawBG.plot" <-
function(dat=NULL,smoother=NULL,type=NULL)
{

  #===========================
  # Layout
  #================== 
  old.par <- par(no.readonly = TRUE)  
  par(mfrow=c(2,1))


  
  #===========================
  # Security checks
  #==================  
  
  if(is.null(dat) | is.null(smoother) | is.null(type))
  {
    stop(paste("Function \"plot.Raw\" returns error.","\n",
               "Reason: One of the arguments (dat, smoother, type) is \"NULL\".")
         )
  }

  if(smoother!="rsmooth" & smoother!="loess")
  {
    stop(paste("Function \"plot.RawBG\" returns error.","\n",
               "Reason: Argument smoother is not \"rsmooth\" or\"loess\".")
         )
  }

  if(type!="raw" & type!="BGcorrected" & type!="rawBG")
  {
    stop(paste("Function \"plot.Raw\" returns error.","\n",
               "Reason: Argument type is not \"raw\", \"BGcorrected\" or \"rawBG\"."
               )
         )
  }

  
  #===========================
  # Variables
  #============
  
  num.spectra <- dim(dat$loess.dat$loess.dat)[2]
  range.mz <- c(dat$region[1],dat$region[2])


  #===========================
  # Plotting
  #============
  
  if(smoother=="rsmooth")
  {
    my.ylim <- range(dat$loess.dat$loess.dat[1:length(dat$loess.dat$fit.loess[[1]]$y),,2])
    if(type=="raw")
    {
      plotData <- dat$read.dat
      subText <- paste("data with",smoother,"curve but without correction")
    }
    if(type=="BGcorrected")
    {
      plotData <- dat$loess.dat$loess.dat
      subText <- paste("data with",smoother,"background correction (rsmooth)")
    }
    if(type=="rawBG")
    {
      plotData.raw <- dat$read.dat
      subText.raw <- paste("data with",smoother,"curve but without correction")

      plotData.BGcorrected <- dat$loess.dat$loess.dat
      subText.BGcorrected <- paste("data with",smoother,"background correction (rsmooth)")
    }
  }
  if(smoother=="loess")
  {  
    my.ylim <- range(dat$loess.dat$loess.dat[dat$loess.dat$ndx,,2])
    if(type=="raw")
    {
      plotData <- dat$read.dat      
      subText <- paste("data with",smoother,"curve but without correction")
    }
    if(type=="BGcorrected")
    {
      plotData <- dat$loess.dat$loess.dat
      subText <- paste("data with",smoother,"background correction (loess)")
    }
    if(type=="rawBG")
    {
      plotData.raw <- dat$read.dat      
      subText.raw <- paste("data with",smoother,"curve but without correction")      

      plotData.BGcorrected <- dat$loess.dat$loess.dat
      subText.BGcorrected <- paste("data with",smoother,"background correction (loess)")
      
    }
    
  }

  for(i in 1:num.spectra)
  {
    if(type=="rawBG")
    {
      # plot the proteomic data without background correction
      plot(plotData.raw[,i,],
           type="l",
           xlab="m/z",
           ylab="Intensity",
           xlim=range.mz,
           ylim=my.ylim,
           main=paste("Spectrum number:", i),
           sub=subText.raw
          )
      if(smoother=="rsmooth")
      {
        my.linesX <- dat$loess.dat$fit.loess[[i]]$x
        my.linesY <- dat$loess.dat$fit.loess[[i]]$y
      }
      if(smoother=="loess")
      {
        my.linesX <- dat$loess.dat$loess.dat[dat$loess.dat$ndx,i,1]
        my.linesY <- dat$loess.dat$fit.loess[[i]]$fitted
      } 
      lines(my.linesX,my.linesY,lwd=3,col="red")

      # plot the proteomic data with background correction
      plot(plotData.BGcorrected[,i,],
           type="l",
           xlab="m/z",
           ylab="Intensity",
           xlim=range.mz,
           ylim=my.ylim,
           main=paste("Spectrum number:", i),
           sub=subText.BGcorrected
          )      
    }else
    {
      # plot the proteomic data with or without background correction
      plot(plotData[,i,],
           type="l",
           xlab="m/z",
           ylab="Intensity",
           xlim=range.mz,
           ylim=my.ylim,
           main=paste("Spectrum number:", i),
           sub=subText
          )
      if(type=="raw")
      {
        if(smoother=="rsmooth")
        {
          my.linesX <- dat$loess.dat$fit.loess[[i]]$x
          my.linesY <- dat$loess.dat$fit.loess[[i]]$y
        }
        if(smoother=="loess")
        {
          my.linesX <- dat$loess.dat$loess.dat[dat$loess.dat$ndx,i,1]
          my.linesY <- dat$loess.dat$fit.loess[[i]]$fitted
        } 
        lines(my.linesX,my.linesY,lwd=3,col="red")
      }
    }
  }# //for()
  on.exit(par(old.par))
}#// plot.rawBG

