##Date: 080107
##  -remove the argument clin.dat, which was "clin.dat=FALSE". 

"read.ProSpect" <-
function(spectra.dir,expt.name=NULL) {


#  spectra<-sort(dir(path=spectra.dir,pattern="\.csv$"))
  spectra<-sort(dir(path=spectra.dir,pattern="csv$"))
                          #store names of found .cvs-files (with extension .csv)
  
  spectra=lapply(spectra, function(x) substr(x,start=1,stop=nchar(x)-4))
                         #the names without extension (is required for following security checks)

#  fullPath.spectra<-sort(dir(path=spectra.dir,pattern="\.csv$",full.names=TRUE))
  fullPath.spectra<-sort(dir(path=spectra.dir,pattern="csv$",full.names=TRUE))
                          #store full name (with path) of found .cvs-files

  num.spectra<-length(spectra)
                          #store number of found .cvs-files

#  clin.dat <- dir(path=spectra.dir,pattern="\.clinDat$")
  clin.dat <- dir(path=spectra.dir,pattern="clinDat$")
  if(length(clin.dat) < 1)
  {
    clin.dat <- FALSE
  }else
  {
  
#    fullPath.clinDat<-dir(path=spectra.dir,pattern="\.clinDat$",full.names=TRUE)
    fullPath.clinDat<-dir(path=spectra.dir,pattern="clinDat$",full.names=TRUE)


    if (length(fullPath.clinDat) > 1){stop("Directory contain more than 1 file with clinical data")}

    clin.dat <- as.data.frame(read.delim(fullPath.clinDat))
                          #otherwise it is a list and data frame better to handle
                          #e.g. clin.dat[1,"Age"]  
  
    if(nrow(clin.dat)!=length(spectra)){stop("The clinical data do not match with the given amount of spectra.")}

    index <- order(clin.dat[[1]])
    clin.dat <- clin.dat[index,]

# print(unlist(spectra))
# print(as.character(clin.dat$Name))
# print(!all(spectra==clin.dat$Name))
    #Check: if names and order are equal
#  if(all.equal(unlist(spectra),as.character(clinData$Name) ) != TRUE)
    if(!all(spectra==clin.dat$Name))   #advantage that type of both can be different
    {
      stop("The spectra names within the clinical data file do not match with the given spectra.")
    }
                                        
  }
  
  # security check if the user given directory contains .cvs-files  
  if(num.spectra<1){
     stop("Error: There are no .cvs-files")
   }

  nrow.spectra<-NULL
  mz.spectra<-NULL
  intensity.spectra<-NULL
                          #are used in the for loop and must defined
                          # for 1st using due to their structure

  
  # read all .cvs-files from the user given directory one by one
  for(i in 1:num.spectra){
    current.spectrum<-read.csv(file=fullPath.spectra[i])
                          #store the current spectrum
                          #full path must given as argument, if
                          # not error message (file not found)

    nrow.spectra<-c(nrow.spectra,nrow(current.spectrum))
                          #store the number of pairs of points for    
                          # each spectrum
                          #contains after last loop the number of rows
                          # for all spectra
                          #nrow(current.spectra) counts the rows of
                          # current spectrum

    #security check if all spectra has the same length of data sets 
    if(length(unique(nrow.spectra))!=1){
                          #if the row numbers of all spectra are the same
                          # the return value of unique(nrow.spectra) is 1
      stop("Error: Length of the spectra are different!!!")
    }
    mz.spectra<-c(mz.spectra,current.spectrum[,1])
                          #store the mz values for each spectrum
    intensity.spectra<-c(intensity.spectra,current.spectrum[,2])
                          #store the intensities values for each spetrum
  }
  ci.dat<-array(c(mz.spectra,intensity.spectra),
               dim=c(nrow.spectra[1],num.spectra,2),
                          #nrow.spectra[1] to get 1st element of
                          # vector nrow.spectra -> num of rows of each
                          # spectrum
                          #1.dim: 48.000 messure points
                          #2.dim: number of spectra
                          #3.dim: 2 values in 3rd dimendion
                          # (1=mz,2=intensity)
               dimnames=list(c(1:nrow.spectra[1]),
                 spectra,c("mz","intensities")
                 )
               )
  attr(ci.dat,"sample")<- spectra
  attr(ci.dat,"project")<- expt.name

  

  return(list(ci.dat,clin.dat))
}

