##NOTE (08/01/2007): Remove typ.range from reduct.ProSpect and comment the if(typ.range==1)

"reduct.ProSpect" <-
function(dat,range.mz,method="grid",grid=10,plot.indicator=FALSE) {

  datDimNames <- dimnames(dat)
                          #dimnames have to be used for loess.dat
                          # (see return value), because them are used
                          # by basic.summary.ProSpect() (summaryPeaks())
  
  #===============================================
  # Reducing the data set to the range of interest
  #===============================================
  
  tf.mz<-dat[,,1]>=range.mz[1] & dat[,,1]<=range.mz[2]
                          #tf.mz contains only the first both dim of
                          #  the  3dim Array 'ci.dat .
                          #tf.mz contains the same
                          # count of rows and columns like 'ci.dat'
                          # but instead of real data point 'tf.mz'
                          # contains Booleans.
                          #T for the values which are within the given
                          # range and F for all otherwise.

  
  ##========================##
  ##Find the reference point##
  ##========================##
  ##to find the spectrum with the most valid intensity
  num.points<-apply(tf.mz,2,"sum")
  max.spec<-which.max(num.points)

  ##The columns of the other spectra
  oth<-matrix(c(1:ncol(dat))[-max.spec],ncol=1)

  ##To find the optimal steps and the valid rows
  #postscript("C:\\thesis\\output\\alignment\\CM10.low.3to4kDa.ps")
  #par(mfrow=c(2,2))
  step.all<-apply(oth,1,"alignOptim",idRef=max.spec,dat=dat,tf.mz=tf.mz,
                  method=method,grid=grid,plot.indicator=plot.indicator)
  #dev.off()
  
  ##To gather all the information about the optimal rows
  valid.all<-sapply(step.all,
                    function(x){
                      return(c(x$step,range(x$valid[,"ref"]),range(x$valid[,"oth"])))
                    })

  dimnames(valid.all)<-list(c("step","minRef","maxRef","minOth","maxOth"),oth)
  
  # Decision which kind of range should be used
#  if(typ.range==1) {
    
    ##The min and max row numbers for reference spectrum
    min.mzRef<-max(valid.all["minRef",])
    max.mzRef<-min(valid.all["maxRef",])

    ##The excess number of rows from the other spectra
    diff.min.mzOth<-min.mzRef-valid.all["minRef",]
    diff.max.mzOth<-valid.all["maxRef",]-max.mzRef
    
    ##The min and max row numbers for other spectra
    min.mzOth<-valid.all["minOth",]+diff.min.mzOth
    max.mzOth<-valid.all["maxOth",]-diff.max.mzOth

    ##The top row is the first row indices and second
    ## row is the last row indices
    row.id<-rbind(c(min.mzRef,min.mzOth),c(max.mzRef,max.mzOth))

    ##row,id w.r.t. dat and not dat[,,1][tf.mz]
    first.nrow<-apply(dat[,,1]<range.mz[1],2,"sum")
    row.id<-row.id[,order(c(max.spec,oth))]+rbind(first.nrow,first.nrow)
#  }

  #cbind(first.nrow,1:30)
  #dat[sort(unique(first.nrow)),,1]
  ##Obtaining the
  nrow.reduct<-diff(row.id[,1])+1
  reduct.dat<-array(0,dim=c(nrow.reduct,ncol(dat),2))
  reduct.dat[,,1]<-apply(matrix(1:ncol(dat),ncol=1),1,
                         function(i,dat,row.id){
                           dat[,i,1][row.id[1,i]:row.id[2,i]]                           
                         },dat=dat,row.id=row.id)
  reduct.dat[,,2]<-apply(matrix(1:ncol(dat),ncol=1),1,
                         function(i,dat,tf.mz,row.id){
                           dat[,i,2][row.id[1,i]:row.id[2,i]]
                         },dat=dat,row.id=row.id)
                          #Restrict the original given 3dim Array ci.dat
                          # to the range of interest.

  ##To plot the difference between the reference and the other spectrum
  if(plot.indicator==TRUE){
    plot(reduct.dat[,1,1]-reduct.dat[,max.spec,1],type="n",ylab="difference",xlab="row indices after alignment algorithm")
    apply(matrix(1:(length(oth)-1),ncol=1),1,
          function(i,reduct.dat,max.spec,oth){
            tmp1<-reduct.dat[,oth[i],1]-reduct.dat[,max.spec,1]
           lines(tmp1,col=oth[i])
          },reduct.dat=reduct.dat,max.spec=max.spec,oth=oth)
  }


  dimnames(reduct.dat) <- list(c(1:nrow(reduct.dat)),datDimNames[[2]],datDimNames[[3]])
  
  return(list(loess.dat=reduct.dat,
              sample=attr(dat,"sample"),
              expt=attr(dat,"project"),
              range=row.id))
}

