"significant.plot" <-
function(dat=NULL,
                             region=NULL,
                             winSizePlot=NULL,
                             grouping=NULL,
                             colour=NULL,
                             ask=FALSE
#                             hex=FALSE
                             )
{

  
  ###################################################################
  # Definition of function color.pValue
  #====================================
   
  color.pValue<-function(p,signLevel)
  {
    log.pValue<--log10(p)
                          #The transformed p-value
    logAlpha<--log10(signLevel)
                          #the transformed p-value of the cut-off
    return(grey((log.pValue<=logAlpha)*1+
                (log.pValue>logAlpha &
                 log.pValue<=5)*0.9 + (log.pValue>5 &
                 log.pValue<=10)*0.85 + (log.pValue>10 &
                 log.pValue<=20)*0.8 + (log.pValue>20)*0.75
               )
           )
                          #Create a vector of colors from a vector of
                          # gray levels.
                          # The arguments must be between 0 and 1.
  }
  ###################################################################  


  #===========================
  # Layout
  #============
  

  old.par <- par(no.readonly = TRUE)

  par(oma=c(3,1,0,0),mar=c(0,0,0.5,0),cex.axis=1,ask=ask)
                          #oma = spaces inner circle
                          #mar = spaces outer circle
  layout(matrix(c(2,2,1,1),2,2,byrow=TRUE), c(1,4), c(1,4), TRUE)


  #===========================
  # Variables
  #============
  
  if(is.null(region))
    {
      region <- dat$region
print("regionNULL")
print(region)
    }
  else
    {
      region <- region*1000
print("regionWasGiven")
print(region)      
    }

  if(is.null(winSizePlot))
    {
      winSizePlot <- 0.5*1000 
    }
  else
    {
      winSizePlot <- winSizePlot
    } 
  
  bin <- seq(region[1],region[2],by=winSizePlot)
  clin.dat <- dat$clin.dat
  num.sample<-dim(dat$loess.dat$loess.dat)[2]
#  categoryName <- names(clin.dat)
#3old.par <- par(no.readonly = TRUE)  
  

  #===========================
  # Security checks
  #================== 

  if(winSizePlot>(dat$region[2] - dat$region[1]) | winSizePlot<=0)
  {
    stop(paste("please choose a value for winSizePlot which is between",
               "0 and the hole choosen region in kDa"))
  }

print(grouping)  
  
  
#  if(length(grouping)>1) #{stop('Value for "grouping" should be one string!')}
#    {    }

  
  #===========================
  # Grouping
  #===========

  
 
  matrix.groupVector <- sapply(grouping, function(x) as.character(eval(parse(text=x),env=clin.dat))) #??? why simplify=FALSE do not work
                          #as.character() has to be used! otherwise return value of eval(...) like
                          # as.numeric(eval(...)) or unclass(eval(...))

#  if(length(grouping)>1) matrix.groupVector <- apply(matrix.groupVector)

print(matrix.groupVector)

  if(length(grouping)<2)
    {
print("grouping<2")      
       matrix.groupVector <- matrix(matrix.groupVector)
                          #grouping=cut(Age,3)[2] to keep dim if only
                          #one groupVector with one element
    }
print("matrix.groupVector")  
print(matrix.groupVector)  
  groupVector <- apply(matrix.groupVector,1,function(x){print(x); toString(x)})
print("groupVector")  
print(groupVector)      
  groupVector.Levels <- levels(factor(groupVector))
print("Levels")
print(groupVector.Levels)  
  groups <- lapply(groupVector.Levels, function(x) which(groupVector==x))
print("groups")
print(groups)

  
  
  #===========================
  # Plotting
  #============  
  
  # The number of bins recreated  
  for(i in 1:(length(bin)-1))
  { 
    mz.reg<-bin[i:(i+1)]
                           #The region of interest for each bin
                           #range for the first bin e.g.: 3000-3500 
    #=======================================
    #Find the points restricted to the range
    #of mz values for the intensities
    #=======================================
    ##CHANGE: 02122010 
    if(!is.null(dat$loess.dat$loess.dat)){
      prot<-dat$loess.dat$loess.dat
    }
                           #extract the intensities and m/z

    mz.i<-(prot[,,1]>=mz.reg[1] & prot[,,1]<=mz.reg[2])
    y.lim<-range(prot[,,2])
 
    #==========================================
    #Find the points restricted to the range
    #of mz values for the modified F-statistics
    #==========================================
    
    mz<-dat$modFcorrFdr.dat$mz
    range <-mz[,"mean.mz"]>=mz.reg[1] & mz[,"mean.mz"]<=mz.reg[2]

    #========================
    #Plot the region selected
    #========================
    plot(prot[mz.i[,1],1,1],
         prot[mz.i[,1],1,2],
         type="n",
         xlim=mz.reg,
         ylim=y.lim,
         ylab="",
         xlab=""           
        )
    mtext("intensity",side=2,outer=FALSE,cex=1,line=3)
    mtext("mz",side=1,outer=FALSE,cex=1,line=2)
                          #Initiating the plot but plot no visible
                          # points. For this will be used lines later.

    zoom.range <- dat$zoom.dat$mz[,"mean.mz"] >= mz.reg[1] &
                  dat$zoom.dat$mz[,"mean.mz"] <= mz.reg[2]

    alpha<-which(as.vector(dat$zoom.dat$select.F[,dat$zoom.dat$pValue]
                           <dat$signLevel & range))
                          #contains the rectangle that are
                          # significant at <0.05      

    # When there is no significant points in this range
    if(length(alpha)>0)
    {
      rect(mz[alpha,"min.mz"],
           y.lim[1],
           mz[alpha,"max.mz"],
           y.lim[2],
           col=color.pValue(dat$zoom.dat$select.F[alpha,dat$zoom.dat$pValue],dat$signLevel),
           border="white",lwd=1.05)
                          #Shaded the region that is significant
                          #rect() draws sequence of rectangles with
                          # given coordinates, fill and border colors.
                          #lwd:line width for borders and shading        
      text(mz[alpha,"mean.mz"],y.lim[1],alpha,cex=0.4)
                          #text() add text to a plot
    }
#print(grouping)
#print(str(grouping))
#print(mode(grouping))
#print(is.null(grouping))
   #Plot the intensities (as visible lines in the plot)
   if(is.null(grouping))
     {       
       #colouring of spectra without grouping 
       for(j in 1:num.sample)
         {
#           if(is.null(colour)){color=j}else{color="black"}
           color="black"
           lines(prot[mz.i[,j],j,1],prot[mz.i[,j],j,2],col=color)
         }
       #if no grouping: add legend in a plot which describes the color shading 
       legend(
              mz.reg[2], #legend starts at the right side of the plot
#              mz.reg[1], #legend starts at the left side of the plot
              y.lim[2],  #legend starts at the top side
              xjust=1,  # legend grows in direction of left side
              yjust=1,  # legend grows in direction of bottom side               
              legend=c(paste(">=",dat$signLevel,sep=""),
                paste("<",dat$signLevel,sep=""),
                "<e-5","<e-10","<e-20"),
              fill=grey(c(1,.9,.85,.8,.75)),
              cex=1
              )
     }
    else
      {
        #coloring of spectra with grouping
        for(j in 1:length(groups))
          {
            if(is.null(colour)){colour <- c(1:length(groups))}           
            lapply(groups[[j]], function(x){lines(prot[mz.i[,x],x,1],prot[mz.i[,x],x,2],col=colour[j])})
          }

        #if grouping: add legend in a plot which describes the
        #colouring of the groups
        t <- paste(toString(grouping),": ",groupVector.Levels,sep="")
        legend(
               mz.reg[1],y.lim[2]-0.9,
               legend=t,     
               fill=colour,
               cex=1,
               ncol=1
              )
      }
    
    
    lim.pValue<-c(0,3)
                          #to get the range of the -log10 of the pValue
    negLog.pValue<- -log10(dat$zoom.dat$select.F[range,
                                                 dat$zoom.dat$pValue])
                          #Convert the p or fdr values which are all in
                          # the range of the current bin
                          # (e.g. 3000-3500) into -log10()
    tf.negLogPValue<-(negLog.pValue>=3)
                          #those p or fdr values which are <=0.001
                          # note: -log10(0.001)=3
                          #contains the TRUE/FALSE values
    negLog.pValue[tf.negLogPValue]<- 3
                          #Replace those p or fdr values with 3 if
                          # they are <=0.001

  #========================
  #plot the p or fdr values
  #========================
    
  plot(mz[range,"mean.mz"],
       negLog.pValue,
       type="l",
       axes=FALSE,
       xlim=mz.reg,
       ylim=lim.pValue
      )
  points(mz[range,"mean.mz"],
         negLog.pValue,
         pch=ifelse(tf.negLogPValue,"x","o")
        )
  mtext(paste("log(",dat$zoom.dat$pValue,"%)",sep=""),
        side=2,outer=FALSE,cex=1,line=3
       )
                          #mtext() write text into the margins of a
                          # plot
  abline(h=-log10(0.05),lty=1)
  abline(h=-log10(0.01),lty=2)

  #============================================
  #Label the ticks marks of the p or fdr values
  #============================================
 
  fdr.label<-c(100,5,1,0.1,0.01)
  axis(2,at=-log10(fdr.label/100),labels=fdr.label)
  axis(1,labels=FALSE)
  legend(mz.reg[1],
         3,
         legend=c("<=0.01%",">0.01%"),
         pch=c("x","o"),
         cex=1,
         bty="n"
        )
                          #Legend of the p or fdr plots
  }# //for() -> num of bin recreated
 on.exit(par(old.par))
}

