"smooth.mse" <-
function(mlmProSpect,B=10,trimmed.propotion=.1) {
  
  # Extracting information needed to do the local mean/median of
  #  smoothing
  mz<-mlmProSpect$mz
                          #m/z information
  mseF<-mlmProSpect$mlmF.dat[,"mseF"]
                          #mse
  Fdf2<-mlmProSpect$mlmF.dat[,"F.df2"]
                          #df of mse
  
  #NOTE THAT IF THE NEAREST N NEIGHBOURS ARE NA, THEN WE HAVE LESS
  #POINTS TO COMPARE IT WITH
  
  ord.mz<-order(mz[,"mean.mz"])
                          #To order the data in increasing manner
                          #according to mean  m/z
  
  #=============================#
  #TODO: check really required? #
  #=============================#
  # quality check if the given mean.mz datas from mlmProSpect are really
  #  increasing according to mean
  num.pts<-nrow(mz)
                          #number of MSE's
                          #for each x-value(mz) exists a y-value
                          # (intensity). The function calculate a
                          #  smooth line based on the mse values of
                          #  the y-values (intensity)

  B1<-ceiling((B/100)*num.pts)
                          #the number of neighbours for the smoothed
                          # MSE
                          #ceilling() returns an the next greater hole
                          # integer if the result of the argument is
                          # a hole number

  # Getting all potential neighbours in the choosen range (depends
  #  from argument B) for each given mse point
  # Potential neighbour means not e.g. the 10 nearest mse's but
  #  an subset of all mse which contains the 10 neighbours at
  #  the left and at the right side (together max. 20 mse's)

  num.run<-NULL    
  tukey<-NULL;
  arithmetic<-NULL;
  trimmed<-NULL;
  med<-NULL;

                          #Initializing variables to start and store
                          # the smoothed MSE

  #=================================#
  #TODO: replace for() with apply() # 
  #=================================#
  for(i in 1:num.pts){
    subset.neighbours = max(i-B1,1) : min(i+B1,num.pts)
                          #If the points are not the first or last
                          # mse point
      
    #Those who reduces the subset.neighbours to the real wanted
    # B1.neighbours
    diff.neighboursCurrMz <- abs(mz[subset.neighbours,"mean.mz"]
                                    -mz[i,"mean.mz"])
                          #returns the difference between each subset
                          # neighbour and the current value (mean.mz)  
                          #abs() returns absolute
                          # value(e.g. abs(-1)=1)
    B1.neighbours<-sort(order(diff.neighboursCurrMz) [1:B1])
                          #order the diff values and returns
                          # only the values in the following range []
                          #after order() sort the return increasing
                          # values

    subset.neighbours <- subset.neighbours[B1.neighbours]
                          #reducing the 1st "pre"-subset to the real
                          # wanted subset choosen by 'B'
    
    #collection of needed variables to do smoothing
    B1.mseF<-na.omit(mseF[subset.neighbours])
                          #mseF[subset.neighbours] contains a reduced
                          # range of values 
                          #na.omit() delete possible NA values
    B1.Fdf2<-na.omit(Fdf2[subset.neighbours])
                          #the same like B1.mseF only for the F.df2
                          # values
    # security check if there is any mistake in the calculation before
    #  in this function or in mlm.ProSpect.R/lm.ProSpect
    stopifnot(attr(B1.Fdf2,"na.omit")==attr(B1.mseF,"na.omit"))
      
    num.run<-c(num.run,length(B1.mseF))

    #tukey.biwt is 1-step tukey biweight
    tukey.mean<-tukey.biwt(B1.mseF)
                          #Note: tukey.biwt.R must be loaded before!!!
                          #Turkey is one of the possibilities to
                          # calculate a mean value
    tukey.df <-(sum(tukey.mean$weights*B1.mseF)^2/
                sum((tukey.mean$weights*B1.mseF)^2/B1.Fdf2))
                          #is the degrees of freedom based on
                          # Sattherwaite approximation
    tukey<-rbind(tukey,c(tukey.mean$mean,tukey.df))
                          #combine df + mean of tukey
      
    #arithmetic mean
    arithmetic.df <-(sum(B1.mseF)^2/sum((B1.mseF)^2/B1.Fdf2))
                          #is the degrees of freedom based on
                          # Sattherwaite approximation      
    arithmetic<-rbind(arithmetic,c(mean(B1.mseF),arithmetic.df))

    #trimmed mean
    ord.B1mseF<-order(B1.mseF)
                          #order the B1.mseF
    trimmed.points<-ceiling((trimmed.propotion*num.run[i])/2)
                          #determine the number of points to trim off
                          #trimmed.propotion:trimmed propotion
                          # trimmed.points: how many points which are
                          # trimmed away of each end 
    if(num.run[i]<=2*trimmed.points)
      stop("The trimmed.mean is too big!!! Reduce it")
    weight<-rep(c(FALSE,TRUE,FALSE),
                times=c(trimmed.points,
                  (num.run[i]-2*trimmed.points),trimmed.points))
                          #using weights to give the trimmed mean
                          # effect
    trimmed.df<-(sum(B1.mseF[ord.B1mseF][weight])^2/
                 sum((B1.mseF[ord.B1mseF][weight])^2/
                     (B1.Fdf2[ord.B1mseF][weight])))
                          #is the degrees of freedom based on
                          # Sattherwaite approximation      
    weight<-weight/(num.run[i]-2*trimmed.points)
    trimmed<-rbind(trimmed,
                        c(weighted.mean(B1.mseF[ord.B1mseF],weight),
                          trimmed.df))
                          #weighted.mean compute the weighted mean of
                          # a vector
      
    #the median of the the B1.neighbours mse
    med<-c(med,median(B1.mseF)) 
    }
    
  #Naming the columns of the runs
  colnames(tukey)<-c("tukey.mean","satter.df")
  colnames(trimmed)<-c("trimmed.mean","satter.df")
  colnames(arithmetic)<-c("normal.mean","satter.df")
    
  return(list(tukey=tukey,
              trimmed=trimmed,
              arithmetic=arithmetic,
              med=med,
              num.run=num.run,
              B.mse=c(B,B1),
              trimmed.propotion.mse=c(trimmed.propotion,
                2*trimmed.points),
              mz=mlmProSpect$mz,
              mlmProSpect=mlmProSpect$mlmF.dat,
              expt=mlmProSpect$expt,
              num.points=mlmProSpect$num.points,
              sample=mlmProSpect$sample,
              range=mlmProSpect$range))

  #Output of smooth.mse:
  #It outputs a list with
  #-$tukey is a matrix with 1st col: robust.mean and 2nd col:
  # satter.df
  #-$trimmed is a matrix with 1st col: trimmed and 2nd col:
  # satter.df
  #-$arithmetic is a matrix with 1st col: mean and 2nd col: satter.df
  #-$med is a vector of the median MSE
  #-$num.run is a vector indicating the number of neighbours used to
  # calculated mean or median
  #-$B.mse is a 2dim vector: 1st entry is the percentage of neighbours
  # and 2nd entry is the actual number of neighbours
  #-$trimmed.propotion is a 2dim vector: 1st entry is the proportion
  # to trim and 2nd entry is the actual number trimed off
  #-$mz is a matrix containing all the summary statistics of m/z of
  # the various windows.
  #-$mlmProSpect is a matrix containing all the F-statistics and its
  # components.
  #-expt is a string that is the name of the experiment
  #-num.points is a scalar that tells the number of paired
  # observations (m/z,intensity) for each window's spectrum.
  #-sample is a vector of strings and each entry correspond to the
  # name of a spectrum.
  #-range is a vector that indicate the row values corresponding to
  # the output of read.prot that are used in the analysis

}

