template.Info<-function(data.default=data.default,row.idPeakPCA=row.id,shift.iter=shift.iter,PC=PC,
                        mz.limit=mz.limit,symm.limit=symm.limit,
                        warning.path=warning.path,cluster.id=cluster.id,idRef="template within PCA"){
  
  ##==============================================================================================================##
  ##Adjusting the mz values of the left valley, peak and right valley of each spectrum according to the shift term##
  ##Compute the variables "max.mz", "left.mz" and "right.mz".                                                     ##
  ##==============================================================================================================##
  adjMz<-apply(matrix(1:ncol(data.default),ncol=1),1,
               function(i,data.default,row.idPeakPCA,shift.iter){
                 ##The mz/ values of the cluster
                 y<-data.default[row.idPeakPCA,i,1]
                 ##The row indices of the cluster to 'data.default'
                 x<-row.idPeakPCA
                 ##Model the qaudratic relationship between the mz and row indice
                 model<-lm(y~x+x2,data=data.frame(y=y,x=x,x2=x^2))
                 ##The new row indice due to shift
                 xNew<-(row.idPeakPCA+shift.iter[i])
                 ##The predicted mz of the peak location for the spectrum
                 predict(model,newdata=data.frame(x=xNew,x2=xNew^2))
               },data.default=data.default,row.idPeakPCA=row.idPeakPCA,
               shift.iter=shift.iter)
  
  
  ##Extract the template intensity profiles from PCA. The mz of the points correspond to the average corrected mz.
  template<-cbind(rowMeans(adjMz),PC[,1])
  
  ##To determine the peak and the valleys of the template
  ##mz.limit<-0.05;symm.limit<-1.2;warning.path="C:\\"
  minmaxTemp<-locate.minmax(template,mz.limit=mz.limit,symm.limit=symm.limit,
                            warning.path=warning.path,cluster.id=cluster.id,idRef=idRef)

  ##Identify the row indices of minmaxTemp that corresponds to the peak of the template
  tempPeak<-which(minmaxTemp[,"max"]==1)
  
  if(nrow(minmaxTemp)>2){##There is only at least 2 valleys for each peak
    
    if(length(tempPeak)>1){##There are more than 2 peaks
      
      ##Choose the max pt with the largest intensity as the peak
      ##(i.e. row indices of 'minmaxTemp' corresponding to the Peak)
      tempPeak<-which.max(minmaxTemp[,"intensity"])
      ##The number of points to the left and right of the template's peak
      diffPt<-diff(c(1,minmaxTemp[tempPeak,"id"],length(row.idPeakPCA)))
      
    }else{##There are only 1 peak with 2 valleys
      
      ##The number of points to the left and right of the template's peak
      diffPt<-diff(c(1,minmaxTemp[tempPeak,"id"],length(row.idPeakPCA)))
    }
    
  }else{##There is only 1 valley for each peak
    ##The number of points to the left and right of the template's peak
    diffPt<-c(ifelse(tempPeak==1,0,length(row.idPeakPCA)),ifelse(tempPeak==2,0,length(row.idPeakPCA)))
    ##diffPt<-rep(ceiling(length(row.idPeakPCA)*0.5),2)
    
  }
  
  return(diffPt)
  
}

  
