"zoom.alpha" <-
function(cmod.F,prot.dat,clin.dat,
                    corr.name=c("median","arithmetic","tukey",
                      "trimmed"),pType=c("f","chisq","fchisq"),
                     sign.type=c("pValue","fdr.BH","fdr.BY"),
                     alpha.level=0.05) {
  num.points<-cmod.F$num.points
                          #choosen window range
                          #see mlm.ProSpect.R (standard = 10)
  
  # securitiy check if corr.name is properly written
  if(!corr.name%in%c("median","arithmetic","tukey","trimmed"))
    stop("the corr.name is not proper!!!")

  
  #===============================================================
  #To extract matrix that contains the p-value with the method of
  # smoothing specified by corr.name
  #===============================================================
  
  name.convert<-cbind(c("median","arithmetic","tukey","trimmed"),
                      c("cmod.F$cMed.mF","cmod.F$cArithmetic.mF",
                        "cmod.F$cTukey.mF","cmod.F$cTrimmed.mF"))
                          #cbind() creates a matrix
                          #1st col: usable names of corr.name
                          #2nd col: object names of cmod.F
  name.i<-name.convert[,1]%in%corr.name
                          #each choosen corr.name by user will
                          # saved in name.i

  eval(parse(text=paste("select.F<-",name.convert[name.i,2],sep="")))
                          #select.F contains each converted name from
                          # name.i in the object name of cmod.F
                          #parse(): returns the parsed but unevaluated
                          # expressions in a list.
                          # e.g.:expression(select.F <- cmod.F$cMed.mF)
                          #eval(): Evaluate an R expression in a
                          # specified environment.
                          # e.g: returns the content of select.F/
                          # cmod.F$cMed.mF

  #======================================================
  # get to the right column of the p-values of fdr values
  #======================================================

  # when it is p-value  
  if(sign.type=="pValue"){
    p.convert<-cbind(c("f","chisq","fchisq"),
                     c("pValue.f","pValue.c","pValue.fc"))
    
    # If the MSE is smoothed using median
    if(cmod.F$smoothingMean.type=="median"){
      pValue<-"pValue" 
     # If the MSE is smoothed by using mean  
    }else{
      pValue<-p.convert[p.convert[,1]%in%pType,2]
    }
    
   # when the FDR is used    
  }else{
    p.convert<-cbind(c("f","chisq","fchisq"),
                     c(".f",".c",".fc"))
    
    # If the MSE is smoothed using median
    if(cmod.F$smoothingMean.type=="median"){
      pValue<-sign.type
     # If the MSE is smoothed by using mean  
    }else{
      pValue<-paste(sign.type,p.convert[p.convert[,1]%in%pType,2],sep="")
    }
  }
  alpha<-which(as.vector(select.F[,pValue]<alpha.level))
                          #alpha contains all rows of select.F for
                          # whose p values are under the choosen
                          # alpha value
                          #as.vector() convert to a vector
 
  # check if there are some values in select.F with a p value
  #  over the choosen alpha level.
  if(length(alpha)==0){
    warning(paste("There are no significant values at all.",
                  "Pleasencrease the level of significance. ",
                  "data: ",cmod.F$expt,
                  " and ","pts(",cmod.F$num.points,
                  ") B.sMse(",cmod.F$B.sMse[1],"%) B.corr(",
                  cmod.F$B.corr[1],
                  "%) trimmedProportion.sMse(",
                  cmod.F$trimmedProportion.sMse[1],
                  ") trimmedProportion.corr(",
                  cmod.F$trimmedProportion.corr[1],
                  ") F'(",corr.name,") ",
                  "MSE(",cmod.F$smoothingMean.type,")",sep=""))
        return(NULL)
  }
  # quality check if the input values are properly specified
  #  if both are not median then only one of the distributions exists
  #  (f OR chisquare distribution, not both)
  if(corr.name!="median" & cmod.F$smoothingMean.type!="median"
     & pValue=="pValue.fc")
    stop('"fchisq" is not suitable for the combination of modification')
  
  #=========================================
  #See if cmod.F and prot.dat are compatible
  # see mlm.ProSpect.R
  #=========================================

#=======================================================
#TODO: usedCount.window as return value in each function
#      since mlm.ProSpect()
#=======================================================
  
    max.window<-ceiling(dim(prot.dat$loess.dat)[1]/num.points)
                          #dim(loess.dat)[1]: number of rows in 3dim
                          # array loess.dat
                          #max.window: the max number of possible
                          # windows

  # quality check: To avoid that a window with only one mz value per
  #  spectrum exists it is better to use (max.window-1) instead of
  #  max.windows.
  lonlyValue<-((max.window-1)*num.points+1)-dim(prot.dat$loess.dat)[1]
  usedCount.window<-ifelse((lonlyValue==0),(max.window-1),max.window)
  
  # security check if you have so many mean values like count of used
  #  windows
  if(dim(select.F)[1]!=usedCount.window){
    stop("proteomic data length/num.points and length of F do not match!!!")
  }
  return(list(id=alpha,
              sign.select.F=select.F[alpha,],  #should be renamed in sign.select.F
              mz=cmod.F$mz[alpha,],
              smoothingMean.type=cmod.F$smoothingMean.type,
              corrF.type=corr.name,
              num.run.Fprime=cmod.F$num.run.Fprime,
              num.run.sMse=cmod.F$num.run.sMse,
              B.corr=cmod.F$B.corr,
              B.sMse=cmod.F$B.sMse,
              trimmedProportion.corr=cmod.F$trimmedProportion.corr,
              trimmedProportion.sMse=cmod.F$trimmedProportion.sMse,
              df1=cmod.F$df1,
              df2=cmod.F$df2,
              expt=cmod.F$expt,
              num.points=cmod.F$num.points,
              range=cmod.F$range,
              pType=pType,
              sign.type=sign.type,
              select.F=select.F,
              pValue=pValue))
}

