\name{ProSpect.README}
\alias{ProSpect.README}

\title{How to start with the R package ProSpect}

\description{To use ProSpect as stand alone package there are 3 key functions.
Named  "findPeaks()", "plotPeaks()" and "summaryPeaks()".}
\details{
findPeaks() imports the .csv files which were exported from
Ciphergen's software before. After this, calculation for finding the
significant peaks begins. The last step is to create a ccl file in the
same folder where the .csv files are. This .ccl file can be imported
into Ciphergen's software.

plotPeaks() creates plots and save them as .pdf files. These files are found
in the folder where the .csv files are stored. The default plot shows the user all spectra
together in the range of interest and with the significant peaks labeled.

summaryPeaks() returns a short statistical summary of the data after
different calculation steps. For example, the summary of the data is
obtained after importing the .csv files from Ciphergen's software.

For further information please check the help files of the package or type e.g. help.search("findPeaks").

The example in the next section uses the path to the library folder for the standard installation of R for Windows.

The data set provided here contains only 4 of 8 spectra which were used for the article.
So results are only similar, but exactly not the same.

Please make sure you have the following R packages: rsmooth and quadprog.
}
\references{

  Tan CS, Ploner A, Quandt A, Lehtio J, Pawitan Y. 2006. Finding
  regions of significance in SELDI measurements for identifying protein
  biomarkers. \emph{Bioinformatics}, \bold{22(12)}:1515--23.
  
  Tan CS, Ploner A, Quandt A, Lehtio J, Pernemalm M, Lewensohn R,
  Pawitan Y. 2006. Annotated regions of significance of SELDI-TOF-MS
  spectra for detecting protein biomarkers. \emph{Proteomics},
  \bold{6(23)}:6124--33
  
  Quandt A, Ploner A, Tan CS, Lehtio J, Pawitan Y. 2005. ProSpect: An
  R Package for Analyzing SELDI Measurements Identifying Protein
  Biomarkers. \emph{Lecture Notes in Computer Science}, \bold{3695}:140--150.
  
 }
\examples{
# R library folder
jj <- .libPaths()[1]

#Path to the example folder
pathToExample <- file.path(jj,"ProSpect","SpikeInData/")

#stores results of calculation in object output
output <- findPeaks(fullFolderName=pathToExample,
region=c(5.5,6),outputName="SpikeInData")

#plot significant peaks
plotPeaks(dat=output,plotWhat="significant",
fileName="SpikeInData",colour=FALSE,winSizePlot=.5)

#statistical summary to imported data
summaryPeaks(object=output,data="read")
}
\author{
Tan Chuen Seng, Andreas Quandt, Alexander Ploner and Yudi
Pawitan
}
\keyword{ProSpect}% at least one, from doc/KEYWORDS
