\name{findPeaks}

\alias{findPeaks}
\alias{ccl.file.merge}
%\alias{}
%\alias{}
%\alias{}
%\alias{}
%\alias{}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{Peak detection and quantification of significant regions within
  the given mass spectra}
\description{
  The mass spectrometry spectra to be analysed are imported via the
  .csv files from Ciphergen's software. After that, the algorithms for
  peak detection and peak quantification is being run. The results are
  exported in a .ccl file which could be re-imported in Ciphergen's
  software for further analysis if neccessary.
}
\usage{
findPeaks(fullFolderName = NULL, region = NULL, outputName = NULL,
saveSettings = TRUE, doBackground = TRUE, smoother = "rsmooth",
bandwidth = 5, subsample = 10, winSize = 5, B.mse = 10, B.Fprime = 10,
mean = 1, distr = "chisq", signCriterion = "fdr.BH", signLevel =
0.05, mz.limit=0.05, symm.limit=1.3, toler=0.000001, maxiter=20, 
msetol=0.975,parPeak=TRUE,par.maxiter=100,plots.TF=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fullFolderName}{Path to the folder which contains the .csv
    files of the mass spectra}
  \item{region}{Region of interest in kDa}
  \item{outputName}{Name of the files created by \code{findPeaks()} }
  \item{saveSettings}{Creates .settings file within the named
    folder. File contains the used set of arguments for the last run.}
  \item{doBackground}{Normalisation of the given data}
  \item{smoother}{The smoother which has to be used for the
    normalisation step\cr
    \code{rsmooth}\cr
    \code{loess}}
  \item{bandwidth}{Specifies the bandwith for the smoother \code{rsmooth}}
  \item{subsample}{Specifies how much values have to be used for the
    calculation of normalisation curve by using the smoother
    \code{loess}(100\% values/ subsample )}
  \item{winSize}{Window size to calculate the MSE values of the F-statistics}
  \item{B.mse}{This is the percentage of nearest MSE among all MSE's. That means B is used for the calculation of MSE mean.
    B is a tuning parameter and choose how many values are basis for the mean values. }
  \item{B.Fprime}{This is the percentage of nearest F' among all F's}
  \item{mean}{The combination of  mean values to calculate F' and F*\cr
    '1' = Median / Median\cr
    '2' = Mean / Median\cr
    '3' = Mean / Mean\cr
    '4' = Median / Mean}
  \item{distr}{The combination of distributions to calculate F' and
    F*\cr
    'chisq' = using Chisquare distribution for both\cr
    'fchisq' = using F and Chisquare distribution for F' and F*\cr
    'f' = using F distribution for both}
  \item{signCriterion}{Specifies the sign criterion which is used to
    detect the significant peaks\cr
    'pValue'\cr
    'fdr.BY'\cr
    'fdr.BH'
  }
  \item{signLevel}{Specifies the sign level alpha}
  \item{mz.limit}{A scalar indicating the percentage limit of the mz
    that assumes the max or min points are too close.Coombes used 0.05\% of the smaller mass.    
    (Tuning paramters used in identify peaks and valleys in a spectrum) }
  \item{symm.limit}{ A scalar that tells us the max ratio of the
    difference in mz between the max and 2 mins. The max difference in
    mz is in the numerator. (Tuning paramters used in identify peaks and valleys in a spectrum) }
  \item{toler}{The tolerance level for the extraction of template via
    PCA (values between 0 and 1).}
  \item{maxiter}{The maximum number of iterations for the extraction of
    template via PCA.}
  \item{msetol}{The value for flagging the fitting of peaks with bad
    quality via the scaled MSE (values between 0 and 1). It can be seen
    as the proportion of peaks detected that are expected to be bad.}
  \item{parPeak}{Logical. Perform a parametric approach to identify the
    peak intensity and m/z.}
  \item{par.maxiter}{The value for the number of iteration for the
    EM-algorithm og the parametric approach.}
  \item{plots}{The logical value for plotting the diagnostics plots (default=TRUE).}
}
\details{
  To get the peak intensities from ARS approach, use the function peakInfoIntMz.
}
\value{
 It creates two directories, warning and diagnostic. They contain a
 warning.txt and .pdf files respectively. The warning.txt keeps track of
 warning from the algorithm while the .pdf files contains diagnostic
 plots to determine if the fits are reasonable.\cr\cr
 It also returns a list:
 \item{read.dat}{An array containing the mz and intensity of the
   spectra. The rows (1st-dimension) corresponding to the no. of TOF points, columns (2nd-dimension) corresponding to the spectra and the 3rd-dimension corresponding to the mz (1st entry) and intensity (2nd entry) values.}
 \item{clin.dat}{If specified, a matrix containing clinical information. The rows should correspond with the spectra.}
 \item{sample}{List of characters of the .csv files from which the mz
   and intensity values of the spectra are read in.}
 \item{reduct.dat}{List from the function reduct.ProSpect(). Its components consist of\cr
  -$loess.dat: the spectra data array from $read.dat but reduced to the region of interest.\cr
  -$sample: see sample.\cr
  -$expt: a string describing the SELDI experiment (optional)}
 \item{loess.dat}{List from the function BgCorrection.ProSpect(). Its components consist of\cr
  -$loess.dat: a matrix containing background-corrected intensity values; rows are the no. of TOF points within the restricted region and columns are the no. of spectra.\cr
  -$fit.loess: a list containing a vector of the intensities of the background noise. The dimension of the list correspond to the number of spectra.\cr
  -$sample: see sample.\cr
  -$expt: see $reduct.dat$sexpt\cr
  -$range: the row indices of read.dat that correspond to the restricted data.\cr
  -$ndx: subset of points used for performing loess on the intensities.\cr}
 \item{mlm.dat}{List from the function mlm.ProSpect(). Its components consist of\cr
  -$mlmF.dat: a matrix with information on the one-way ANOVA. The row corresponds to the window sequence while the columns are F-statistics (statF), its numerators df (F.df1), denominator df (F.df2), p.value (pValueF), MSE (mseF) and MSR (msrF).\cr
  -$mz: a matrix with information on each window's m/z summary statistics. The row corresponds to the window sequence while the columns are the mean mz (mean.mz), sd mz (sd.mz), min mz (min.mz) and max mz (max.mz).\cr
  -$num.points: a scalar that tells the number of paired observations (m/z,intensity) for each window's spectrum.\cr
  -$expt: see reduct.dat$expt.\cr
  -$sample: see sample.\cr
  -$range: see loess.dat$range.}
 \item{modF.dat}{List from the function mod.F(). Its components consist of\cr
  -$median: a vector of the MSR/local median of MSE. (MSR and MSE correspond to the numerator and denominator of the F-statistics from mlm.dat).\cr
  -$arithmetic: a matrix with 1st col: MSR/local mean and 2nd col: satter.df.\cr
  -$tukey: a matrix with 1st col: MSR/tukey mean and 2nd col: satter.df.\cr
  -$trimmed: a matrix with 1st col: MSR/local trimmed mean and 2nd col: satter.df.\cr
  -$df1: the df for the numerator of the modified F-statistics (F').\cr
  -$mz: see mlm.dat$mz.\cr
  -$trimmed.sMse: a matrix with 1st col: local trimmed mean and 2nd col: satter.df.\cr
  -$tukey.sMse: a matrix with 1st col:tukey mean and 2nd col: satter.df.\cr
  -$arithmetic.sMse: a matrix with 1st col: local mean and 2nd col: satter.df.\cr
  -$med.sMse: a vector of local median of MSE.\cr
  -$num.run.sMse: a vector indicating the number of neighbours used to calculate mean or median.\cr
  -$B.sMse: a vector of length 2 where 1st entry is the percentage of neighbours and 2nd entry is the actual number of neighbours.\cr
  -$trimmed.propotion.sMse: a vector of length 2 where the 1st entry is the proportion to trim and 2nd entry is the actual number trimed off.\cr
  -$expt: see reduct.dat$expt\cr
  -$num.points: see mlm.dat$num.points.\cr
  -$sample: see sample.\cr
  -$range: see loess.dat$range.\cr}
 \item{modFcorrFdr.dat}{List from the function mod.Fcorr.fdr(). Its components consist of\cr
  -$cArithmetic.mF: a matrix of with the rows corresponding to the window sequence while the columns are the final modified F-statistcs (F*) by arithmetric (c.mF), its p-value (pValue), its fdr from BH (fdr.BH), and BY (fdr.BY).\cr
  -$cTrimmed.mF: same as $cArithmetic.mF but final modified F-statistcs (F*) by trimmed mean.\cr
  -$cTukey.mF: same as $cArithmetic.mF but final modified F-statistcs (F*) by tukey mean.\cr
  -$cMed.mF: same as $cArithmetic.mF but final modified F-statistcs (F*) by median.\cr
  -$scalar.trimmed: a vector of the trimmed mean of the modified F-statistics (F').\cr
  -$scalar.tukey: a vector of the tukey mean of the modified F-statistics (F').\cr
  -$scalar.arithmetic: a vector of the arithmetic mean of the modified F-statistics (F').\cr
  -$scalar.med: a vector of the median of the modified F-statistics (F').\cr
  -$num.run.Fprime: a vector indicating the number of neighbours used to calculate mean or median of F'.\cr
  -$num.run.sMse: see $modF.dat$num.run.sMse.\cr
  -$B.corr: a vector of length 2 where 1st entry is the percentage of neighbours and 2nd entry is the actual number of neighbours when taking the mean/median of F'.\cr
  -$B.sMse: see modF.dat$B.sMse.\cr
  -$trimmedProportion.corr: a vector of length 2 where the 1st entry is the proportion to trim and 2nd entry is the actual number trimed off for F'.\cr
  -$trimmedProportion.sMse: see modF.dat$trimmed.propotion.sMse.\cr
  -$df1: the df of the numerator of the final modified F-statistics (F*).\cr
  -$smoothingMean.type: a string indicating the type of smoothing for the MSE of the F-statistics from $modF.dat.\cr
  -$df2: the df of the denominator of the final modified F-statistics (F*). Provided the median is not used to smooth F'.\cr
  -$mz: see mlm.dat$mz.\cr
  -$expt: see reduct.dat$expt\cr
  -$num.points: see mlm.dat$num.points.\cr
  -$sample: see sample.\cr
  -$range: see loess.dat$range.\cr}
 \item{zoom.dat}{List from the function zoom.alpha(). Its components consist of\cr
  -$id: a vector indicating the row indices of either $cArithmetic.mF,
  or $cTrimmed.mF, $cTukey.mF, or $cMed.mF from $modFcorrFdr.dat,
  depending on the type of correction wanted.\cr
  -$sign.select.F: subset of either $cArithmetic.mF, or $cTrimmed.mF, $cTukey.mF, or $cMed.mF from $modFcorrFdr.dat that are significant.\cr
  -$mz: subset of mlm.dat$mz that are significant.\cr
  -$smoothingMean.type: a string indicating the type of smoothing on the MSE of the F-statistics.\cr
  -$corrF.type: a string indicating the type of smoothing on the F'.\cr
  -$num.run.Fprime: see modFcorrFdr.dat$num.run.Fprime.\cr
  -$num.run.sMse: see modF.dat$num.run.sMse.\cr
  -$B.corr: see modFcorrFdr.dat$B.corr.\cr
  -$B.sMse: see modF.dat$B.sMse.\cr
  -$trimmedProportion.corr: see modFcorrFdr.dat$trimmedProportion.corr.\cr
  -$trimmedProportion.sMse: see modF.dat$trimmed.propotion.sMse.\cr
  -$df1: see modFcorrFdr.dat$df1\cr
  -$df2: see modFcorrFdr.dat$df2\cr
  -$expt: see reduct.dat$sexpt\cr
  -$num.points: see mlm.dat$num.points.\cr
  -$range: see loess.dat$range.\cr
  -$pType: a string indicating the type of modified F-statistics used.\cr
  -$sign.type: a string indicating the type of fdr used.\cr
  -$select.F: Either $cArithmetic.mF, or $cTrimmed.mF, $cTukey.mF, or $cMed.mF from $modFcorrFdr.dat.\cr
  -$pValue: a string indicating the type of fdr used.\cr}
 \item{quantify.dat}{List from the function quantifyPeaks(). Its components consist of\cr
   -$arrayInfo/arrayFinal (arrayFinal sets NA to those MSE>msetol):
   Array consisting of the peak detected in the regions that are
   significant with the estimates of the peak's intensity and mz
   obtained via ARS (see second reference). The rows correspond to the the number of peaks
   detected in the entire region, the columns the spectra and the 3rd
   dimension consist of the different information of the peak. The 1st
   entry is the finalized peak mz value (max.mz), the peak mz value from
   the PCA when extracting the template (max.mzPC), the finalized
   optimum shift term from the least-squares with mixture model approach
   (step), the the number of points used to fit the spectrum (df), the
   finalized left mz value (left.mz) and right mz value (right.mz) of
   the valley of the peak, the finalized unadjusted peak intensity value
   from the mixture model approach (max.ampMix), the scale parameter
   from the mixture model approach (scaleMix), the scaled mean sum of
   squares of error from the mixture model approach (mseMix), the
   unadjusted peak intensity value from PCA when extracting the template
   (max.ampPC), optimum shift term from the PCA when extracting the
   template (shiftPC), the estimated scale parameter from PCA (scalePC),
   the the left mz value (left.mzPC) and right mz value(right.mzPC) of
   the valley of the peak from the PCA when extracting the template, the
   smallest negative intensity for the cluster (min.int), the excess
   number of points to the right of the  cluster to ensure the fitted
   spectrum do not go out of the cluster region (excessPt).\cr
  -$qualityTemplate: Array with the rows corresponding to the number
  of peaks and the columns correspond to the cluster identity
  (cluster.id), the peak identity within a cluster (peak.id), the
  dominant spectrum identity (idRef), the counting the number of times a
  spectrum is rejected as reference spectrum (rank), the ratio between
  the range of the peak and the valley with the noise level
  (noise.ratio), the template spectrum, peak intensity value
  (tempPeakInt), the quality of the template spectrum (qc1), the
  absolute difference between the percentage of variation explained by
  the first PC (convPer), the mean square difference between the first
  PC of consecutive iterations (convPC), the number of iteration for the
  PCA (iter), the left mz value (domPeakLeftMz) and right mz value
  (domPeakRightMz) of the valley of the dominant spectrum peak, the left
  row indices (idTempL) and right row indices (idTempR) of the template
  spectrum.\cr
  -$templateInfo: List consisting of matrices with the mz and
  'intensity' profiles of the template obtained from the PCA. The number
  of components corresponding to the number of clusters and the number
  of components within each cluster corresponds to the number of peaks.\cr
  -$infoPA: Array consisting of the peak detected in the regions that are
   significant with the estimates of the peak's intensity and mz
   obtained via parametric approach by assuming a log-normal peak
   shape. The rows correspond to the the number of peaks detected in the
   entire region, the columns the spectra and the 3rd dimension consist
   of the different information of the peak. The 1st entry is the
   estimated shift parameter s of the log-normal distribution (sEst),
   the estimated mu parameter of the log-normal distribution (muEst),
   the estimated sigma parameter of the log-normal distribution
   (sigmaEst), the estimated mixing probabilities (pEst), the area of
   the observed spectrum after making it non-negative (area.spect), the
   smallest negative intensity for the entire cluster to make it
   non-negative (min.min.int), indicator whether there is convergence in
   the EM-aglorithm and NA if EM-aglorithm was not used (converge), and
   the number of iteration used in the EM-algorithm (iter).\cr
   }
 \item{region}{Vector indicating the mz range (in Da) in which the
   analysis was performed.}
 \item{folder}{String indicating the working directory (i.e. the
   spectrum information is stored.)}
 \item{smoother}{String indicating the type of smoothing to remove the
   background noise.}
 \item{signLevel}{FDR cut-off used to flag whether a window is significant.}  
}
\references{

  Tan CS, Ploner A, Quandt A, Lehtio J, Pawitan Y. 2006. Finding
  regions of significance in SELDI measurements for identifying protein
  biomarkers. \emph{Bioinformatics}, \bold{22(12)}:1515--23.
  
  Tan CS, Ploner A, Quandt A, Lehtio J, Pernemalm M, Lewensohn R,
  Pawitan Y. 2006. Annotated regions of significance of SELDI-TOF-MS
  spectra for detecting protein biomarkers. \emph{Proteomics},
  \bold{6(23)}:6124--33
  
  Quandt A, Ploner A, Tan CS, Lehtio J, Pawitan Y. 2005. ProSpect: An
  R Package for Analyzing SELDI Measurements Identifying Protein
  Biomarkers. \emph{Lecture Notes in Computer Science}, \bold{3695}:140--150.
  
}
\seealso{ \code{\link{read.ProSpect}},
  \code{\link{reduct.ProSpect}}, \code{\link{BgCorrection.ProSpect}},
  \code{\link{mlm.ProSpect}}, \code{\link{mod.F}},
  \code{\link{modFcorr.fdr}}, \code{\link{zoom.alpha}},
  \code{\link{quantifyPeaks}}, \code{\link{arrayInfoPA}}}
\examples{
# R library folder
jj <- .libPaths()[1]

#Path to the example folder
pathToExample <- file.path(jj,"ProSpect","SpikeInData/")

#stores results of calculation in object output
output <- findPeaks(fullFolderName=pathToExample,
region=c(5.5,6),outputName="SpikeInData")
}
\author{
Tan Chuen Seng, Andreas Quandt, Alexander Ploner and Yudi
Pawitan
}
\keyword{ utilities}% at least one, from doc/KEYWORDS

