\name{mlm.ProSpect}

\alias{mlm.ProSpect}


%\alias{}
%\alias{}
%\alias{}
%\alias{}
%\alias{}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{Perform one-way ANOVA on the spectra}
\description{
  Perform a one-way ANOVA on each non-overlapping and sequential windows
  in the spectra.
}
\usage{
mlm.ProSpect(loessProSpect, num.points = 10) 
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{loessProSpect}{Object from either the function of reduct.ProSpect(),
    if no further background correction was wanted, or
    BgCorrection.ProSpect(), otherwise.}
  \item{num.points}{Window size of the non-overlapping and sequential windows}
}
%\details{
%  
%}
\value{
 Returns a list:
 \item{mlmF.dat}{a matrix with information on the one-way ANOVA. The
    row corresponds to the window sequence while the columns are
    F-statistics (statF), its numerators df (F.df1), denominator df
    (F.df2), p.value (pValueF), MSE (mseF) and MSR (msrF).}
 \item{mz}{a matrix with information on each window's m/z summary
    statistics. The row corresponds to the window sequence while the
    columns are the mean mz (mean.mz), sd mz (sd.mz), min mz (min.mz)
    and max mz (max.mz).}
 \item{num.points}{a scalar that tells the number of paired
    observations (m/z,intensity) for each window's spectrum.}
 \item{expt}{a string describing the SELDI experiment.}
 \item{sample}{list of characters of the .csv files from which the mz
   and intensity values of the spectra are read in.}
 \item{range}{the row indices from read.ProSpect that correspond to the restricted data.}
}
\references{

  Tan CS, Ploner A, Quandt A, Lehti\"o J, Pawitan Y. 2006. Finding
  regions of significance in SELDI measurements for identifying protein
  biomarkers. \emph{Bioinformatics}, 22(12):1515--23.

  Quandt A, Ploner A, Tan CS, Lehti\"o J, Pawitan Y. 2005. ProSpect: An
  R Package for Analyzing SELDI Measurements Identifying Protein
  Biomarkers. \emph{Lecture Notes in Computer Science}, 3695:140--150.
  
}
%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
# R library folder
jj <- .libPaths()[1]

#Path to the example folder
pathToExample <- file.path(jj,"ProSpect","SpikeInData/")

#Read in the .csv files from Ciphergen
read.dat <- read.ProSpect(spectra.dir=pathToExample,expt.name="An example
data")

#Reduce and align the spectra
reduct.dat <- reduct.ProSpect(dat=read.dat[[1]],range.mz=c(5000,6000))

#Perform background correction on the reduced and aligned spectra
loess.dat <- BgCorrection.ProSpect(dat=reduct.dat)

#Perform one-way ANOVA on the windows
mlm.dat <- mlm.ProSpect(loess.dat,num.points=5)

}
\author{
Tan Chuen Seng, Andreas Quandt, Alexander Ploner and Yudi
Pawitan
}
\keyword{ utilities}% at least one, from doc/KEYWORDS

