\name{mod.F}

\alias{mod.F}
\alias{smooth.mse}
\alias{tukey.biwt}

%\alias{}
%\alias{}
%\alias{}
%\alias{}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{Smooth the MSE from the one-way ANOVA}
\description{
  Smooth the MSE from the one-way ANOVA by using a running (robust) mean or
  median (i.e. MSE').
}
\usage{
mod.F(mlmProSpect, B = 10, t.prop = 0.1) 
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mlmProSpect}{Object from the function mlm.ProSpect().}
  \item{B}{This is the percentage of nearest MSE among all MSE's. That means B is used for the calculation of MSE means.
    B is a tuning parameter and choose how many values are basis for the mean values. }
  \item{t.prop}{The proportion of the data to be trimmed off for the
    trimmed mean.}
}
\details{
The robust means considered in this function includes the trimmed mean
and tukey biweight.
}
\value{
 Returns a list:
 \item{median}{a vector of the MSR/local median of MSE. (MSR and MSE correspond to the numerator and denominator of the F-statistics from mlm.dat).}
 \item{arithmetic}{a matrix with 1st col: MSR/local mean and 2nd col:
   satter.df.}
 \item{tukey}{a matrix with 1st col: MSR/tukey mean and 2nd col: satter.df.}
 \item{trimmed}{a matrix with 1st col: MSR/local trimmed mean and 2nd
   col: satter.df.}
 \item{df1}{the df for the numerator of the modified F-statistics (F').}
 \item{mz}{a matrix with information on each window's m/z summary
   statistics. The row corresponds to the window sequence while the
   columns are the mean mz (mean.mz), sd mz (sd.mz), min mz (min.mz)
   and max mz (max.mz).}
 \item{trimmed.sMse}{a matrix with 1st col: local trimmed mean and 2nd
   col: satter.df.}
 \item{tukey.sMse}{a matrix with 1st col:tukey mean and 2nd col:
   satter.df.}
 \item{arithmetic.sMse}{a matrix with 1st col: local mean and 2nd col:
   satter.df.}
 \item{med.sMse}{a vector of local median of MSE.}
 \item{num.run.sMse}{a vector indicating the number of neighbours used
   to calculate mean or median.}
 \item{B.sMse}{a vector of length 2: 1st entry is the percentage of
   neighbours and 2nd entry is the actual number of neighbours.}
 \item{trimmed.propotion.sMse}{a vector of length 2: 1st entry is the
   proportion to trim and 2nd entry is the actual number trimed off.}
 \item{expt}{a string describing the SELDI experiment.}
 \item{num.points}{a scalar that tells the number of paired
   observations (m/z,intensity) for each window's spectrum.}
 \item{sample}{list of characters of the .csv files from which the mz
   and intensity values of the spectra are read in.}
 \item{range}{the row indices from read.ProSpect that correspond to the
   restricted data.}
}
\references{

  Tan CS, Ploner A, Quandt A, Lehtio J, Pawitan Y. 2006. Finding
  regions of significance in SELDI measurements for identifying protein
  biomarkers. \emph{Bioinformatics}, \bold{22(12)}:1515--23.
  
  Quandt A, Ploner A, Tan CS, Lehtio J, Pawitan Y. 2005. ProSpect: An
  R Package for Analyzing SELDI Measurements Identifying Protein
  Biomarkers. \emph{Lecture Notes in Computer Science}, \bold{3695}:140--150.
  
}
\seealso{\code{\link{findPeaks}}}
\examples{
# R library folder
jj <- .libPaths()[1]

#Path to the example folder
pathToExample <- file.path(jj,"ProSpect","SpikeInData/")

#Read in the .csv files from Ciphergen
read.dat <- read.ProSpect(spectra.dir=pathToExample,expt.name="An example
data")

#Reduce and align the spectra
reduct.dat <- reduct.ProSpect(dat=read.dat[[1]],range.mz=c(5000,6000))

#Perform background correction on the reduced and aligned spectra
loess.dat <- BgCorrection.ProSpect(dat=reduct.dat)

#Perform one-way ANOVA on the windows
mlm.dat <- mlm.ProSpect(loess.dat,num.points=5)

#Smooth the MSE from the one-way ANOVA
modF.dat <- mod.F(mlm.dat,10)

}
\author{
Tan Chuen Seng, Andreas Quandt, Alexander Ploner and Yudi
Pawitan
}
\keyword{ utilities}% at least one, from doc/KEYWORDS

