\name{modFcorr.fdr}

\alias{modFcorr.fdr}

%\alias{}
%\alias{}
%\alias{}
%\alias{}
%\alias{}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{Smooth the F-statistics (F'=MSR/MSE').}
\description{
  Smooth the F-statistics (F'=MSR/MSE') and compute the false discovery
  rates (FDR) based on the modified F-statistics (F*).
}
\usage{
modFcorr.fdr(mod.F, name = c("median", "arithmetic", "turkey", "trimmed"), 
    B = 10, t.prop = 0.1, proc = c("BH", "BY")) 
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mod.F}{Object from the function mod.F().}
  \item{name}{String specifying the type of smoothing on MSE: "median",
    "arithmetic", "turkey" or "trimmed".}
  \item{B}{This is the percentage of nearest F' among all F's}
  \item{t.prop}{The proportion of the data to be trimmed off for the
    trimmed mean.}
  \item{proc}{Specifies the type of FDR to compute: Benjamini and
    Hochberg (1995) 'fdr.BH', or, Benjamini and Yekutieli (2001)
    'fdr.BY'.}
}
%\details{
%  
%}
\value{
 Returns a list:
 \item{cArithmetic.mF}{a matrix of with the rows corresponding to the
   window sequence while the columns are the final modified F-statistcs
   (F*) by arithmetric (c.mF), its p-value (pValue), its fdr from BH
   (fdr.BH), and BY (fdr.BY).}
 \item{cTrimmed.mF}{same as $cArithmetic.mF but final modified
   F-statistcs (F*) by trimmed mean.}
 \item{cTukey.mF}{same as $cArithmetic.mF but final modified F-statistcs
   (F*) by tukey mean.}
 \item{cMed.mF}{ same as $cArithmetic.mF but final modified F-statistcs
   (F*) by median.}
 \item{scalar.trimmed}{ a vector of the trimmed mean of the modified
   F-statistics (F').}
 \item{scalar.tukey}{ a vector of the tukey mean of the modified
   F-statistics (F').}
 \item{scalar.arithmetic}{ a vector of the arithmetic mean of the
   modified F-statistics (F').}
 \item{scalar.med}{ a vector of the median of the modified F-statistics
   (F').}
 \item{num.run.Fprime}{ a vector indicating the number of neighbours
   used to calculate mean or median of F'. }
 \item{num.run.sMse}{a vector indicating the number of neighbours used
   to calculate mean or median.}
 \item{B.corr}{a vector of length 2: 1st entry is the percentage of
   neighbours and 2nd entry is the actual number of neighbours when
   taking the mean/median of F'. }
 \item{B.sMse}{a vector of length 2: 1st entry is the percentage of
   neighbours and 2nd entry is the actual number of neighbours.}
 \item{trimmedProportion.corr}{ a vector of length 2 where the 1st entry is the proportion to trim and 2nd entry is the actual number trimed off for F'.}
 \item{trimmedPropotion.sMse}{a vector of length 2: 1st entry is the
   proportion to trim and 2nd entry is the actual number trimed off.}
 \item{df1}{ the df of the numerator of the final modified F-statistics
   (F*).}
 \item{smoothingMean.type}{ a string indicating the type of smoothing
   for the MSE of the F-statistics from mod.F}
 \item{df2}{ the df of the denominator of the final modified
   F-statistics (F*). Provided the median is not used to smooth F'.}
 \item{mz}{a matrix with information on each window's m/z summary
   statistics. The row corresponds to the window sequence while the
   columns are the mean mz (mean.mz), sd mz (sd.mz), min mz (min.mz)
   and max mz (max.mz).}
 \item{expt}{a string describing the SELDI experiment.}
 \item{num.points}{a scalar that tells the number of paired
   observations (m/z,intensity) for each window's spectrum.}
 \item{sample}{list of characters of the .csv files from which the mz
   and intensity values of the spectra are read in.}
 \item{range}{the row indices from read.ProSpect that correspond to the
   restricted data.}

}
\references{

  Tan CS, Ploner A, Quandt A, Lehtio J, Pawitan Y. 2006. Finding
  regions of significance in SELDI measurements for identifying protein
  biomarkers. \emph{Bioinformatics}, \bold{22(12)}:1515--23.
  
  Quandt A, Ploner A, Tan CS, Lehtio J, Pawitan Y. 2005. ProSpect: An
  R Package for Analyzing SELDI Measurements Identifying Protein
  Biomarkers. \emph{Lecture Notes in Computer Science}, \bold{3695}:140--150.

  Benjamini Y, Hockberg Y. 1995. Controlling the false discovery rate: a
  practical and powerful approach to multiple testing. \emph{Journal of
    the Royal Statistical Society, B}, \bold{57}: 289--300.

  Benjamini Y, Yekutieli D. 1995. The control of the false discovery
  rate in multiple testing under dependency. \emph{Annals of
    Statistics}, \bold{29}:1165--88.
  
}
%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
# R library folder
jj <- .libPaths()[1]

#Path to the example folder
pathToExample <- file.path(jj,"ProSpect","SpikeInData/")

#Read in the .csv files from Ciphergen
read.dat <- read.ProSpect(spectra.dir=pathToExample,expt.name="An example
data")

#Reduce and align the spectra
reduct.dat <- reduct.ProSpect(dat=read.dat[[1]],range.mz=c(5000,6000))

#Perform background correction on the reduced and aligned spectra
loess.dat <- BgCorrection.ProSpect(dat=reduct.dat)

#Perform one-way ANOVA on the windows
mlm.dat <- mlm.ProSpect(loess.dat,num.points=5)

#Smooth the MSE from the one-way ANOVA
modF.dat <- mod.F(mlm.dat,10)

#Compute the modified F-statistics (F*)
modFcorrFdr.dat <- modFcorr.fdr(modF.dat,name="median",proc="BH")

}
\author{
Tan Chuen Seng, Andreas Quandt, Alexander Ploner and Yudi
Pawitan
}
\keyword{ utilities}% at least one, from doc/KEYWORDS

