\name{peakInfoIntMz}

\alias{peakInfoIntMz}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{Extract peak information}
\description{
  To extract the mz and intensity of the peaks identified by ARS and parametric approach.
}
\usage{
out<-peakInfoIntMz(quantify.dat,read.dat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{

  \item{quantify.dat}{Object from the function quantifyPeaks().}
  \item{read.dat}{Object from the function read.ProSpect().}

}
%\details{
%  
%}
\value{
 Returns a list:
 \item{infoARS}{ Array with rows corresponding to peaks quantified by ARS approach,
     columns correpsonding to spectra and the 3rd dimension corresponding to
     c("mz","intensity").}
 \item{infoPA}{ Same as infoARS but corresponds to peaks quantified by parametric
     approach. If parametric approach was not requested (i.e. parPeak=TRUE), it will be NULL}
}
\references{

   Tan CS, Ploner A, Quandt A, Lehtio J, Pernemalm M, Lewensohn R,
   Pawitan Y. 2006. Annotated regions of significance of SELDI-TOF-MS
   spectra for detecting protein biomarkers. \emph{Proteomics},
   \bold{6(23)}:6124--33

   Dijkstra M, Roelofsen H, Vonk RJ, Jansen RC. 2006. Peak quantification
   in surface-enhance laser desorption/ionization by using mixture
   models. \emph{Proteomics}, \bold{6(19)}:6124--33
  
}
%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
# R library folder
jj <- .libPaths()[1]

#Path to the example folder
pathToExample <- file.path(jj,"ProSpect","SpikeInData/")

#Read in the .csv files from Ciphergen
read.dat <- read.ProSpect(spectra.dir=pathToExample,expt.name="An example
data")

#Reduce and align the spectra
reduct.dat <- reduct.ProSpect(dat=read.dat[[1]],range.mz=c(5000,6000))

#Perform background correction on the reduced and aligned spectra
loess.dat <- BgCorrection.ProSpect(dat=reduct.dat)

#Perform one-way ANOVA on the windows
mlm.dat <- mlm.ProSpect(loess.dat,num.points=5)

#Smooth the MSE from the one-way ANOVA
modF.dat <- mod.F(mlm.dat,10)

#Compute the modified F-statistics (F*)
modFcorrFdr.dat <- modFcorr.fdr(modF.dat,name="median",proc="BH")

#Identify the significant windows
zoom.dat<- zoom.alpha(modFcorrFdr.dat,loess.dat,NULL,
                      corr.name="median",pType="chisq",sign.type="fdr.BH",
                      alpha.level=0.05)
                      
#Quantify the peak in the significant regions
quantify.dat<-quantifyPeaks(loess.dat=loess.dat,zoom.dat=zoom.dat,
                            fullFolderName=pathToExample)
                            
#Extract the quantified peaks in the significant regions
peaks.dat<-peakInfoIntMz(quantify.dat,read.dat)
                      
}
\author{
Tan Chuen Seng, Andreas Quandt, Alexander Ploner and Yudi
Pawitan
}
\keyword{ utilities}% at least one, from doc/KEYWORDS

