\name{quantifyPeaks}

\alias{quantifyPeaks}
\alias{windows.merge}
\alias{locate.minmax}
\alias{ObjFunctionMultiPeak}
\alias{pickSpect}
\alias{alignOptimMultiPeak}
\alias{alignSpec}

\alias{EMregion}
\alias{mix}
\alias{Mstep}
\alias{peakPA}

%\alias{}
%\alias{}
%\alias{}
%\alias{}
%\alias{}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{Quantify peaks in the significiant region}
\description{
  Quantify the peaks in the significant regions via either the ARS
  approach or the parametric approach by assuming the peak to be
  log-normal.
}
\usage{
quantifyPeaks(loess.dat=NULL,zoom.dat=NULL,fullFolderName=NULL,mz.limit=0.05,symm.limit=1.3,
         toler=0.000001,maxiter=20,msetol=0.975,parPeak=TRUE,par.maxiter=100,plots=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{loess.dat}{Object from either the function of reduct.ProSpect(),
    if no further background correction was used in computing the
    modified F-statistics, or BgCorrection.ProSpect(), otherwise.}
  \item{zoom.dat}{Object from the function zoom.alpha()}
  \item{fullFolderName}{Path to the folder which contains the .csv
    files of the mass spectra}
  \item{mz.limit}{A scalar indicating the percentage limit of the mz
    that assumes the max or min points are too close. Coombes used 0.05\% of the smaller mass.    
    (Tuning paramters used in identify peaks and valleys in a spectrum) }
  \item{symm.limit}{ A scalar that tells us the max ratio of the
    difference in mz between the max and 2 mins. The max difference in
    mz is in the numerator. (Tuning paramters used in identify peaks and valleys in a spectrum) }
  \item{toler}{The tolerance level for the extraction of template via
    PCA (values between 0 and 1).}
  \item{maxiter}{The maximum number of iterations for the extraction of
    template via PCA.}
  \item{msetol}{The value for flagging the fitting of peaks with bad
    quality via the scaled MSE (values between 0 and 1). It can be seen
    as the proportion of peaks detected that are expected to be bad.}
  \item{parPeak}{Logical. Perform a parametric approach to identify the
    peak intensity and m/z.}
  \item{par.maxiter}{The value for the number of iteration for the
    EM-algorithm og the parametric approach.}
  \item{plots}{The logical value for plotting the diagnostics plots (default=TRUE).}
}
%\details{
%  
%}
\value{
 Returns a list:
 \item{arrayFinal}{Array consisting of the peak detected in the regions that are
   significant with the estimates of the peak's intensity and mz
   obtained via ARS (see first reference). Those peaks that have
   MSE>msetol are set to NA. The rows correspond to the the number of peaks
   detected in the entire region, the columns the spectra and the 3rd
   dimension consist of the different information of the peak. The 1st
   entry is the finalized peak mz value (max.mz), the peak mz value from
   the PCA when extracting the template (max.mzPC), the finalized
   optimum shift term from the least-squares with mixture model approach
   (step), the the number of points used to fit the spectrum (df), the
   finalized left mz value (left.mz) and right mz value (right.mz) of
   the valley of the peak, the finalized unadjusted peak intensity value
   from the mixture model approach (max.ampMix), the scale parameter
   from the mixture model approach (scaleMix), the scaled mean sum of
   squares of error from the mixture model approach (mseMix), the
   unadjusted peak intensity value from PCA when extracting the template
   (max.ampPC), optimum shift term from the PCA when extracting the
   template (shiftPC), the estimated scale parameter from PCA (scalePC),
   the the left mz value (left.mzPC) and right mz value(right.mzPC) of
   the valley of the peak from the PCA when extracting the template, the
   smallest negative intensity for the cluster (min.int), the excess
   number of points to the right of the  cluster to ensure the fitted
   spectrum do not go out of the cluster region (excessPt).}
 \item{arrayInfo}{Same as arrayFinal except those peaks that have
   MSE>msetol are set not set to NA.}
 \item{templateInfo}{List consisting of matrices with the mz and
  'intensity' profiles of the template obtained from the PCA. The number
  of components corresponding to the number of clusters and the number
  of components within each cluster corresponds to the number of peaks.}
 \item{infoPA}{ Array consisting of the peak detected in the regions that are
  significant with the estimates of the peak's intensity and mz
  obtained via parametric approach by assuming a log-normal peak
  shape (see second reference). The rows correspond to the the number of peaks detected in the
  entire region, the columns the spectra and the 3rd dimension consist
  of the different information of the peak. The 1st entry is the
  estimated shift parameter s of the log-normal distribution (sEst),
  the estimated mu parameter of the log-normal distribution (muEst),
  the estimated sigma parameter of the log-normal distribution
  (sigmaEst), the estimated mixing probabilities (pEst), the area of
  the observed spectrum after making it non-negative (area.spect), the
  area of the the predicted spectrum (area.temp), the
  smallest negative intensity for the entire cluster to make it
  non-negative (min.min.int), indicator whether there is convergence in
  the EM-aglorithm and NA if EM-aglorithm was not used (converge), and
  the number of iteration used in the EM-algorithm (iter).}
}
\references{

  Tan CS, Ploner A, Quandt A, Lehtio J, Pernemalm M, Lewensohn R,
  Pawitan Y. 2006. Annotated regions of significance of SELDI-TOF-MS
  spectra for detecting protein biomarkers. \emph{Proteomics},
  \bold{6(23)}:6124--33

  Dijkstra M, Roelofsen H, Vonk RJ, Jansen RC. 2006. Peak quantification
  in surface-enhance laser desorption/ionization by using mixture
  models. \emph{Proteomics}, \bold{6(19)}:6124--33
  
    
}
%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
# R library folder
jj <- .libPaths()[1]

#Path to the example folder
pathToExample <- file.path(jj,"ProSpect","SpikeInData/")

#Read in the .csv files from Ciphergen
read.dat <- read.ProSpect(spectra.dir=pathToExample,
                          expt.name="An example data")

#Reduce and align the spectra
reduct.dat <- reduct.ProSpect(dat=read.dat[[1]],range.mz=c(5000,6000))

#Perform background correction on the reduced and aligned spectra
loess.dat <- BgCorrection.ProSpect(dat=reduct.dat)

#Perform one-way ANOVA on the windows
mlm.dat <- mlm.ProSpect(loess.dat,num.points=5)

#Smooth the MSE from the one-way ANOVA
modF.dat <- mod.F(mlm.dat,10)

#Compute the modified F-statistics (F*)
modFcorrFdr.dat <- modFcorr.fdr(modF.dat,name="median",proc="BH")

#Identify the significant windows
zoom.dat <- zoom.alpha(modFcorrFdr.dat,loess.dat,NULL,
                      corr.name="median",pType="chisq",
                      sign.type="fdr.BH",alpha.level=0.05)

#Quantify the peak in the significant regions
quantify.dat<-quantifyPeaks(loess.dat=loess.dat,zoom.dat=zoom.dat,
                            fullFolderName=pathToExample)

}
\author{
Tan Chuen Seng, Andreas Quandt, Alexander Ploner and Yudi
Pawitan
}
\keyword{ utilities}% at least one, from doc/KEYWORDS

