\name{zoom.alpha}

\alias{zoom.alpha}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{Identify the significant windows}
\description{
  The criterion to flag significant windows are specified before
  determining which windows are significant.
}
\usage{
zoom.alpha(cmod.F, prot.dat, clin.dat, corr.name = c("median", 
    "arithmetic", "tukey", "trimmed"), pType = c("f", "chisq", 
    "fchisq"), sign.type = c("pValue", "fdr.BH", "fdr.BY"), alpha.level = 0.05) 
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{cmod.F}{Object from the function modFcorr.fdr().}
  \item{prot.dat}{Objects from the functions reduct.ProSpect() or
    BgCorrection.ProSpect().}
  \item{clin.dat}{Clinical data. If not available, set it to NULL.}
  \item{corr.name}{String specifying the type of smoothing on F': "median",
    "arithmetic", "turkey" or "trimmed".}
  \item{pType}{The combination of distributions to calculate F' and
    F*\cr
    'chisq' = using Chisquare distribution for both\cr
    'fchisq' = using F and Chisquare distribution for F' and F*\cr
    'f' = using F distribution for both}
  \item{sign.type}{Specifies the criterion used to detect the significant peaks\cr
    'pValue'\cr
    'fdr.BY'\cr
    'fdr.BH'
  }
  \item{alpha.evel}{Specifies the sign level alpha}
}
%\details{
%  
%}
\value{
 Returns a list:
 \item{id}{ a vector indicating the row indices of either $cArithmetic.mF,
  or $cTrimmed.mF, $cTukey.mF, or $cMed.mF from $modFcorrFdr.dat,
  depending on the type of correction wanted.}
 \item{sign.select.F}{ subset of either $cArithmetic.mF, or
   $cTrimmed.mF, $cTukey.mF, or $cMed.mF from $modFcorrFdr.dat that are
   significant.}
 \item{mz}{ subset of mlm.dat$mz that are significant.}
 \item{smoothingMean.type}{ a string indicating the type of smoothing on
   the MSE of the F-statistics.}
 \item{corrF.type}{ a string indicating the type of smoothing on the F'.} 
 \item{num.run.Fprime}{ a vector indicating the number of neighbours
   used to calculate mean or median of F'. }
 \item{num.run.sMse}{a vector indicating the number of neighbours used
   to calculate mean or median.}
 \item{B.corr}{a vector of length 2: 1st entry is the percentage of
   neighbours and 2nd entry is the actual number of neighbours when
   taking the mean/median of F'. }
 \item{B.sMse}{a vector of length 2: 1st entry is the percentage of
   neighbours and 2nd entry is the actual number of neighbours.}
 \item{trimmedProportion.corr}{ a vector of length 2 where the 1st entry is the proportion to trim and 2nd entry is the actual number trimed off for F'.}
 \item{trimmedPropotion.sMse}{a vector of length 2: 1st entry is the
   proportion to trim and 2nd entry is the actual number trimed off.}
 \item{df1}{ the df of the numerator of the final modified F-statistics
   (F*).}
 \item{df2}{ the df of the denominator of the final modified
   F-statistics (F*). Provided the median is not used to smooth F'.}
 \item{expt}{a string describing the SELDI experiment.}
 \item{num.points}{a scalar that tells the number of paired
   observations (m/z,intensity) for each window's spectrum.}
 \item{range}{the row indices from read.ProSpect that correspond to the
   restricted data.}
 \item{pType}{ a string indicating the type of modified F-statistics
   used.}
 \item{sign.type}{ a string indicating the type of fdr used.} 
 \item{select.F}{ either $cArithmetic.mF, or $cTrimmed.mF, $cTukey.mF,
   or $cMed.mF from $modFcorrFdr.dat.} 
 \item{pValue}{ a string indicating the type of fdr used.}
 
}
\references{

  Tan CS, Ploner A, Quandt A, Lehtio J, Pawitan Y. 2006. Finding
  regions of significance in SELDI measurements for identifying protein
  biomarkers. \emph{Bioinformatics}, \bold{22(12)}:1515--23.
  
  Quandt A, Ploner A, Tan CS, Lehtio J, Pawitan Y. 2005. ProSpect: An
  R Package for Analyzing SELDI Measurements Identifying Protein
  Biomarkers. \emph{Lecture Notes in Computer Science}, \bold{3695}:140--150.
  
}
%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
# R library folder
jj <- .libPaths()[1]

#Path to the example folder
pathToExample <- file.path(jj,"ProSpect","SpikeInData/")

#Read in the .csv files from Ciphergen
read.dat <- read.ProSpect(spectra.dir=pathToExample,expt.name="An example
data")

#Reduce and align the spectra
reduct.dat <- reduct.ProSpect(dat=read.dat[[1]],range.mz=c(5000,6000))

#Perform background correction on the reduced and aligned spectra
loess.dat <- BgCorrection.ProSpect(dat=reduct.dat)

#Perform one-way ANOVA on the windows
mlm.dat <- mlm.ProSpect(loess.dat,num.points=5)

#Smooth the MSE from the one-way ANOVA
modF.dat <- mod.F(mlm.dat,10)

#Compute the modified F-statistics (F*)
modFcorrFdr.dat <- modFcorr.fdr(modF.dat,name="median",proc="BH")

#Identify the significant windows
zoom.dat<- zoom.alpha(modFcorrFdr.dat,loess.dat,NULL,
                      corr.name="median",pType="chisq",sign.type="fdr.BH",
                      alpha.level=0.05)
}
\author{
Tan Chuen Seng, Andreas Quandt, Alexander Ploner and Yudi
Pawitan
}
\keyword{ utilities}% at least one, from doc/KEYWORDS

